@@ -2,8 +2,12 @@
  *
  *
  * (C) Copyright IBM Corp 2003,2004
+ * (C) Copyright Nokia Siemens Networks 2010
+ * (C) Copyright Ulrich Kleber 2011
+ *
  * Authors:  
  *     Peter D.Phan pdphan@users.sourceforge.net
+ *     Ulrich Kleber <ulikleber@users.sourceforge.net>
  *  
  *     01/13/2004 pdphan  module created
  *			  Change clock for event log on IBM Blade Center E.
@@ -11,6 +15,9 @@
  *     10/12/2004 kouzmich  porting to HPI B.
  *			    check month, day, year, hours, minutes and seconds
  *			    for correctness
+ *     07/06/2010 ulikleber  New option -D to select domain
+ *     20/01/2011 ulikleber  Refactoring to use glib for option parsing and
+ *                           introduce common options for all clients
  */
 /*
 Redistribution and use in source and binary forms, with or without 
@@ -37,37 +44,27 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <getopt.h>
-#include <string.h>
-#include <time.h>
+#include "oh_clients.h"
 
-#include <SaHpi.h>
-#include <oh_utils.h>
-#include <oh_clients.h>
+#define OH_SVN_REV "$Revision: 7633 $"
 
-#define OH_SVN_REV "$Revision: 6571 $"
+static gchar *findate = NULL;
+static gchar *fintime = NULL;
+static oHpiCommonOptionsT copt;
 
-int fdebug = 0;
-int findate = 0;
-int fintime = 0;
-
-static void usage(char **argv)
+static GOptionEntry my_options[] =
 {
-	printf("Usage\n\t%s -d mm/dd/yyyy -t HH:MM:SS [-x]\n",argv[0]);
-	printf("\twhere -d date in mm/dd/yyyy format\n");
-	printf("\t      -t time of day in 24-hr format\n");
-	printf("\t      -x displays eXtra debug messages\n");
+  { "date",  'd', 0, G_OPTION_ARG_STRING, &findate,  "New date", "mm/dd/yyyy" },
+  { "time",  't', 0, G_OPTION_ARG_STRING, &fintime,  "New time of day in 24-hr format", "24:12:60" },
+  { NULL }
+};
+
+#define EXIT1 g_free(findate);g_free(fintime);return 1;
 
-	return;
-}
 
 int main(int argc, char **argv)
 {
-	int c, month, day, year;
-	char i_newdate[20];
-	char i_newtime[20];
+	int month, day, year;
 	int day_array[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 	struct tm  new_tm_time;
 	SaErrorT rv;
@@ -77,54 +74,51 @@ int main(int argc, char **argv)
 	SaHpiEntryIdT rptentryid;
 	SaHpiEntryIdT nextrptentryid;
 	SaHpiResourceIdT resourceid;
-	SaHpiEventLogEntryIdT entryid;
 	SaHpiTimeT oldtime;
 	SaHpiTimeT newtime;
 	SaHpiTimeT readbacktime;
 	SaHpiTextBufferT buffer;
+        GOptionContext *context;
 
-	oh_prog_version(argv[0], OH_SVN_REV);
-        
-	while ( (c = getopt( argc, argv,"d:t:x")) != EOF )
-	{
-		switch(c) {
-			case 'd': 
-				findate = 1;
-				strcpy(i_newdate, optarg);
-				break;
-			case 't': 
-				fintime = 1;
-				strcpy(i_newtime, optarg);
-				break;
-			case 'x':
-				fdebug = 1;
-				break;
-			default:
-				usage(argv);
-				exit(1);
-		}
+        /* Print version strings */
+	oh_prog_version(argv[0]);
+
+        /* Parsing options */
+        static char usetext[]="- Exercises Event Log clock APIs.\n  "
+                              OH_SVN_REV; 
+        OHC_PREPARE_REVISION(usetext);
+        context = g_option_context_new (usetext);
+        g_option_context_add_main_entries (context, my_options, NULL);
+
+        if (!ohc_option_parse(&argc, argv, 
+                context, &copt, 
+                OHC_ALL_OPTIONS 
+                    - OHC_ENTITY_PATH_OPTION   // not applicable
+                    - OHC_VERBOSE_OPTION )) {      // no verbose mode
+                g_option_context_free (context);
+		EXIT1;
 	}
-	
+        g_option_context_free (context);
+
 	if ( !findate || !fintime) {
-		usage(argv);
-		exit(1);
+		CRIT("Please enter date and time to be set, or try --help.");
+		EXIT1;
 	}
 
 	if (findate) {
-		if (fdebug) printf("New date to be set: %s\n",i_newdate);
-	        if (sscanf(i_newdate,"%2d/%2d/%4d", &month, &day, &year) < 8) {
-			printf("%s: Invalid date\n", argv[0]);
+		if (copt.debug) printf("New date to be set: %s\n",findate);
+	        if (sscanf(findate,"%2d/%2d/%4d", &month, &day, &year) != 3) {
+			CRIT("%s: Invalid date", argv[0]);
+			EXIT1;
 		}
 		/* check month, day and year for correctness */
 		if ((month < 1) || (month > 12)) {
-			printf("%s: Month out of range: (%d)\n", argv[0], month);
-			usage(argv);
-			exit(1);
+			CRIT("%s: Month out of range: (%d)", argv[0], month);
+			EXIT1;
 		};
 		if (year < 1900) {
-			printf("%s: Year out of range: (%d)\n", argv[0], year);
-			usage(argv);
-			exit(1);
+			CRIT("%s: Year out of range: (%d)", argv[0], year);
+			EXIT1;
 		};
 		month--;
 		if (month == 1) {
@@ -133,9 +127,8 @@ int main(int argc, char **argv)
 				day_array[1] = 29;
 		};
 		if ((day < 1) || (day > day_array[month])) {
-			printf("%s: Day out of range: (%d)\n", argv[0], day);
-			usage(argv);
-			exit(1);
+			CRIT("%s: Day out of range: (%d)", argv[0], day);
+			EXIT1;
 		};
 
 		new_tm_time.tm_mon = month;
@@ -144,50 +137,45 @@ int main(int argc, char **argv)
 	}
 
 	if (fintime) {
-		if (fdebug)  printf("New time to be set:  %s\n",i_newtime);
-	        if (sscanf(i_newtime,"%2d:%2d:%2d",
-                  &new_tm_time.tm_hour, &new_tm_time.tm_min, &new_tm_time.tm_sec) < 6) {
-			printf("%s: Invalid time\n", argv[0]);
+		if (copt.debug)  DBG("New time to be set:  %s",fintime);
+	        if (sscanf(fintime,"%2d:%2d:%2d",
+                  &new_tm_time.tm_hour, &new_tm_time.tm_min, &new_tm_time.tm_sec) != 3) {
+			CRIT("%s: Invalid time", argv[0]);
+			EXIT1;
 		}
 		/* check hours, minutes and seconds for correctness */
 		if ((new_tm_time.tm_hour < 0) || (new_tm_time.tm_hour > 24)) {
-			printf("%s: Hours out of range: (%d)\n", argv[0], new_tm_time.tm_hour);
-			usage(argv);
-			exit(1);
+			CRIT("%s: Hours out of range: (%d)", argv[0], new_tm_time.tm_hour);
+			EXIT1;
 		};
 		if ((new_tm_time.tm_min < 0) || (new_tm_time.tm_min > 60)) {
-			printf("%s: Minutes out of range: (%d)\n", argv[0], new_tm_time.tm_min);
-			usage(argv);
-			exit(1);
+			CRIT("%s: Minutes out of range: (%d)", argv[0], new_tm_time.tm_min);
+			EXIT1;
 		};
 		if ((new_tm_time.tm_sec < 0) || (new_tm_time.tm_sec > 60)) {
-			printf("%s: Seconds out of range: (%d)\n", argv[0], new_tm_time.tm_sec);
-			usage(argv);
-			exit(1);
+			CRIT("%s: Seconds out of range: (%d)", argv[0], new_tm_time.tm_sec);
+			EXIT1;
 		}
 	}
 
-	if (fdebug) printf("Values passed to mktime():\n\tmon %d\n\tday %d\n\tyear %d\n\tHH %d\n\tMM %d\n\tSS %d\n",
+	if (copt.debug) DBG("Values passed to mktime():\n\tmon %d\n\tday %d\n\tyear %d\n\tHH %d\n\tMM %d\n\tSS %d",
 			new_tm_time.tm_mon, new_tm_time.tm_mday, new_tm_time.tm_year,
 			new_tm_time.tm_hour, new_tm_time.tm_min, new_tm_time.tm_sec);
 
 	newtime = (SaHpiTimeT) mktime(&new_tm_time) * 1000000000;
-	if (fdebug) printf("New date and time in SaHpiTimeT %lli\n", (long long int)newtime);
 
-	rv = saHpiSessionOpen(SAHPI_UNSPECIFIED_DOMAIN_ID, &sessionid,NULL);
-	if (rv != SA_OK) {
-		if (rv == SA_ERR_HPI_ERROR) 
-			printf("saHpiSessionOpen: error %d, SpiLibd not running\n",rv);
-		else
-			printf("saHpiSessionOpen: %s\n", oh_lookup_error(rv));
-		exit(-1);
-	}
- 
+	if (copt.debug) 
+           DBG("New date and time in SaHpiTimeT %" PRId64 "\n", (int64_t)newtime);
+
+        rv = ohc_session_open_by_option ( &copt, &sessionid);
+	if (rv != SA_OK) { EXIT1; }
+
+	if (copt.debug) DBG("saHpiDiscover");
 	rv = saHpiDiscover(sessionid);
-	if (fdebug) printf("saHpiDiscover %s\n", oh_lookup_error(rv));
+	if (copt.debug) DBG("saHpiDiscover %s", oh_lookup_error(rv));
 	rv = saHpiDomainInfoGet(sessionid, &domainInfo);
-	if (fdebug) printf("saHpiDomainInfoGet %s\n", oh_lookup_error(rv));
-	printf("DomainInfo: RptUpdateCount = %d, RptUpdateTimestamp = %lx\n",
+	if (copt.debug) DBG("saHpiDomainInfoGet %s", oh_lookup_error(rv));
+	printf("DomainInfo: RptUpdateCount = %u, RptUpdateTimestamp = %lx\n",
 		domainInfo.RptUpdateCount, (unsigned long)domainInfo.RptUpdateTimestamp);
         
 	/* walk the RPT list */
@@ -195,10 +183,10 @@ int main(int argc, char **argv)
 	while ((rv == SA_OK) && (rptentryid != SAHPI_LAST_ENTRY))
 	{
                 rv = saHpiRptEntryGet(sessionid,rptentryid,&nextrptentryid,&rptentry);
-                if (fdebug) printf("saHpiRptEntryGet %s\n", oh_lookup_error(rv));
+                if (copt.debug) DBG("saHpiRptEntryGet %s", oh_lookup_error(rv));
                 if ((rv == SA_OK) && (rptentry.ResourceCapabilities & SAHPI_CAPABILITY_EVENT_LOG)) {
                         resourceid = rptentry.ResourceId;
-                        if (fdebug) printf("RPT %x capabilities = %x\n", resourceid,
+                        if (copt.debug) DBG("RPT %x capabilities = %x", resourceid,
                                            rptentry.ResourceCapabilities);
 			rv = saHpiEventLogTimeGet(sessionid, resourceid, &oldtime);
 			oh_decode_time(oldtime, &buffer);
@@ -207,19 +195,22 @@ int main(int argc, char **argv)
 		 	rv = saHpiEventLogTimeSet(sessionid, resourceid, newtime);
 			if (rv != SA_OK) 
 			{
-                		printf("saHpiEventLogTimeSet %s\n", oh_lookup_error(rv));
+                		CRIT("saHpiEventLogTimeSet returned %s", oh_lookup_error(rv));
 			}
 			rv = saHpiEventLogTimeGet(sessionid, resourceid, &readbacktime);
 			oh_decode_time(readbacktime, &buffer);
 			printf ("Read-Back-Check event log time: %s\n", buffer.Data);
 
                 }
-                entryid = SAHPI_OLDEST_ENTRY;
+                // entryid = SAHPI_OLDEST_ENTRY;
                 rptentryid = nextrptentryid;
 	} 
         
         rv = saHpiSessionClose(sessionid);
-        
+
+        g_free(findate);
+        g_free(fintime);
+   
         return(0);
 }
  
