@@ -1,6 +1,8 @@
 /*      -*- linux-c -*-
  *
  * Copyright (c) 2003 by Intel Corp.
+ * (C) Copyright Nokia Siemens Networks 2010
+ * (C) Copyright Ulrich Kleber 2011
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,24 +14,22 @@
  * Authors:
  *     Bill Barkely 
  *     Andy Cress <arcress@users.sourceforge.net>
+ *     Ulrich Kleber <ulikleber@users.sourceforge.net>
+ *
  * Changes:
  * 02/26/04 ARCress - added general search for any Fan sensor.
  * 03/17/04 ARCress - changed to Temp sensor (always has Lower Major)
  * 11/03/2004  kouzmich   porting to HPI B
  * 11/25/2004  kouzmich   changed as new threshold client (first release)
+ * 09/06/2010  ulikleber  New option -D to select domain
+ * 01/02/2011  ulikleber  Refactoring to use glib for option parsing and
+ *                        introduce common options for all clients
+ *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <getopt.h>
+#include "oh_clients.h"
 
-#include <SaHpi.h>
-#include <oh_utils.h>
-#include <oh_clients.h>
-
-#define OH_SVN_REV "$Revision: 6847 $"
+#define OH_SVN_REV "$Revision: 7663 $"
 
 #define READ_BUF_SIZE	1024
 
@@ -93,8 +93,7 @@ Commands_def_t	Coms[] = {
 Rpt_t	*Rpts;
 int	nrpts = 0;
 
-int	fdebug = 0;
-
+static oHpiCommonOptionsT copt;
 SaHpiSessionIdT		sessionid;
 
 static void *resize_array(void *Ar, int item_size, int *last_num, int add_num)
@@ -128,10 +127,10 @@ static void print_value(SaHpiSensorReadingT *item, char *mes)
 		return;
 	switch (item->Type) {
 		case SAHPI_SENSOR_READING_TYPE_INT64:
-			printf("%lld %s\n", item->Value.SensorInt64, mes);
+			printf("%" PRId64 " %s\n", (int64_t) item->Value.SensorInt64, mes);
 			return;
 		case SAHPI_SENSOR_READING_TYPE_UINT64:
-			printf("%llu %s\n", item->Value.SensorUint64, mes);
+			printf("%" PRIu64 " %s\n", (uint64_t) item->Value.SensorUint64, mes);
 			return;
 		case SAHPI_SENSOR_READING_TYPE_FLOAT64:
 			printf("%10.3f %s\n", item->Value.SensorFloat64, mes);
@@ -278,7 +277,7 @@ static void show_reading_value(Rpt_t *Rpt, Rdr_t *R)
 		R->Rdr.RdrTypeUnion.SensorRec.Num, &(R->reading), NULL);
 
 	if (rv != SA_OK) {
-		printf("ERROR: %s\n", oh_lookup_error(rv));
+		CRIT("ERROR: saHpiSensorReadingGet returned %s", oh_lookup_error(rv));
 		return;
 	};
 	k = R->Rdr.RdrTypeUnion.SensorRec.DataFormat.BaseUnits;
@@ -295,7 +294,7 @@ static void show_thresholds(Rpt_t *Rpt, Rdr_t *R)
 	rv = saHpiSensorThresholdsGet(sessionid, Rpt->Rpt.ResourceId,
 		R->Rdr.RdrTypeUnion.SensorRec.Num, &buf);
 	if (rv != SA_OK) {
-		printf("ERROR: %s\n", oh_lookup_error(rv));
+		CRIT("ERROR: saHpiSensorThresholdsGet returned %s", oh_lookup_error(rv));
 		return;
 	};
 	printf("  Thresholds:\n");
@@ -362,13 +361,14 @@ static int get_number(char *mes, int *res)
 
 	printf("%s", mes);
 	ret = fgets(buf, READ_BUF_SIZE, stdin);
-	return (sscanf(buf, "%d", res));
+	if (ret) return (sscanf(buf, "%d", res));
+        else return -1;
 }
 
 static void set_thres_value(SaHpiSensorReadingT *R, double val)
 {
 	if (R->IsSupported == 0) {
-		printf("ERROR: this threshold isn't supported\n");
+		CRIT("ERROR: this threshold isn't supported");
 		return;
 	};
 	R->Value.SensorFloat64 = val;
@@ -388,12 +388,12 @@ static void mod_sen(void)
 
 	i = get_number("RPT number: ", &rpt);
 	if (i != 1) {
-		printf("ERROR: no RPT number\n");
+		CRIT("ERROR: no RPT number");
 		return;
 	};
 	i = find_rpt_by_id(rpt);
 	if (i < 0) {
-		printf("ERROR: invalid RPT number\n");
+		CRIT("ERROR: invalid RPT number");
 		return;
 	};
 	rpt = i;
@@ -401,12 +401,12 @@ static void mod_sen(void)
 	show_sens_for_rpt(Rpt);
 	i = get_number("Sensor number: ", &num);
 	if (i != 1) {
-		printf("ERROR: no Sensor number\n");
+		CRIT("ERROR: no Sensor number");
 		return;
 	};
 	rdr = find_sensor_by_num(Rpt, num);
 	if (rdr < 0) {
-		printf("ERROR: invalid sensor number\n");
+		CRIT("ERROR: invalid sensor number");
 		return;
 	};
 	Rdr = Rpt->rdrs + rdr;
@@ -415,7 +415,7 @@ static void mod_sen(void)
 	rv = saHpiSensorThresholdsGet(sessionid, Rpt->Rpt.ResourceId,
 		Rdr->Rdr.RdrTypeUnion.SensorRec.Num, &thres);
 	if (rv != SA_OK) {
-		printf("ERROR: %s\n", oh_lookup_error(rv));
+		CRIT("ERROR: saHpiSensorThresholdsGet returned %s", oh_lookup_error(rv));
 		return;
 	};
 	printf("  Thresholds:\n");
@@ -428,7 +428,7 @@ static void mod_sen(void)
 	S = fgets(buf, READ_BUF_SIZE, stdin);
 	while (*S == ' ') S++;
 	if (strlen(S) < 2) {
-		printf("ERROR: invalid threshold type: %s\n", S);
+		CRIT("ERROR: invalid threshold type: %s", S);
 		return;
 	};
 
@@ -442,7 +442,7 @@ static void mod_sen(void)
 	if (strncmp(S, "nh", 2) == 0) type = NEG_HYST;
 
 	if (type == UNKNOWN_TYPE) {
-		printf("ERROR: unknown threshold type: %s\n", S);
+		CRIT("ERROR: unknown threshold type: %s", S);
 		return;
 	};
 	
@@ -450,7 +450,7 @@ static void mod_sen(void)
 	S = fgets(buf, READ_BUF_SIZE, stdin);
 	i = sscanf(buf, "%f", &f);
 	if (i == 0) {
-		printf("ERROR: no value\n");
+		CRIT("ERROR: no value");
 		return;
 	};
 	switch (type) {
@@ -479,7 +479,7 @@ static void mod_sen(void)
 			set_thres_value(&(thres.NegThdHysteresis), (double)f);
 			break;
 		default:
-			printf("ERROR: unknown threshold\n");
+			CRIT("ERROR: unknown threshold");
 	};
 	printf("\n  Nem threshold:\n");
 	ShowThres(&thres);
@@ -491,7 +491,7 @@ static void mod_sen(void)
 	rv = saHpiSensorThresholdsSet(sessionid, Rpt->Rpt.ResourceId,
 		Rdr->Rdr.RdrTypeUnion.SensorRec.Num, &thres);
 	if (rv != SA_OK) {
-		printf("ERROR: saHpiSensorThresholdsSet: %s\n", oh_lookup_error(rv));
+		CRIT("ERROR: saHpiSensorThresholdsSet returned %s\n", oh_lookup_error(rv));
 		return;
 	};
 	Rdr->modify = 1;
@@ -500,8 +500,8 @@ static void mod_sen(void)
 			NULL, NULL, NULL);
 		if (rv == SA_OK)
 			break;
-		if (fdebug) printf("sleep before saHpiEventGet\n");
-		sleep(1);
+		if (copt.debug) DBG("sleep before saHpiEventGet");
+		g_usleep(G_USEC_PER_SEC);
 	};
 	saHpiSensorThresholdsGet(sessionid, Rpt->Rpt.ResourceId,
 		Rdr->Rdr.RdrTypeUnion.SensorRec.Num, &thres);
@@ -603,6 +603,9 @@ static int parse_command(char *Str)
 	char		*S, *S1;
 	Com_enum_t	com;
 
+	// EOF
+	if (!Str) return(-1);
+
 	S = Str;
 	while (*S != 0) {
 		if ((*S == '\t') || (*S == '\n')) *S = ' ';
@@ -638,38 +641,42 @@ static int parse_command(char *Str)
 	
 int main(int argc, char **argv)
 {
-	int		c, i;
+	int		i;
 	SaErrorT	rv;
 	char		buf[READ_BUF_SIZE];
 	char		*S;
+        GOptionContext  *context;
 
-	oh_prog_version(argv[0], OH_SVN_REV);
-	while ( (c = getopt( argc, argv,"x?")) != EOF )
-		switch(c)  {
-			case 'x':
-				fdebug = 1;
-				break;
-			default:
-				printf("Usage: %s [-x]\n", argv[0]);
-				printf("   -x  Display debug messages\n");
-				return(1);
-		}
+        /* Print version strings */
+	oh_prog_version(argv[0]);
 
-	rv = saHpiSessionOpen(SAHPI_UNSPECIFIED_DOMAIN_ID, &sessionid, NULL);
+        /* Parsing options */
+        static char usetext[]="- Display sensors and sensor info\n  "
+                              OH_SVN_REV; 
+        OHC_PREPARE_REVISION(usetext);
+        context = g_option_context_new (usetext);
 
-	if (rv != SA_OK) {
-		printf("saHpiSessionOpen: %s\n", oh_lookup_error(rv));
-		return(-1);
+        if (!ohc_option_parse(&argc, argv, 
+                context, &copt, 
+                OHC_ALL_OPTIONS 
+                    - OHC_ENTITY_PATH_OPTION //TODO: Feature 880127
+                    - OHC_VERBOSE_OPTION )) {    // no verbose mode implemented
+                g_option_context_free (context);
+		return 1;
 	}
+        g_option_context_free (context);
  
+        rv = ohc_session_open_by_option ( &copt, &sessionid);
+	if (rv != SA_OK) return rv;
+	
 	rv = saHpiDiscover(sessionid);
 
-	if (fdebug) printf("saHpiDiscover: %s\n", oh_lookup_error(rv));
+	if (copt.debug) DBG("saHpiDiscover: %s", oh_lookup_error(rv));
 
 	rv = saHpiSubscribe(sessionid);
 	if (rv != SA_OK) {
-		printf( "saHpiSubscribe error %d\n",rv);
-		return(-1);
+		CRIT( "saHpiSubscribe error %d",rv);
+		return rv;
 	}	
 	
 	/* make the RPT list */
