@@ -25,10 +25,11 @@
 #include <string.h>
 #include <time.h>
 #include <ctype.h>
-#include <unistd.h>
+#include <glib.h>
 #include <config.h>
 #include <oHpi.h>
-#include <hpi_ui.h>
+
+#include "hpi_ui.h"
 #include "hpi_cmd.h"
 
 #define SEV_BUF_SIZE    32
@@ -66,7 +67,7 @@ void help(int as)
                                 (cmd->type != block_type) &&
                                 (cmd->type != UNDEF_COM))
                                 continue;
-                        printf("%-20s", cmd->cmd);
+                        printf("%-19s", cmd->cmd);
                         if ((++width % 4) == 0)
                                 printf("\n");
                 }
@@ -172,6 +173,16 @@ static ret_code_t event(void)
         return HPI_SHELL_OK;
 }
 
+#ifdef _WIN32
+static int setenv(const char * var, const char * val, int dummy)
+{
+    static const size_t BUFSIZE = 1024;
+    char buf[BUFSIZE];
+    snprintf(buf, BUFSIZE, "%s=%s", var, val);
+    return _putenv(buf);
+}
+#endif
+
 static ret_code_t debugset(void)
 {
         char            *val;
@@ -222,6 +233,9 @@ static ret_code_t newdomain(void)
 {
     int i;
     char buf[SAHPI_MAX_TEXT_BUFFER_LENGTH + 1];
+    char * hostbegin = NULL;
+    char * hostend = NULL;
+    char * portbegin = NULL;
     char * p;
 
     SaHpiUint16T port;
@@ -235,28 +249,39 @@ static ret_code_t newdomain(void)
     if (i != 0) {
         return HPI_SHELL_PARM_ERROR;
     }
-    if (buf[0] == ':') {
-        return HPI_SHELL_PARM_ERROR;
-    }
-
-    p = strchr(buf, ':');
-    if (p != 0) {
-        *p = '\0';
-        ++p;
-        if (isdigit(*p) == 0) {
+    if (buf[0] == '[') {
+        hostbegin = &buf[1];
+        hostend = strchr(hostbegin, ']');
+        if (hostend == NULL) {
             return HPI_SHELL_PARM_ERROR;
         }
-        port = (SaHpiUint16T)atoi(p);
     } else {
-        port = OPENHPI_DEFAULT_DAEMON_PORT;
+        hostbegin = &buf[0];
+        hostend = strchr(hostbegin, ':');
+    }
+    if (hostend) {
+        portbegin = strchr(hostend, ':' );
+        if (portbegin) {
+            ++portbegin;
+        }
+        *hostend = '\0';
     }
 
     host.DataType = SAHPI_TL_TYPE_TEXT;
     host.Language = SAHPI_LANG_ENGLISH;
-    host.DataLength = strlen(buf);
-    memcpy(&host.Data[0], &buf[0], host.DataLength );
+    host.DataLength = strlen(hostbegin);
+    memcpy(&host.Data[0], hostbegin, host.DataLength );
     host.Data[host.DataLength] = '\0';
 
+    if (portbegin) {
+        if (isdigit(*portbegin) == 0) {
+            return HPI_SHELL_PARM_ERROR;
+        }
+        port = (SaHpiUint16T)atoi(portbegin);
+    } else {
+        port = OPENHPI_DEFAULT_DAEMON_PORT;
+    }
+
     oh_init_ep(&entity_root);
 
     i = get_string_param("Domain Id: ", buf, SAHPI_MAX_TEXT_BUFFER_LENGTH);
@@ -279,7 +304,7 @@ static ret_code_t newdomain(void)
         }
     }
 
-    printf("Created new domain %u (%s:%u)\n",
+    printf("Created new domain %u (host %s port %u)\n",
            did,
            (const char*)(&host.Data[0]),
            port);
@@ -969,8 +994,6 @@ static ret_code_t wdt_get(void)
                         str = "POWER_DOWN"; break;
                 case SAHPI_WA_POWER_CYCLE:
                         str = "POWER_CYCLE"; break;
-                //case SAHPI_WAE_TIMER_INT:           
-                //        str = "TIMER_INT"; break;
                 default: str = "Unknown"; break;
         };
         printf("  Action: %s", str);
@@ -1069,22 +1092,20 @@ static ret_code_t wdt_set(void)
         };
 
         i = get_string_param(
-                "TimerAction(no|reset|pwr_down|pwr_cycle|int): ",
+                "TimerAction(no|reset|pwr_down|pwr_cycle): ",
                 tmp, 255);
         if (i != 0) {
                 printf("Invalid TimerAction value: %s\n", tmp);
                 return(HPI_SHELL_PARM_ERROR);
         };
         if (strcmp(tmp, "no") == 0)
-                watchdog.TimerAction = SAHPI_WAE_NO_ACTION;
+                watchdog.TimerAction = SAHPI_WA_NO_ACTION;
         else if (strcmp(tmp, "reset") == 0)
-                watchdog.TimerAction = SAHPI_WAE_RESET;
+                watchdog.TimerAction = SAHPI_WA_RESET;
         else if (strcmp(tmp, "pwr_down") == 0)
-                watchdog.TimerAction = SAHPI_WAE_POWER_DOWN;
+                watchdog.TimerAction = SAHPI_WA_POWER_DOWN;
         else if (strcmp(tmp, "pwr_cycle") == 0)
-                watchdog.TimerAction = SAHPI_WAE_POWER_CYCLE;
-        else if (strcmp(tmp, "int") == 0)
-                watchdog.TimerAction = SAHPI_WAE_TIMER_INT;
+                watchdog.TimerAction = SAHPI_WA_POWER_CYCLE;
         else {
                 printf("Invalid TimerAction value: %s\n", tmp);
                 return(HPI_SHELL_PARM_ERROR);
@@ -1210,8 +1231,12 @@ static ret_code_t reopen_session(void)
         };
         do {
            rv = saHpiSessionClose(Domain->sessionId);
-           sleep( 1 );
-        } while ( fflag == 0 && rv != SA_OK && rv != SA_ERR_HPI_NO_RESPONSE );
+           g_usleep(G_USEC_PER_SEC);
+        } while ( ( fflag == 0 ) &&
+                  ( rv != SA_OK ) &&
+                  ( rv != SA_ERR_HPI_NO_RESPONSE ) &&
+                  ( rv != SA_ERR_HPI_INVALID_SESSION )
+        );
         if (rv != SA_OK) {
                 printf("saHpiSessionClose error %s\n", oh_lookup_error(rv));
         }
@@ -1336,10 +1361,10 @@ static ret_code_t entity_resources(void)
 
 static ret_code_t domain_info(void)
 {
-        SaHpiDomainInfoT        info;
-        SaHpiTextBufferT        *buf;
-        SaErrorT                rv;
-        char                    date[30];
+        SaHpiDomainInfoT info;
+        SaHpiTextBufferT *buf;
+        SaHpiTextBufferT tb;
+        SaErrorT         rv;
 
         rv = saHpiDomainInfoGet(Domain->sessionId, &info);
         if (rv != SA_OK) {
@@ -1353,15 +1378,15 @@ static ret_code_t domain_info(void)
         buf = &(info.DomainTag);
         print_text_buffer_text("    Tag: ", buf, NULL, ui_print);
         printf("\n");
-        time2str(info.DrtUpdateTimestamp, date, 30);
-        printf("    DRT update count: %d   DRT Timestamp : %s\n",
-                info.DrtUpdateCount, date);
-        time2str(info.RptUpdateTimestamp, date, 30);
-        printf("    RPT update count: %d   RPT Timestamp : %s\n",
-                info.RptUpdateCount, date);
-        time2str(info.DatUpdateTimestamp, date, 30);
-        printf("    DAT update count: %d   DAT Timestamp : %s\n",
-                info.DatUpdateCount, date);
+        printf("    DRT update count: %d", info.DrtUpdateCount);
+        oh_decode_time(info.DrtUpdateTimestamp, &tb);
+        print_text_buffer_text("   DRT Timestamp : ", &tb, "\n", ui_print);
+        printf("    RPT update count: %d", info.RptUpdateCount);
+        oh_decode_time(info.RptUpdateTimestamp, &tb);
+        print_text_buffer_text("   RPT Timestamp : ", &tb, "\n", ui_print);
+        printf("    DAT update count: %d", info.DatUpdateCount);
+        oh_decode_time(info.DatUpdateTimestamp, &tb);
+        print_text_buffer_text("   DAT Timestamp : ", &tb, "\n", ui_print);
         printf("        ActiveAlarms: %d   CriticalAlarms: %d   Major: %d "
                 "Minor: %d   Limit: %d\n",
                 info.ActiveAlarms, info.CriticalAlarms, info.MajorAlarms,
