@@ -15,10 +15,13 @@
  *
  */
 
+#include <inttypes.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <hpi_ui.h>
+
+#include "hpi_ui.h"
 #include "hpi_cmd.h"
 
 typedef struct {
@@ -107,7 +110,7 @@ static ret_code_t show_test_info( SaHpiSessionIdT sessionId,
         printf( "\n" );
     }
 
-    printf( "    Expected Run Duration: %lld nsec\n", test.ExpectedRunDuration );
+    printf( "    Expected Run Duration: %" PRId64 " nsec\n", (int64_t)test.ExpectedRunDuration );
 
     oh_decode_dimitestcapabilities( test.TestCapabilities, &tmpbuf );
     printf( "    Test capabilities: %s\n", tmpbuf.Data );
@@ -122,7 +125,8 @@ static ret_code_t show_test_info( SaHpiSessionIdT sessionId,
         }
         printf( "        Parameter %d:\n", i );
         printf( "            Name: \"%s\"\n", param->ParamName );
-        printf( "            Info: %s\n", param->ParamInfo.Data );
+        print_text_buffer_text( "            Info: ", &param->ParamInfo, NULL, ui_print );
+        printf( "\n" );
         switch ( param->ParamType ) {
             case SAHPI_DIMITEST_PARAM_TYPE_BOOLEAN:
                 printf( "            Type: boolean\n" );
@@ -143,7 +147,11 @@ static ret_code_t show_test_info( SaHpiSessionIdT sessionId,
                 break;
             case SAHPI_DIMITEST_PARAM_TYPE_TEXT:
                 printf( "            Type: text\n" );
-                printf( "            Default value: %s\n", param->DefaultParam.paramtext.Data );
+                print_text_buffer_text( "            Default value: ",
+                                        &param->DefaultParam.paramtext,
+                                        NULL,
+                                        ui_print );
+                printf( "\n" );
                 break;
             default:
                 printf( "            Type: unknown\n" );
@@ -217,7 +225,8 @@ static ret_code_t start_test( SaHpiSessionIdT sessionId,
         }
         printf( "Parameter %d:\n", i );
         printf( "    Name: \"%s\"\n", def->ParamName );
-        printf( "    Info: %s\n", def->ParamInfo.Data );
+        print_text_buffer_text( "    Info: ", &def->ParamInfo, NULL, ui_print );
+        printf( "\n" );
         memcpy( param->ParamName, def->ParamName, SAHPI_DIMITEST_PARAM_NAME_LEN );
         param->ParamType = def->ParamType;
         switch ( def->ParamType ) {
@@ -270,7 +279,11 @@ static ret_code_t start_test( SaHpiSessionIdT sessionId,
                 break;
             case SAHPI_DIMITEST_PARAM_TYPE_TEXT:
                 printf( "        Type: text\n" );
-                printf( "        Default value: %s\n", def->DefaultParam.paramtext.Data );
+                print_text_buffer_text( "        Default value: ",
+                                        &def->DefaultParam.paramtext,
+                                        NULL,
+                                        ui_print );
+                printf( "\n" );
                 cc = get_string_param( "Text Value ==> ", buf, sizeof(buf) );
                 if ( cc == 0 ) {
                     param->Value.paramtext.DataType = SAHPI_TL_TYPE_TEXT;
@@ -350,8 +363,7 @@ static ret_code_t show_test_results( SaHpiSessionIdT sessionId,
 {
     SaErrorT rv;
     SaHpiDimiTestResultsT results;
-    char date[30];
-    
+    SaHpiTextBufferT tb;
 
     rv = saHpiDimiTestResultsGet( sessionId, rptid, diminum, testnum, &results );
     if ( rv != SA_OK ) {
@@ -361,9 +373,9 @@ static ret_code_t show_test_results( SaHpiSessionIdT sessionId,
 
     printf( "Test number: %d\n", (int)testnum );
 
-    time2str( results.ResultTimeStamp, date, sizeof(date) );
-    printf( "    Result timestamp: %s\n", date );
-    printf( "    Run duration: %lld nsec\n", results.RunDuration );
+    oh_decode_time( results.ResultTimeStamp, &tb );
+    print_text_buffer_text("    Result timestamp: ", &tb, "\n", ui_print);
+    printf( "    Run duration: %" PRId64 " nsec\n", (int64_t)results.RunDuration );
     printf( "    Last run status: %s\n", oh_lookup_dimitestrunstatus( results.LastRunStatus ) );
 
     printf( "    Test error code: " );
@@ -382,10 +394,18 @@ static ret_code_t show_test_results( SaHpiSessionIdT sessionId,
     }
     printf( "\n" );
 
-    printf( "    Test result %s: \"%s\"\n",
-            results.TestResultStringIsURI == SAHPI_TRUE ? "URI" : "string",
-            results.TestResultString.Data
-          );
+    if ( results.TestResultStringIsURI == SAHPI_FALSE ) {
+        print_text_buffer_text( "    Test result string: ",
+                                &results.TestResultString,
+                                NULL,
+                                ui_print );
+    } else {
+        print_text_buffer_text( "    Test result URI: ",
+                                &results.TestResultString,
+                                NULL,
+                                ui_print );
+    }
+    printf( "\n" );
 
     return HPI_SHELL_OK;
 }
