@@ -32,6 +32,7 @@
 #include <oh_error.h>
 #include <oh_rpc_params.h>
 #include <strmsock.h>
+#include <sahpi_wrappers.h>
 
 
 /*--------------------------------------------------------------------*/
@@ -49,38 +50,40 @@ static SaErrorT process_msg(cHpiMarshal * hm,
 /*--------------------------------------------------------------------*/
 /* Local Definitions                                                  */
 /*--------------------------------------------------------------------*/
-
+#if GLIB_CHECK_VERSION (2, 32, 0)
+static GRecMutex lock;
+#else
 static GStaticRecMutex lock = G_STATIC_REC_MUTEX_INIT;
+#endif
 static volatile bool stop = false;
 
 static GList * sockets = 0; 
 
-
 /*--------------------------------------------------------------------*/
 /* Socket List                                                        */
 /*--------------------------------------------------------------------*/
 static void add_socket_to_list( cStreamSock * sock )
 {
-    g_static_rec_mutex_lock(&lock);
+    wrap_g_static_rec_mutex_lock(&lock);
     sockets = g_list_prepend( sockets, sock );
-    g_static_rec_mutex_unlock(&lock);
+    wrap_g_static_rec_mutex_unlock(&lock);
 }
 
 static void remove_socket_from_list( const cStreamSock * sock )
 {
-    g_static_rec_mutex_lock(&lock);
+    wrap_g_static_rec_mutex_lock(&lock);
     sockets = g_list_remove( sockets, sock );
-    g_static_rec_mutex_unlock(&lock);
+    wrap_g_static_rec_mutex_unlock(&lock);
 }
 
 static void close_sockets_in_list( void )
 {
-    g_static_rec_mutex_lock(&lock);
+    wrap_g_static_rec_mutex_lock(&lock);
     for ( GList * iter = sockets; iter != 0; iter = g_list_next( iter ) ) {
         cStreamSock * sock = reinterpret_cast<cStreamSock *>(iter->data);
         sock->Close();
     }
-    g_static_rec_mutex_unlock(&lock);
+    wrap_g_static_rec_mutex_unlock(&lock);
 }
 
 
@@ -101,6 +104,40 @@ static void dehash_handler_cfg(gpointer key, gpointer value, gpointer data)
     ++hc->NumberOfParams;
 }
 
+/*--------------------------------------------------------------------*/
+/* Function to log connection address                                 */
+/*--------------------------------------------------------------------*/
+static void LogIp( const cStreamSock * sock )
+{
+    sockaddr_storage storage;
+    bool rc = sock->GetPeerAddress( storage );
+    if ( !rc ) {
+        WARN( "Cannot determine connection address!" );
+        return;
+    }
+
+    if ( storage.ss_family == AF_INET ) {
+        const struct sockaddr_in * sa4
+            = reinterpret_cast<const struct sockaddr_in *>(&storage);
+        const uint8_t * ip = reinterpret_cast<const uint8_t *>( &sa4->sin_addr );
+        INFO( "Got connection from %u.%u.%u.%u port %u",
+              ip[0], ip[1], ip[2], ip[3],
+              g_ntohs( sa4->sin_port ) );
+    } else if ( storage.ss_family == AF_INET6 ) {
+        const struct sockaddr_in6 * sa6
+            = reinterpret_cast<const struct sockaddr_in6 *>(&storage);
+        const uint8_t * ip = reinterpret_cast<const uint8_t *>( &sa6->sin6_addr );
+        INFO( "HPI: Got connection from"
+              " %02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x"
+              " port %u",
+              ip[0], ip[1], ip[2], ip[3], ip[4], ip[5], ip[6], ip[7],
+              ip[8], ip[9], ip[10], ip[11], ip[12], ip[13], ip[14], ip[15],
+              g_ntohs( sa6->sin6_port ) );
+    } else {
+        WARN( "Unsupported socket address family!" );
+    }
+}
+
 
 /*--------------------------------------------------------------------*/
 /* HPI Server Interface                                               */
@@ -124,16 +161,35 @@ bool oh_server_run( int ipvflags,
     GThreadPool *pool;
     pool = g_thread_pool_new(service_thread, 0, max_threads, FALSE, 0);
 
+    cStreamSock::eWaitCc wc;
     // wait for a connection and then service the connection
     while (!stop) {
+        if ( ( wc = ssock->Wait() ) == cStreamSock::eWaitError ) {
+            if (stop) {
+                break;
+            }
+            g_usleep( 1000000 ); // in case the problem is persistent
+            CRIT( "Waiting on server socket failed" );
+            continue;
+        }
+
+        if ( wc == cStreamSock::eWaitTimeout ) {
+            continue;
+        }
+
         cStreamSock * sock = ssock->Accept();
+
+        if (!sock) {
+            CRIT("Error accepting server socket.");
+            g_usleep( 1000000 ); // in case the problem is persistent
+            continue;
+        }
+
         if (stop) {
             break;
         }
-        if (!sock) {
-            CRIT("Error accepting server socket.");
-            break;
-        }
+
+        LogIp( sock );
         add_socket_to_list( sock );
         DBG("### Spawning thread to handle connection. ###");
         g_thread_pool_push(pool, (gpointer)sock, 0);
@@ -189,7 +245,10 @@ static void service_thread(gpointer sock_ptr, gpointer /* user_data */)
             break;
         }
         if (!rc) {
-            CRIT("%p Error or Timeout while reading socket.", thrdid);
+            // The following error message need not be there as the
+            // ReadMsg captures the error when it returns false and
+            // one of the false return is not a real error
+            // CRIT("%p Error or Timeout while reading socket.", thrdid);
             break;
         } else if (type != eMhMsg) {
             CRIT("%p Unsupported message type. Discarding.", thrdid);
@@ -198,14 +257,18 @@ static void service_thread(gpointer sock_ptr, gpointer /* user_data */)
             cHpiMarshal *hm = HpiMarshalFind(id);
             SaErrorT process_rv;
             SaHpiSessionIdT changed_sid = 0;
-            process_rv = process_msg(hm, rq_byte_order, data, data_len, changed_sid);
+            if ( hm ) {
+                process_rv = process_msg(hm, rq_byte_order, data, data_len, changed_sid);
+            } else {
+                process_rv = SA_ERR_HPI_UNSUPPORTED_API;
+            }
             if (process_rv != SA_OK) {
-                int mr = HpiMarshalReply0(hm, data, &process_rv);
-                if (mr < 0) {
-                    CRIT("%p Marshal failed.", thrdid);
+                int cc = HpiMarshalReply0(hm, data, &process_rv);
+                if (cc < 0) {
+                    CRIT("%p Marshal failed, cc = %d", thrdid, cc);
                     break;
                 }
-                data_len = (uint32_t)mr;
+                data_len = (uint32_t)cc;
             }
             rc = sock->WriteMsg(eMhMsg, id, data, data_len);
             if (stop) {
@@ -246,19 +309,19 @@ static void service_thread(gpointer sock_ptr, gpointer /* user_data */)
 
 #define DEMARSHAL_RQ(rq_byte_order, hm, data, iparams) \
 { \
-    int mr = HpiDemarshalRequest(rq_byte_order, hm, data, iparams.array); \
-    if (mr < 0) { \
+    int cc = HpiDemarshalRequest(rq_byte_order, hm, data, iparams.array); \
+    if (cc < 0) { \
         return SA_ERR_HPI_INVALID_PARAMS; \
     } \
 }
 
 #define MARSHAL_RP(hm, data, data_len, oparams) \
 { \
-    int mr = HpiMarshalReply(hm, data, oparams.const_array); \
-    if (mr < 0) { \
+    int cc = HpiMarshalReply(hm, data, oparams.const_array); \
+    if (cc < 0) { \
         return SA_ERR_HPI_INTERNAL_ERROR; \
     } \
-    data_len = (uint32_t)mr; \
+    data_len = (uint32_t)cc; \
 }
 
 
@@ -301,6 +364,7 @@ static SaErrorT process_msg(cHpiMarshal * hm,
             RpcParams iparams(&did);
             DEMARSHAL_RQ(rq_byte_order, hm, data, iparams);
 
+            DBG("OpenHPID calling the saHpiSessionOpen");
             rv = saHpiSessionOpen(OH_DEFAULT_DOMAIN_ID, &sid, security);
             if (rv == SA_OK) {
                 changed_sid = sid;
@@ -1956,7 +2020,6 @@ static SaErrorT process_msg(cHpiMarshal * hm,
 
             RpcParams oparams(&rv, &info, &cfg);
             MARSHAL_RP(hm, data, data_len, oparams);
-// TODO memory leak
             // cleanup
             g_hash_table_destroy(cfg_tbl);
         }
