@@ -44,7 +44,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <bits/local_lim.h>
 
 #include <SaHpi.h>
 #include <oh_utils.h>
@@ -53,6 +52,7 @@
 #include <oh_ssl.h>
 
 #include "ilo2_ribcl_cmnds.h"
+#include "sahpi_wrappers.h"
 
 /* The macro ILO2_RIBCL_SIMULATE_iLO2_RESPONSE is used to conditionally
  * compile code which reads a simulated iLO2 response from a local
@@ -108,11 +108,15 @@
 #define ILO2_RIBCL_POWER_ON	1
 #define ILO2_RIBCL_RESET_SUCCESS	1	
 #define ILO2_RIBCL_RESET_FAILED	0	
-/*
- *Moving this definition to this file
- *from ilo2_ribcl_discover.h file
+
+/* The size used for the temporary buffer to contain the response
+ * of the IR_CMD_GET_SERVER_DATA command. The current return size for iLO2 is
+ * a little over 24K and a little over 45K for iLO3, so we use 64K to give 
+ * us some margin for the future.
  */
-#define ILO2_RIBCL_DISCOVER_RESP_MAX 1024*48
+
+#define ILO2_RIBCL_DISCOVER_RESP_MAX 1024*256
+
 /*
  * For a oh_set_power_state() call with a state parameter of
  * SAHPI_POWER_CYCLE, we must wait until the server actually powers off
@@ -140,6 +144,7 @@
 #define ILO	1
 #define ILO2	2
 #define ILO3	3
+#define ILO4	4
 #define ILO2_RIBCL_CMD_MAX_LEN 5
 /******************************************************************************
  * The following data structures and macros are used for our implementation
@@ -247,6 +252,8 @@ struct ilo2_ribcl_sensinfo {
 	SaHpiEventStateT  sens_assertmask;
 	SaHpiEventStateT  sens_deassertmask;
 	int		  sens_value;
+	SaHpiSensorThresholdsT threshold;
+	char *status;
 };
 
 /* Structure used by ilo2_ribcl_get_sensor_rdr_data() to return a pointer to
@@ -255,6 +262,7 @@ struct ilo2_ribcl_sens_allinfo {
 	SaHpiRptEntryT *rpt;
 	SaHpiRdrT *rdr;
 	struct ilo2_ribcl_sensinfo *sens_dat;
+        struct ir_tsdata *ts_data;
 };
 
 /* These are the state for software state machines that process the iLo2
@@ -345,6 +353,11 @@ typedef struct ir_tsdata{
 	char *status;
 	char *reading;
 	char *readingunits;
+        SaHpiResourceIdT rid;
+	char *cautionvalue;
+	char *cautionunit;
+	char *criticalvalue;
+	char *criticalunit;
 } ir_tsdata_t;
 
 /* Firmware Revision Information */
@@ -359,9 +372,16 @@ typedef struct ir_fwdata{
 #define ILO2_RIBCL_DISCOVER_FAN_MAX 16
 #define ILO2_RIBCL_DISCOVER_PSU_MAX 8
 #define ILO2_RIBCL_DISCOVER_VRM_MAX 8
-#define ILO2_RIBCL_DISCOVER_TS_MAX  48
+#define ILO2_RIBCL_DISCOVER_TS_MAX  120
 
 #define ILO2_RIBCL_CHASSIS_INDEX    -1;	/* Index is not aplicable to chassis */
+
+typedef struct iLO_thread_info {
+        GThread *hthread;
+        GCond   *iLO_cond;
+        GMutex  *iLO_mutex;
+        struct oh_handler_state *oh_handler;
+} iLO_info_t;
  
 typedef struct ilo2_ribcl_DiscoveryData {
 	char *product_name;
@@ -382,10 +402,15 @@ typedef struct ilo2_ribcl_handler {
 	char *entity_root;
 	int first_discovery_done;
 	int ilo_type;
-	char ir_hostname[HOST_NAME_MAX];
+	char ir_hostname[ILO2_HOST_NAME_MAX_LEN];
 	/* Storehouse for data obtained during discovery */
 	ilo2_ribcl_DiscoveryData_t DiscoveryData;
 
+	iLO_info_t *ilo_thread_data;
+	SaHpiBoolT need_rediscovery;
+	SaHpiBoolT discovery_complete;
+	SaHpiTimeT iml_log_time;
+
 	/* RIBCL data */
 	char *user_name;
 	char *password;
@@ -452,6 +477,7 @@ typedef struct ilo2_ribcl_resource_info {
 */
 #define ILO2_RIBCL_UID_OFF	0
 #define ILO2_RIBCL_UID_ON	1
+#define ILO2_RIBCL_UID_FLASHING	2
 #define ILO2_RIBCL_UID_SET_SUCCESS	1
 #define ILO2_RIBCL_UID_SET_FAILED	0
 
@@ -482,6 +508,7 @@ typedef struct ilo2_ribcl_resource_info {
    server. The values are
 	Yes = Enable automatic power on with a minimum delay.
 	No = Disable automatic power on.
+	Restore = Restore Previous state.
 	15 = Enable automatic power on with 15 seconds delay.
 	30 = Enable automatic power on with 30 seconds delay.
 	45 = Enable automatic power on with 45 seconds delay.
@@ -492,6 +519,8 @@ typedef struct ilo2_ribcl_resource_info {
 #define ILO2_RIBCL_AUTO_POWER_ENABLED 1
 #define ILO2_RIBCL_AUTO_POWER_DISABLED 2
 #define ILO2_RIBCL_AUTO_POWER_DELAY_RANDOM 3
+#define ILO2_RIBCL_AUTO_POWER_RESTORE 4
+#define ILO2_RIBCL_AUTO_POWER_OFF 5 
 #define ILO2_RIBCL_AUTO_POWER_DELAY_15 15
 #define ILO2_RIBCL_AUTO_POWER_DELAY_30 30
 #define ILO2_RIBCL_AUTO_POWER_DELAY_45 45
@@ -595,5 +624,7 @@ extern SaErrorT ilo2_ribcl_get_sensor_event_masks(void *, SaHpiResourceIdT,
 extern SaErrorT ilo2_ribcl_set_sensor_event_masks(void *, SaHpiResourceIdT,
 				SaHpiSensorNumT, SaHpiSensorEventMaskActionT,
 				SaHpiEventStateT, SaHpiEventStateT);
+extern SaErrorT ilo2_ribcl_get_sensor_thresholds(void *, SaHpiResourceIdT,
+                                SaHpiSensorNumT, SaHpiSensorThresholdsT *);
 
 #endif /* _INC_ILO2_RIBCL_H_ */
