@@ -43,6 +43,7 @@
 #include <sys/stat.h>   /* For test routine ilo2_ribcl_getfile() */
 #include <fcntl.h>      /* For test routine ilo2_ribcl_getfile() */
 #endif /* ILO2_RIBCL_SIMULATE_iLO2_RESPONSE */
+#include "sahpi_wrappers.h"
 
 /* Foreward decls: */
 
@@ -62,6 +63,8 @@ static SaErrorT ilo2_ribcl_discover_fans( struct oh_handler_state *,
 static SaErrorT ilo2_ribcl_discovered_fru( struct oh_handler_state *,
 	SaHpiEntityPathT *, enum ir_discoverstate *, int, char *,
 	struct ilo2_ribcl_idr_info *);
+static void ilo2_ribcl_discover_temp_sensors( struct oh_handler_state *,
+        struct oh_event *);
 static SaErrorT ilo2_ribcl_resource_set_failstatus( struct oh_handler_state *,
 	 SaHpiEntityPathT *, SaHpiBoolT);
 static SaErrorT ilo2_ribcl_undiscovered_fru( struct oh_handler_state *,
@@ -72,71 +75,46 @@ static SaErrorT ilo2_ribcl_controls(struct oh_handler_state *, int,
 static SaErrorT ilo2_ribcl_add_severity_sensor( struct oh_handler_state *,
 	struct oh_event *, int, SaHpiSensorTypeT, SaHpiEventStateT,
 	struct ilo2_ribcl_sensinfo *, char *);
+static SaErrorT ilo2_ribcl_add_threshold_sensor( struct oh_handler_state *,
+        struct oh_event *, int, SaHpiSensorTypeT, SaHpiEventStateT,
+        struct ilo2_ribcl_sensinfo *, char *);
 static void ilo2_ribcl_discover_chassis_sensors( struct oh_handler_state *, 
 	struct oh_event *);
 void ilo2_ribcl_add_resource_capability( struct oh_handler_state *, 
 	struct oh_event *, SaHpiCapabilitiesT);
+static SaErrorT ilo2_ribcl_get_iml( struct oh_handler_state *);
+static SaErrorT ilo2_ribcl_discovery(void *);
+gpointer ilo_thread_func( gpointer);
 
 #ifdef ILO2_RIBCL_SIMULATE_iLO2_RESPONSE
 static int ilo2_ribcl_getfile( char *, char *, int);
 #endif /* ILO2_RIBCL_SIMULATE_iLO2_RESPONSE */
 
-
+extern SaHpiBoolT close_handler;
 
 /**
- * ilo2_ribcl_discover_resources: Discovers resources for this instance 
- * of the iLO2 RIBCL plug-in 
- * @handler:  Handler data pointer.
+ * ilo2_ribcl_discovery() 
+ * @handler Handler data pointer.
  *
- * This function discovers resources for this instance of the iLO2 
- * RIBCL plugin.
- * Detailed description:
+ * This function is called by ilo2_ribcl_discover_resources and it is
+ * used for initail discovery and re-discovery, if  required
  *
- *	- Reset the discovery flags for each resource in DiscoveryData
- *	  via a call to ilo2_ribcl_clear_discoveryflags(). This allows us
- *	  to detect removed resources.
- *	- Call ilo2_ribcl_do_discovery(), which will open an SSL connection
- *	  to iLO2 using the hostname and port saved in the ilo2_ribcl_handler
- * 	  send the ILO2_RIBCL_GET_SERVER_DATA xml commands, and parse the
- *	  results into information stored in the DiscoveryData structure
- *	  within our private handler.
- *	- Finally, call a series of discovery routines that examine
- *	  DiscoveryData and generate rpt entries and events for
- *	  resources that have been added, removed, or failed.
- *
- * Return values:
- * Builds/updates internal RPT cache - normal operation.
- * Returns SA_OK for success
- * SA_ERR_HPI_OUT_OF_MEMORY - Cannot allocate space for internal memory
- * SA_ERR_HPI_INTERNAL_ERROR - SSL and XML errors
- * SA_ERR_HPI_INVALID_PARAMS - if plungin handler or private handler is null
+ * Return Values
+ *	SA_OK - On Success
+ *	SA_ERR_HPI_INTERNAL_ERROR if no entity_root
+ *	return values of other functions, if they fail
  *
  **/
-SaErrorT ilo2_ribcl_discover_resources(void *handler)
+static SaErrorT ilo2_ribcl_discovery(void *handler)
 {
 	struct oh_handler_state *oh_handler =
 		(struct oh_handler_state *) handler;
-        ilo2_ribcl_handler_t *ilo2_ribcl_handler;
+	ilo2_ribcl_handler_t *ilo2_ribcl_handler =
+		(ilo2_ribcl_handler_t *) oh_handler->data;
 	SaErrorT  ret;
 	SaHpiEntityPathT ep_root;
-	
-	if (!handler) {
-		err("ilo2_ribcl_discover_resources(): Invalid handler parameter.");
-		return(SA_ERR_HPI_INVALID_PARAMS);
-	}
 
-        ilo2_ribcl_handler = (ilo2_ribcl_handler_t *) oh_handler->data;
-	if(! ilo2_ribcl_handler) {
-		err("ilo2_ribcl_discover_resources(): Invalid private handler parameter.");
-		return(SA_ERR_HPI_INVALID_PARAMS);
-	}
-
-	if( ilo2_ribcl_handler->entity_root == NULL){
-		err("ilo2_ribcl_discover_resources(): entity_root is NULL.");
-		return(SA_ERR_HPI_INTERNAL_ERROR);
-	}
-
-	/* If our plugin becomes multhithreaded, lock here? */
+	dbg("Begin discovery/re-discovery");
 
 	ret = oh_encode_entitypath(ilo2_ribcl_handler->entity_root, &ep_root);
 	if( ret != SA_OK){
@@ -212,11 +190,158 @@ SaErrorT ilo2_ribcl_discover_resources(void *handler)
 		return( ret);
 	}
 
-	/* Since we do not have a polling thread for sensors (yet), we process
-	 * any changed sensor values as part of a discovery operation, since
-	 * the openhpid should be regularly calling us.
-	 */
-	ilo2_ribcl_process_sensors( oh_handler);
+	dbg("End discovery/re-discovery");
+
+	return (ret);
+}
+
+/**
+ * ilo_thread_func()
+ * @ilo_info: With mutex, cond and handler 
+ * Thread responsible for re-discover, if required
+ * Process sensors regularly
+ * In a continuous loop till close_handler is set
+ * Mutex and condition timer are used to exit immediately on signal
+ *
+ **/
+
+gpointer ilo_thread_func( gpointer ilo_info)
+{
+
+        SaErrorT ret;
+        iLO_info_t *pilo_info = (iLO_info_t*)ilo_info;
+        struct oh_handler_state *oh_handler = pilo_info->oh_handler;
+        ilo2_ribcl_handler_t *ilo2_ribcl_handler = oh_handler->data;
+
+	dbg("iLO thread started: process sensor, iml log");
+
+        wrap_g_mutex_lock(pilo_info->iLO_mutex);
+
+        while (close_handler == SAHPI_FALSE) {
+
+		ilo2_ribcl_process_sensors( oh_handler);
+		ret = ilo2_ribcl_get_iml(oh_handler);
+		if( ret != SA_OK){
+			err("ilo2_ribcl_get_iml():failed, check network");
+			err("May have to restart daemon if it continuous");
+			// TODO : on error ?
+		}
+
+		/* A sort of re-discovery */
+		if (ilo2_ribcl_handler->need_rediscovery == SAHPI_TRUE) {
+			dbg("Do a discovery due to a PS/FAN event");
+			ilo2_ribcl_discovery(oh_handler);
+			ilo2_ribcl_handler->need_rediscovery = SAHPI_FALSE;
+		}
+
+	#if GLIB_CHECK_VERSION (2, 32, 0)
+		gint64 time;
+		time = g_get_monotonic_time();
+		time = time + (180 * G_USEC_PER_SEC);
+		wrap_g_cond_timed_wait( pilo_info->iLO_cond,
+					pilo_info->iLO_mutex, time);
+	#else
+		GTimeVal time;
+		g_get_current_time(&time);
+		g_time_val_add(&time, (180 * G_USEC_PER_SEC));
+		wrap_g_cond_timed_wait( pilo_info->iLO_cond,
+					pilo_info->iLO_mutex, &time);
+	#endif
+
+	}
+
+	wrap_g_mutex_unlock(pilo_info->iLO_mutex);
+
+	dbg("iLO thread exited: process sensor, iml log");
+	return (gpointer *) SA_OK;
+}
+
+/**
+ * ilo2_ribcl_discover_resources: Discovers resources for this instance 
+ * of the iLO2 RIBCL plug-in 
+ * @handler:  Handler data pointer.
+ *
+ * This function discovers resources for this instance of the iLO2 
+ * RIBCL plugin.
+ * Detailed description:
+ *
+ *	- Reset the discovery flags for each resource in DiscoveryData
+ *	  via a call to ilo2_ribcl_clear_discoveryflags(). This allows us
+ *	  to detect removed resources.
+ *	- Call ilo2_ribcl_do_discovery(), which will open an SSL connection
+ *	  to iLO2 using the hostname and port saved in the ilo2_ribcl_handler
+ * 	  send the ILO2_RIBCL_GET_SERVER_DATA xml commands, and parse the
+ *	  results into information stored in the DiscoveryData structure
+ *	  within our private handler.
+ *	- Finally, call a series of discovery routines that examine
+ *	  DiscoveryData and generate rpt entries and events for
+ *	  resources that have been added, removed, or failed.
+ *
+ * Return values:
+ * Builds/updates internal RPT cache - normal operation.
+ * Returns SA_OK for success
+ * SA_ERR_HPI_OUT_OF_MEMORY - Cannot allocate space for internal memory
+ * SA_ERR_HPI_INTERNAL_ERROR - SSL and XML errors
+ * SA_ERR_HPI_INVALID_PARAMS - if plungin handler or private handler is null
+ *
+ **/
+SaErrorT ilo2_ribcl_discover_resources(void *handler)
+{
+	struct oh_handler_state *oh_handler =
+		(struct oh_handler_state *) handler;
+        ilo2_ribcl_handler_t *ilo2_ribcl_handler;
+	SaErrorT  ret;
+        
+        if( close_handler == SAHPI_TRUE ) {
+            INFO("ilo2_ribcl_handler is closed. Thread %p returning",
+                     g_thread_self());
+            return(SA_OK);
+        }
+
+	if (!handler) {
+		err("ilo2_ribcl_discover_resources(): NULL handler parameter.");
+		return(SA_ERR_HPI_INVALID_PARAMS);
+	}
+
+        ilo2_ribcl_handler = (ilo2_ribcl_handler_t *) oh_handler->data;
+	if(! ilo2_ribcl_handler) {
+		err("ilo2_ribcl_discover_resources(): NULL private handler");
+		return(SA_ERR_HPI_INVALID_PARAMS);
+	}
+
+	if( ilo2_ribcl_handler->entity_root == NULL){
+		err("ilo2_ribcl_discover_resources(): entity_root is NULL.");
+		return(SA_ERR_HPI_INTERNAL_ERROR);
+	}
+
+	/* If our plugin becomes multhithreaded, lock here? */
+
+	if (ilo2_ribcl_handler->discovery_complete == SAHPI_TRUE)
+		return SA_OK;
+
+	ret = ilo2_ribcl_discovery(oh_handler);
+	if ( ret != SA_OK){
+		err("ilo2_ribcl_discovery():failed");
+		return (ret);
+	}
+
+	ilo2_ribcl_handler->discovery_complete = SAHPI_TRUE;
+
+	if (ilo2_ribcl_handler->ilo_thread_data->hthread == NULL) {
+                ilo2_ribcl_handler->ilo_thread_data->hthread =
+			wrap_g_thread_create_new(
+					"ilo_sensor_thread",
+					ilo_thread_func,
+					ilo2_ribcl_handler->ilo_thread_data,
+					TRUE, 0);
+
+                if (ilo2_ribcl_handler->ilo_thread_data->hthread == NULL) {
+                        err("wrap_g_thread_create_new failed");
+                        return SA_ERR_HPI_INTERNAL_ERROR;
+                }
+        }
+
+	dbg("ilo_sensor_thread: Thread created successfully");
 
 	return(SA_OK);
 }
@@ -300,6 +425,7 @@ static SaErrorT ilo2_ribcl_do_discovery( ilo2_ribcl_handler_t *ir_handler)
 			if( ret <0){
 				err("ilo2_ribcl_do_discovery():"
 					"could not detect iLO type.");
+				free( d_response);
 				return( SA_ERR_HPI_INTERNAL_ERROR);
 			}
 			/*
@@ -311,6 +437,14 @@ static SaErrorT ilo2_ribcl_do_discovery( ilo2_ribcl_handler_t *ir_handler)
 		/* Send the command to iLO2 and get the response. */
 		ret = ilo2_ribcl_ssl_send_command( ir_handler, discover_cmd,
 				d_response, ILO2_RIBCL_DISCOVER_RESP_MAX);
+		if(ir_handler->ilo_type == ILO3) {
+			if(strstr(d_response,"Gen8") || strstr(d_response,"Gen9")) {
+				dbg("Found iLO4 MP");
+				ir_handler->ilo_type = ILO4;
+			} else
+				dbg("Found iLO3 MP");
+		}
+
 	}
 
 #else /* ILO2_RIBCL_SIMULATE_iLO2_RESPONSE not defined. This the the
@@ -320,6 +454,7 @@ static SaErrorT ilo2_ribcl_do_discovery( ilo2_ribcl_handler_t *ir_handler)
 		if( ret <0){
 			err("ilo2_ribcl_do_discovery():"
 				"could not detect iLO type.");
+			free( d_response);
 			return( SA_ERR_HPI_INTERNAL_ERROR);
 		}
 		/*
@@ -332,6 +467,15 @@ static SaErrorT ilo2_ribcl_do_discovery( ilo2_ribcl_handler_t *ir_handler)
 	/* Send the command to iLO2 and get the response. */
 	ret = ilo2_ribcl_ssl_send_command( ir_handler, discover_cmd,
 				d_response, ILO2_RIBCL_DISCOVER_RESP_MAX);
+	if(ir_handler->ilo_type == ILO3) {
+		if(strstr(d_response,"Gen8") || strstr(d_response,"Gen9")) {
+			dbg("Found iLO4 MP");
+			ir_handler->ilo_type = ILO4;
+		}else
+			dbg("Found iLO3 MP");
+
+	}
+
 
 #endif /* ILO2_RIBCL_SIMULATE_iLO2_RESPONSE */
 
@@ -356,13 +500,15 @@ static SaErrorT ilo2_ribcl_do_discovery( ilo2_ribcl_handler_t *ir_handler)
 							d_response);
 			break;
 		case ILO3:
+		case ILO4:
 			new_buffer = ir_xml_decode_chunked(d_response);
 			ret = ir_xml_parse_discoveryinfo( ir_handler, 
 							new_buffer);
 			break;
 		default:
 			err("ilo2_ribcl_do_discovery():"
-				"failed to detect ilo type.");
+				"failed to detect ilo type %d.",
+						ir_handler->ilo_type);
 	}
 	if( ret != RIBCL_SUCCESS){
 		err("ilo2_ribcl_do_discovery(): response parse failed.");
@@ -424,6 +570,11 @@ void ilo2_ribcl_clear_discoveryflags( ilo2_ribcl_handler_t *ir_handler){
 		ddata->vrmdata[idex].vrmflags &= ~IR_DISCOVERED;
 	}
 
+        /* Clear the Temperature Sensor flags */
+        for( idex = 4; idex <= ILO2_RIBCL_DISCOVER_TS_MAX; idex++){
+                ddata->tsdata[idex].tsflags &= ~IR_DISCOVERED;
+        }
+
 } /* end ilo2_ribcl_clear_discoveryflags() */
 
 
@@ -461,7 +612,7 @@ static SaErrorT ilo2_ribcl_discover_chassis(
 	struct oh_event *ev = NULL;
 	ilo2_ribcl_handler_t *ir_handler = NULL;
 	ilo2_ribcl_resource_info_t *res_info = NULL;
-	char *tmptag = NULL;
+	char *tmptag = NULL,*autopower= NULL;
 	size_t tagsize;
 	SaErrorT  ret;
 
@@ -528,7 +679,7 @@ static SaErrorT ilo2_ribcl_discover_chassis(
 		(SAHPI_CAPABILITY_RESOURCE | SAHPI_CAPABILITY_RESET 
 			| SAHPI_CAPABILITY_POWER | SAHPI_CAPABILITY_RDR); 
 	ev->resource.HotSwapCapabilities = ILO2_RIBCL_MANAGED_HOTSWAP_CAP_FALSE;
-	ev->resource.ResourceSeverity = SAHPI_CRITICAL;
+	ev->resource.ResourceSeverity = SAHPI_OK;
 	ev->resource.ResourceFailed = SAHPI_FALSE;
 
 	oh_init_textbuffer(&(ev->resource.ResourceTag));
@@ -595,9 +746,32 @@ static SaErrorT ilo2_ribcl_discover_chassis(
 	} else {
 		err("iLO2 RIBCL: Failed to setup Power Saver Control RDR. Plug-in will run without this control feature");
 	}
+	switch(ir_handler->ilo_type){
+		case ILO:
+		case ILO2:
+			autopower = "Auto Power On Control Delay: Min.(1)"
+				"/Disabled(2)/Random (3)"
+				"/15 sec (15)/30 sec (30)"
+				"/45 sec(45)/60 sec(60)";
+			break;
+		case ILO3:
+                        autopower = "Auto Power On Control Delay: "
+                                "Min.(1)/Random (3)/Always Off (5)";
+                        break;
+		case ILO4:
+			autopower = "Auto Power On Control Delay: " 
+				"Min.(1)/Random (3)"    
+				"/Restore (4)/Always Off (5)";
+			break;
+		default:
+			err("ilo2_ribcl_discover_chassis():"
+					"failed to detect ilo type.");
+			free( tmptag);
+			return SA_OK;
+	}
 
 	if(ilo2_ribcl_controls(oh_handler, ILO2_RIBCL_CTL_AUTO_POWER, ev,
-		"Auto Power Control Delay:Min.(1)/Disabled(2)/random (3)/15 sec (15)/30 sec (30)/45 sec(45)/60 sec(60)") == SA_OK) {
+		autopower) == SA_OK) {
 		/* Auto Power control RDR has beeen added successfully.
 		   Set RDR and CONTROL capability flags. Please note that
 		   failing to add control rdr is not treated as a critical
@@ -613,6 +787,9 @@ static SaErrorT ilo2_ribcl_discover_chassis(
 	/* Add sensor RDRs. We use three general system health sensors. */
 	ilo2_ribcl_discover_chassis_sensors( oh_handler, ev); 
 
+        /* Add Temperature Sensor RDR */
+        ilo2_ribcl_discover_temp_sensors( oh_handler, ev);
+
 	/* Add an inventory RDR */
 	ilo2_ribcl_discover_chassis_idr( oh_handler, ev, tmptag);
 	free( tmptag);
@@ -892,6 +1069,20 @@ static SaErrorT ilo2_ribcl_discover_fans( struct oh_handler_state *oh_handler,
 		if( (fanstatus != NULL) && !strcmp( fanstatus, "Failed")){
 			failed = SAHPI_TRUE;
 		}
+		
+		/**	If the FAN is not installed then FAN Status comes as
+		 *	"Not installed" or "Unknown", so do not discover this
+		 *	resource and skip to next FAN resource.
+		 * 	<FAN>
+		 *		<ZONE VALUE = "System"/>
+		 *  		<LABEL VALUE = "Fan 1"/>
+		 *		<STATUS VALUE = "Not Installed"/>
+		 *		<SPEED VALUE = "0" UNIT="Percentage"/>
+		 *        </FAN>	
+ 		 **/
+		if(!strcmp(fanstatus, "Not Installed")|| !strcmp(fanstatus, "Unknown")){
+			fandata->fanflags = ~IR_DISCOVERED;
+		}
 
 		/* include the fan location in the text tag */
 		tagsize =  strlen( fandata->label) +
@@ -990,8 +1181,16 @@ static SaErrorT ilo2_ribcl_discover_powersupplies( struct oh_handler_state *oh_h
 		/* Check if the discovery reported a failed power supply */
 		failed = SAHPI_FALSE;
 		psustatus = psudata->status;
-		if( (psustatus != NULL) && !strcmp( psustatus, "Failed")){
-			failed = SAHPI_TRUE;
+		if(psustatus != NULL) {
+			if(strstr( psustatus, "Fail") ||
+			                strstr( psustatus, "Lost")) {
+		                  failed = SAHPI_TRUE;
+		        }
+		}
+
+		if(!strcmp(psustatus, "Not Installed") || 
+			   !strcmp(psustatus, "Unknown")){
+			psudata->psuflags = ~IR_DISCOVERED;
 		}
 
 		if( psudata->psuflags & IR_DISCOVERED ){
@@ -1189,6 +1388,8 @@ static SaErrorT ilo2_ribcl_discovered_fru( struct oh_handler_state *oh_handler,
 	SaErrorT ret;
 	SaHpiBoolT resource_wasfailed;
 	ilo2_ribcl_resource_info_t *res_info = NULL;
+	ilo2_ribcl_handler_t *ilo2_ribcl_handler = 
+		(ilo2_ribcl_handler_t *) oh_handler->data;
 
 	switch( *d_state){
 
@@ -1213,7 +1414,12 @@ static SaErrorT ilo2_ribcl_discovered_fru( struct oh_handler_state *oh_handler,
 			(SAHPI_CAPABILITY_RESOURCE | SAHPI_CAPABILITY_FRU); 
 
 		ev->resource.HotSwapCapabilities = 0;
-		ev->resource.ResourceSeverity = SAHPI_CRITICAL;
+		if(ilo2_ribcl_handler->discovery_complete == SAHPI_FALSE){
+			ev->resource.ResourceSeverity = SAHPI_OK;
+		}else {
+			ev->resource.ResourceSeverity = SAHPI_CRITICAL;
+		}
+
 		ev->resource.ResourceFailed = isfailed;
 		oh_init_textbuffer(&(ev->resource.ResourceTag));
 		oh_append_textbuffer( &(ev->resource.ResourceTag), tag);
@@ -1447,6 +1653,131 @@ static SaErrorT ilo2_ribcl_discovered_fru( struct oh_handler_state *oh_handler,
 } /* end ilo2_ribcl_discovered_fru() */
 
 
+/**
+ * ilo2_ribcl_discover_temp_sensors:
+ * @oh_handler:  Handler data pointer.
+ * @event: Pointer to event structure for chassis resource event.
+ *
+ * This routine will create RDRs for the all temperature sensors that
+ * have been detected during a discovery operation.
+ * These sensors correspond to the system's temperature's sensors.
+ * Return values:
+ * None
+ **/
+static void ilo2_ribcl_discover_temp_sensors(
+                        struct oh_handler_state *oh_handler,
+                        struct oh_event *event)
+{
+        SaErrorT ret = SA_OK;
+        ilo2_ribcl_handler_t *ir_handler = NULL;
+        struct ilo2_ribcl_sensinfo si_initial;
+        ir_tsdata_t *tsdata = NULL;
+        int idex, cur_reading = I2R_SEN_VAL_UNINITIALIZED;
+        char *label = NULL, *location = NULL, *description = NULL;
+
+        ir_handler = ( ilo2_ribcl_handler_t *) oh_handler->data;
+        memset( &si_initial, 0, sizeof( struct ilo2_ribcl_sensinfo));
+
+        /* Look for the all temperature isensors from the RIBCL */
+	for( idex = 4; idex <= ILO2_RIBCL_DISCOVER_TS_MAX; idex++){
+		tsdata = &(ir_handler->DiscoveryData.tsdata[idex]);
+		if(tsdata->tsflags != IR_DISCOVERED){
+			break;
+		}
+
+		cur_reading = atoi( tsdata->reading );
+
+		if ( cur_reading != I2R_SEN_VAL_UNINITIALIZED){
+			si_initial.sens_num = idex;
+			si_initial.sens_ev_state = SAHPI_ES_UNSPECIFIED;
+			si_initial.sens_enabled = SAHPI_TRUE;
+			si_initial.sens_ev_enabled = SAHPI_FALSE;
+			si_initial.sens_assertmask = SAHPI_ES_UNSPECIFIED;
+			si_initial.sens_deassertmask = SAHPI_ES_UNSPECIFIED;
+			si_initial.sens_value = I2R_SEN_VAL_UNINITIALIZED;
+
+			si_initial.status =  tsdata->status;
+			si_initial.threshold.LowCritical.IsSupported =
+								SAHPI_FALSE;
+			si_initial.threshold.LowMajor.IsSupported = SAHPI_FALSE;
+			si_initial.threshold.LowMinor.IsSupported = SAHPI_FALSE;
+
+			si_initial.threshold.UpCritical.IsSupported =
+								SAHPI_TRUE;
+			si_initial.threshold.UpCritical.Type =
+						SAHPI_SENSOR_READING_TYPE_INT64;
+			si_initial.threshold.UpCritical.Value.SensorInt64 =
+						atoi(tsdata->criticalvalue);
+
+			si_initial.threshold.UpMajor.IsSupported = SAHPI_TRUE;
+			si_initial.threshold.UpMajor.Type =
+						SAHPI_SENSOR_READING_TYPE_INT64;
+			si_initial.threshold.UpMajor.Value.SensorInt64 =
+						atoi(tsdata->cautionvalue);
+
+			si_initial.threshold.UpMinor.IsSupported = SAHPI_FALSE;
+			si_initial.threshold.PosThdHysteresis.IsSupported =
+								SAHPI_FALSE;
+			si_initial.threshold.NegThdHysteresis.IsSupported =
+								SAHPI_FALSE;
+
+			/* Update the sensor info with current reading, this
+			 * reading will be utilized sensor event assetion post
+			 * discovery.
+			 */
+			if ((cur_reading >=
+                             si_initial.threshold.UpMajor.Value.SensorInt64) &&
+                             (cur_reading <
+                             si_initial.threshold.UpCritical.
+                                                Value.SensorInt64)) {
+                                si_initial.sens_ev_state = SAHPI_ES_UPPER_MAJOR;
+			} else if (cur_reading >
+                                   si_initial.threshold.UpCritical.Value.
+                                   SensorInt64) {
+                                        si_initial.sens_ev_state =
+                                                SAHPI_ES_UPPER_CRIT;
+			}
+
+			label = tsdata->label;
+			location = tsdata->location;
+			description = (char*)malloc(snprintf(NULL, 0, "%s %s",
+						label, location) + 3);
+
+			if ( description != NULL) {
+				strcpy(description, label);
+				strcat(description, ": ");
+				strcat(description, location);
+
+				ret = ilo2_ribcl_add_threshold_sensor(
+				       oh_handler, event,idex,SAHPI_TEMPERATURE,
+				       SAHPI_ES_UNSPECIFIED, &si_initial,
+				       description);
+
+				if ( ret == SA_OK){
+					tsdata->rid = 
+						event->resource.ResourceId;
+					ilo2_ribcl_add_resource_capability( 
+							oh_handler, event,
+							(SAHPI_CAPABILITY_RDR | 
+							 SAHPI_CAPABILITY_SENSOR));
+
+				} else {
+					err("ilo2_ribcl_discover_temp_sensors: "
+					      "Failed to set up temp sensor.");
+				}
+
+				free(description);
+
+			} else {
+				err("ilo2_ribcl_discover_temp_sensors: "
+						"Memory Allocation failed.");
+			}
+		}
+
+	} /* end for idex */
+
+}/* end ilo2_ribcl_discover_temperator_sensors() */
+
 
 /**
  * ilo2_ribcl_resource_set_failstatus
@@ -1484,7 +1815,6 @@ static SaErrorT ilo2_ribcl_resource_set_failstatus(
 		return( SA_ERR_HPI_NOT_PRESENT);
 	}
 
-	rpt->ResourceFailed = resource_failed;
 
 	/* Generate a RESOURCE_FAILURE event */
 
@@ -1497,19 +1827,26 @@ static SaErrorT ilo2_ribcl_resource_set_failstatus(
 	ev->resource = *rpt;
 	ev->hid = oh_handler->hid;
 	ev->event.EventType = SAHPI_ET_RESOURCE;
-	ev->event.Severity = ev->resource.ResourceSeverity;
+	ev->event.Severity = SAHPI_CRITICAL;
 	ev->event.Source = ev->resource.ResourceId;
 	if ( oh_gettimeofday(&(ev->event.Timestamp)) != SA_OK){
 		ev->event.Timestamp = SAHPI_TIME_UNSPECIFIED;
 	}
-	if( resource_failed == SAHPI_FALSE){
+	if(( resource_failed == SAHPI_FALSE) && 
+                                  (rpt->ResourceFailed == SAHPI_TRUE)) {
 		ev->event.EventDataUnion.ResourceEvent.ResourceEventType
 					= SAHPI_RESE_RESOURCE_RESTORED;
-	} else {
+	} else if((resource_failed == SAHPI_TRUE) && 
+                                  (rpt->ResourceFailed != SAHPI_TRUE)) {
 		ev->event.EventDataUnion.ResourceEvent.ResourceEventType
 					= SAHPI_RESE_RESOURCE_FAILURE;
+	} else {
+		/* Return as the state has not changed */
+		oh_event_free(ev, 0);
+		return( SA_OK);
 	}
 				
+	rpt->ResourceFailed = resource_failed;
 	oh_evt_queue_push(oh_handler->eventq, ev);
 
 	return( SA_OK);
@@ -1663,7 +2000,7 @@ void ilo2_ribcl_free_discoverydata( ilo2_ribcl_handler_t *ir_handle)
 		free(  ddata->fwdata.version_string);
 	}
 
-	if( ddata->serial_number != NULL){
+	if( ddata->system_cpu_speed != NULL){
 		free( ddata->system_cpu_speed);
 	}
 
@@ -1723,7 +2060,36 @@ void ilo2_ribcl_free_discoverydata( ilo2_ribcl_handler_t *ir_handle)
 		}
 	}
 
-	/* Temperature data not yet implemented */
+	/* Free the temp sensor data */
+	for( idex = 4; idex <= ILO2_RIBCL_DISCOVER_TS_MAX; idex++){
+                if( ddata->tsdata[idex].label){
+                        free( ddata->tsdata[idex].label);
+                }
+                if( ddata->tsdata[idex].location){
+                        free( ddata->tsdata[idex].location);
+                }
+                if( ddata->tsdata[idex].status){
+                        free( ddata->tsdata[idex].status);
+                }
+                if( ddata->tsdata[idex].reading){
+                        free( ddata->tsdata[idex].reading);
+                }
+                if( ddata->tsdata[idex].readingunits){
+                        free( ddata->tsdata[idex].readingunits);
+                }
+                if( ddata->tsdata[idex].cautionvalue){
+                        free( ddata->tsdata[idex].cautionvalue);
+                }
+                if( ddata->tsdata[idex].cautionunit){
+                        free( ddata->tsdata[idex].cautionunit);
+                }
+                if( ddata->tsdata[idex].criticalvalue){
+                        free( ddata->tsdata[idex].criticalvalue);
+                }
+                if( ddata->tsdata[idex].criticalunit){
+                        free( ddata->tsdata[idex].criticalunit);
+                }
+        }
 	
 }
 
@@ -1853,7 +2219,7 @@ static SaErrorT ilo2_ribcl_controls(struct oh_handler_state *oh_handler,
 		default:
 		{
 			err("ilo2_ribcl_controls(): Invalid iLO2 RIBCL control type");
-			g_free(rdrptr);
+			wrap_g_free(rdrptr);
 			return(SA_ERR_HPI_INTERNAL_ERROR);
 		}
 	}
@@ -1868,7 +2234,7 @@ static SaErrorT ilo2_ribcl_controls(struct oh_handler_state *oh_handler,
 	cinfo_ptr = g_memdup(&cinfo, sizeof(cinfo));
 	if(cinfo_ptr == NULL) {
 		err("ilo2_ribcl_controls(): Out of memory.");
-		g_free(rdrptr);
+		wrap_g_free(rdrptr);
 		return(SA_ERR_HPI_OUT_OF_MEMORY);
 	}
 	
@@ -1876,8 +2242,8 @@ static SaErrorT ilo2_ribcl_controls(struct oh_handler_state *oh_handler,
 		rdrptr, cinfo_ptr, 0);
 	if (err) {
 		err("Could not add RDR. Error=%s.", oh_lookup_error(err));
-		g_free(rdrptr);
-		g_free(cinfo_ptr);
+		wrap_g_free(rdrptr);
+		wrap_g_free(cinfo_ptr);
 		return(SA_ERR_HPI_INTERNAL_ERROR);
 	} else {
 		event->rdrs = g_slist_append(event->rdrs, rdrptr);
@@ -1969,7 +2335,7 @@ static SaErrorT ilo2_ribcl_add_severity_sensor(
  	 * to be associated with this RDR. */
 	si = g_memdup(sens_info, sizeof(struct ilo2_ribcl_sensinfo));
 	if( si == NULL){
-		g_free( rdr);
+		wrap_g_free( rdr);
 		err("ilo2_ribcl_add_severity_sensor: Memory allocation failed.");
 		return(SA_ERR_HPI_OUT_OF_MEMORY);
 	}
@@ -1979,8 +2345,8 @@ static SaErrorT ilo2_ribcl_add_severity_sensor(
 	if( ret != SA_OK){
 		err("ilo2_ribcl_add_severity_sensor: could not add RDR. Error = %s.",
 			oh_lookup_error(ret));
-		g_free( si);
-		g_free( rdr);
+		wrap_g_free( si);
+		wrap_g_free( rdr);
 		return( SA_ERR_HPI_INTERNAL_ERROR);
 	} else {
 		event->rdrs = g_slist_append(event->rdrs, rdr);
@@ -1991,6 +2357,129 @@ static SaErrorT ilo2_ribcl_add_severity_sensor(
 } /* end ilo2_ribcl_add_severity_sensor() */
 
 
+/**
+ * ilo2_ribcl_add_threshold_sensor:
+ * @oh_handler:  Handler data pointer.
+ * @event: Pointer to event structure for sensor parent resource event.
+ * @sens_num: Sensor number for new sensor.
+ * @sens_type: HPI type of new sensor.
+ * @supported_states: Mask of all the EV states this sensor can support.
+ * @sens_info: Private sensor info associated with RDR.
+ * @description: Character string description of temperature sensor.
+ *
+ * This routine creates a new sensor of category SAHPI_EC_THRESHOLD, using
+ * the data given by the parameters. The new sensor RDR is added to
+ * the parent resource given in the oh_event structure paramenter.
+ *
+ * The following fields in the SensorRec of the RDR will be set to fixed
+ * values:
+ *      EnableCtrl = SAHPI_TRUE
+ *      EventCtrl  = SAHPI_SEC_READ_ONLY
+ *      DataFormat.IsSupported = SAHPI_TRUE
+ *      DataFormat.ReadingType = SAHPI_SENSOR_READING_TYPE_UINT64
+ *      DataFormat.BaseUnits   = SAHPI_SU_DEGREES_C
+ *      DataFormat.ModifierUse = SAHPI_SMUU_NONE
+ *      DataFormat.Percentage  = SAHPI_FALSE
+ *      ThresholdDefn.IsAccessible = SAHPI_FALSE
+ *
+ * Return values:
+ * SA_ERR_HPI_OUT_OF_MEMORY - memory allocation failed.
+ * SA_ERR_HPI_INTERNAL_ERROR - could not add sensor RDR
+ **/
+static SaErrorT ilo2_ribcl_add_threshold_sensor(
+                        struct oh_handler_state *oh_handler,
+                        struct oh_event *event,
+                        int sens_num,
+                        SaHpiSensorTypeT sens_type,
+                        SaHpiEventStateT supported_states,
+                        struct ilo2_ribcl_sensinfo *sens_info,
+                        char *description)
+{
+
+	SaErrorT ret = SA_OK;
+	SaHpiRdrT *rdr;
+	SaHpiSensorRecT *sensor_rec;
+	struct ilo2_ribcl_sensinfo *si;
+
+	rdr = (SaHpiRdrT *)g_malloc0(sizeof(SaHpiRdrT));
+	if( rdr == NULL){
+		err("ilo2_ribcl_add_threshold_sensor: "
+				"Memory allocation failed.");
+		return(SA_ERR_HPI_OUT_OF_MEMORY);
+	}
+
+	/* Fill in generic RDR stuff */
+	rdr->RdrType = SAHPI_SENSOR_RDR;
+	rdr->Entity  = event->resource.ResourceEntity;
+	rdr->IsFru   = SAHPI_FALSE;
+
+	/* Fill in sensor specific info */
+	sensor_rec = &(rdr->RdrTypeUnion.SensorRec);
+	sensor_rec->Num = sens_num;
+	sensor_rec->Type = sens_type;
+	sensor_rec->Category = SAHPI_EC_THRESHOLD;
+	sensor_rec->EnableCtrl = SAHPI_TRUE;
+	sensor_rec->EventCtrl  = SAHPI_SEC_READ_ONLY;
+	sensor_rec->Events = SAHPI_ES_UNSPECIFIED;
+
+	sensor_rec->DataFormat.IsSupported = SAHPI_TRUE;
+	sensor_rec->DataFormat.ReadingType = SAHPI_SENSOR_READING_TYPE_INT64;
+	sensor_rec->DataFormat.BaseUnits   = SAHPI_SU_DEGREES_C;
+	sensor_rec->DataFormat.ModifierUse = SAHPI_SMUU_NONE;
+	sensor_rec->DataFormat.Percentage  = SAHPI_FALSE;
+	/* Range and AccuracyFactor have been cleared by g_malloc0() */
+
+	sensor_rec->DataFormat.ModifierUnits = SAHPI_SU_UNSPECIFIED;
+	sensor_rec->DataFormat.Range.Flags = SAHPI_SRF_MAX |
+                                                SAHPI_SRF_NORMAL_MAX;
+        sensor_rec->DataFormat.Range.Max.IsSupported = SAHPI_TRUE;
+        sensor_rec->DataFormat.Range.Max.Type = SAHPI_SENSOR_READING_TYPE_INT64;
+        sensor_rec->DataFormat.Range.Max.Value.SensorInt64 =
+                        sens_info->threshold.UpCritical.Value.SensorInt64;
+
+        sensor_rec->DataFormat.Range.NormalMax.IsSupported = SAHPI_TRUE;
+        sensor_rec->DataFormat.Range.NormalMax.Type =
+                                        SAHPI_SENSOR_READING_TYPE_INT64;
+        sensor_rec->DataFormat.Range.NormalMax.Value.SensorInt64 =
+                                sens_info->threshold.UpMajor.Value.SensorInt64;
+        sensor_rec->DataFormat.AccuracyFactor =  0;
+
+        sensor_rec->ThresholdDefn.IsAccessible = SAHPI_TRUE;
+        sensor_rec->ThresholdDefn.ReadThold = SAHPI_ES_UPPER_CRIT |
+                                SAHPI_ES_UPPER_MAJOR;
+        sensor_rec->ThresholdDefn.WriteThold = 0x0;
+
+	oh_init_textbuffer(&(rdr->IdString));
+	oh_append_textbuffer(&(rdr->IdString), description);
+
+
+	/* Copy the private sensor data initial values into a new allocation
+	 * to be associated with this RDR. */
+	si = g_memdup(sens_info, sizeof(struct ilo2_ribcl_sensinfo));
+	if( si == NULL){
+		wrap_g_free( rdr);
+		err("ilo2_ribcl_add_threshold_sensor: Memory allocation "
+				"failed.");
+		return(SA_ERR_HPI_OUT_OF_MEMORY);
+	}
+
+	ret = oh_add_rdr(oh_handler->rptcache, event->resource.ResourceId,
+			rdr, si, 0);
+
+	if( ret != SA_OK){
+		err("ilo2_ribcl_add_threshold_sensor: could not add RDR. "
+				"Error = %s.", oh_lookup_error(ret));
+		wrap_g_free( si);
+		wrap_g_free( rdr);
+		return( SA_ERR_HPI_INTERNAL_ERROR);
+	} else {
+		event->rdrs = g_slist_append(event->rdrs, rdr);
+	}
+
+	return( SA_OK);
+
+} /* end ilo2_ribcl_add_threshold_sensor() */
+
 
 /**
  * ilo2_ribcl_discover_chassis_sensors:
@@ -2182,6 +2671,98 @@ void ilo2_ribcl_add_resource_capability( struct oh_handler_state *oh_handler,
 
 
 
+/**
+ * ilo2_ribcl_get_iml
+ * @oh_handler: The private handler for this plugin instance.
+ *
+ *	This routine sends the ILO2_RIBCL_GET_EVENT_LOG xml commands 
+ *	to an iLO2 via a SSL connection, receive the resulting output
+ *	from iLO2, parses that output, and writes the IML into log file.
+ *
+ *
+ * Return values:
+ *	SA_OK for success.
+ *	SA_ERR_HPI_OUT_OF_MEMORY if allocation fails
+ *	SA_ERR_HPI_INTERNAL_ERROR if our customized command is null or
+ *		if response parsing fails.
+ *
+ **/
+static SaErrorT ilo2_ribcl_get_iml( struct oh_handler_state *oh_handler)
+{
+	ilo2_ribcl_handler_t *ir_handler = NULL;
+	char *discover_cmd = NULL;
+	char *d_response = NULL;	/* command response buffer */
+	int ret = 0;
+	char *new_buffer = NULL;
+	ir_handler = (ilo2_ribcl_handler_t *) oh_handler->data;
+
+	/* Allocate a temporary response buffer. Since the discover
+	 * command should be infrequently run, we dynamically allocate
+	 * instead of keeping a buffer around all the time.
+	 */
+
+	d_response = malloc(ILO2_RIBCL_DISCOVER_RESP_MAX);
+	if( d_response == NULL){
+		err("ilo2_ribcl_get_iml(): failed to allocate resp buffer.");
+		return( SA_ERR_HPI_OUT_OF_MEMORY);
+	}
+
+	/* Retrieve our customized command buffer */
+	discover_cmd = ir_handler->ribcl_xml_cmd[IR_CMD_GET_EVENT_LOG];
+	if( discover_cmd == NULL){
+		err("ilo2_ribcl_get_iml(): null customized command.");
+		free( d_response);
+		return( SA_ERR_HPI_INTERNAL_ERROR);
+	}
+
+
+        /* Send the command to iLO2 and get the response. */
+        ret = ilo2_ribcl_ssl_send_command( ir_handler, discover_cmd,
+				d_response, ILO2_RIBCL_DISCOVER_RESP_MAX);
+        if( ret !=0 ){
+            err("ilo2_ribcl_get_iml(): command send failed.");
+            free( d_response);
+            return( SA_ERR_HPI_INTERNAL_ERROR);
+        }
+
+	/* Now, parse the response. The information related to DIMM error will 
+	 * be extracted.*/
+	/*switch based on ilo*/
+	switch(ir_handler->ilo_type){
+
+		case ILO:
+		case ILO2:
+			ret = ir_xml_parse_iml( oh_handler, 
+							d_response);
+			break;
+		case ILO3:
+		case ILO4:
+			new_buffer = ir_xml_decode_chunked(d_response);
+			ret = ir_xml_parse_iml( oh_handler, 
+							new_buffer);
+			break;
+		default:
+			err("ilo2_ribcl_get_iml():"
+				"failed to detect ilo type.");
+	}
+	if( ret != RIBCL_SUCCESS){
+		err("ilo2_ribcl_get_iml(): response parse failed in \
+                        ir_xml_parse_iml().");
+		free( d_response);
+		free( new_buffer);
+		return( SA_ERR_HPI_INTERNAL_ERROR);
+	}
+	/* We're finished. Free up the temporary response buffer */
+	free( d_response);
+	/*
+	 * new_buffer will be NULL for ILO2
+	 * it is absolutely fine to free NULL
+	 */
+	free( new_buffer);
+	return( SA_OK);
+	
+}
+
 
 #ifdef ILO2_RIBCL_SIMULATE_iLO2_RESPONSE
 /**
