@@ -42,27 +42,8 @@ cIpmiControl::CreateRdr( SaHpiRptEntryT &resource, SaHpiRdrT &rdr )
   if ( cIpmiRdr::CreateRdr( resource, rdr ) == false )
        return false;
 
-  if (    !(resource.ResourceCapabilities & SAHPI_CAPABILITY_RDR)
-       || !(resource.ResourceCapabilities & SAHPI_CAPABILITY_CONTROL ) )
-     {
-       // update resource
-       resource.ResourceCapabilities |= SAHPI_CAPABILITY_RDR|SAHPI_CAPABILITY_CONTROL;
-
-       struct oh_event *e = (struct oh_event *)g_malloc0( sizeof( struct oh_event ) );
-
-       if ( !e )
-          {
-            stdlog << "out of space !\n";
-            return false;
-          }
-
-       memset( e, 0, sizeof( struct oh_event ) );
-       e->type              = OH_ET_RESOURCE;
-       e->u.res_event.entry = resource;
-
-       stdlog << "cIpmiControl::CreateRdr OH_ET_RESOURCE Event resource " << resource.ResourceId << "\n";
-       m_mc->Domain()->AddHpiEvent( e );       
-     }
+  // update resource
+  resource.ResourceCapabilities |= SAHPI_CAPABILITY_RDR|SAHPI_CAPABILITY_CONTROL;
 
   // control record
   SaHpiCtrlRecT &rec = rdr.RdrTypeUnion.CtrlRec;
