@@ -38,7 +38,6 @@
 #ifndef _OA_SOAP_CALLS_H_
 #define _OA_SOAP_CALLS_H_
 
-
 /* Include files */
 #include "oa_soap_callsupport.h"
 
@@ -46,6 +45,7 @@
 #define MAX_SERIAL_NUM_LENGTH 32
 #define MAX_PART_NUM_LENGTH 32
 #define MAX_MODEL_NUM_LENGTH 32
+#define MAX_PRODUCT_NAME_LENGTH 32
 
 /* Moved these #defines from oa_soap.h - which includes this header file. */
 /* Max Blade in HP BladeSystem c7000 c-Class enclosure*/
@@ -63,6 +63,11 @@ typedef unsigned char byte;
                         "<hpoa:bayNumber>%d</hpoa:bayNumber>" \
                         "</hpoa:getBladeInfo>\n"
 
+#define GET_BLADE_PORTMAP \
+                        "<hpoa:getBladePortMap>" \
+                        "<hpoa:bayNumber>%d</hpoa:bayNumber>" \
+                        "</hpoa:getBladePortMap>\n"
+
 #define SUBSCRIBE_FOR_EVENTS \
                         "<hpoa:subscribeForEvents>" \
                         "</hpoa:subscribeForEvents>\n"
@@ -80,13 +85,14 @@ typedef unsigned char byte;
                         "<hpoa:lcdEvents>%d</hpoa:lcdEvents>" \
                         "</hpoa:getEvent>\n"
 
-#define GET_ALL_EVENTS \
-                        "<hpoa:getAllEvents>" \
+#define GET_ALL_EVENTSEX \
+                        "<hpoa:getAllEventsEx>" \
                         "<hpoa:pid>%d</hpoa:pid>" \
                         "<hpoa:waitTilEventHappens>" \
                                 "%d</hpoa:waitTilEventHappens>" \
                         "<hpoa:lcdEvents>%d</hpoa:lcdEvents>" \
-                        "</hpoa:getAllEvents>\n"
+                        "<hpoa:oaFwVersion>%s</hpoa:oaFwVersion>" \
+                        "</hpoa:getAllEventsEx>\n"
 
 #define SET_BLADE_POWER \
                         "<hpoa:setBladePower>" \
@@ -119,6 +125,10 @@ typedef unsigned char byte;
                         "<hpoa:bayNumber>%d</hpoa:bayNumber>" \
                         "</hpoa:getOaInfo>\n"
 
+#define GET_OA_ID \
+                        "<hpoa:getOaId>" \
+                        "</hpoa:getOaId>\n"
+
 #define GET_POWER_CONFIG_INFO \
                         "<hpoa:getPowerConfigInfo>" \
                         "</hpoa:getPowerConfigInfo>\n"
@@ -171,6 +181,11 @@ typedef unsigned char byte;
                         "<hpoa:bayNumber>%d</hpoa:bayNumber>" \
                         "</hpoa:getInterconnectTrayInfo>\n"
 
+#define GET_INTERCONNECT_TRAY_PORT_MAP \
+                        "<hpoa:getInterconnectTrayPortMap>" \
+                        "<hpoa:bayNumber>%d</hpoa:bayNumber>" \
+                        "</hpoa:getInterconnectTrayPortMap>\n"
+
 #define GET_FAN_INFO \
                         "<hpoa:getFanInfo>" \
                         "<hpoa:bayNumber>%d</hpoa:bayNumber>" \
@@ -280,6 +295,63 @@ typedef unsigned char byte;
 			"<hpoa:buttonLock>%d</hpoa:buttonLock>" \
 			"</hpoa:setLcdButtonLock>\n"
 
+#define GET_BLADE_INFO_ARRAY \
+                        "<hpoa:getBladeInfoArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getBladeInfoArray>\n"
+
+#define GET_INTERCONNECT_TRAY_INFO_ARRAY \
+                        "<hpoa:getInterconnectTrayInfoArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getInterconnectTrayInfoArray>\n"
+
+#define GET_INTERCONNECT_TRAY_STATUS_ARRAY \
+                        "<hpoa:getInterconnectTrayStatusArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getInterconnectTrayStatusArray>\n"
+						
+#define GET_INTERCONNECT_TRAY_PORTMAP_ARRAY \
+                        "<hpoa:getInterconnectTrayPortMapArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getInterconnectTrayPortMapArray>\n"
+
+#define GET_POWER_SUPPLY_INFO_ARRAY \
+                        "<hpoa:getPowerSupplyInfoArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getPowerSupplyInfoArray>\n"
+						
+#define GET_POWER_SUPPLY_STATUS_ARRAY \
+                        "<hpoa:getPowerSupplyStatusArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getPowerSupplyStatusArray>\n"
+
+#define GET_FAN_INFO_ARRAY \
+                        "<hpoa:getFanInfoArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getFanInfoArray>\n"
+						
+#define GET_OA_INFO_ARRAY \
+                        "<hpoa:getOaInfoArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getOaInfoArray>\n"
+
+#define GET_OA_STATUS_ARRAY \
+                        "<hpoa:getOaStatusArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getOaStatusArray>\n"
+
+#define GET_BLADE_STATUS_ARRAY \
+                        "<hpoa:getBladeStatusArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getBladeStatusArray>\n"
+
+#define GET_BLADE_PORTMAP_ARRAY \
+                        "<hpoa:getBladePortMapArray>" \
+                        "<hpoa:bayArray>%s</hpoa:bayArray>" \
+                        "</hpoa:getBladePortMapArray>\n"
+
+
+              
 /* Enumerated types used for specific SOAP commands */
 #define HPOA_BOOLEAN_LENGTH    11      /* Max length of these enums + 1 */
 OA_SOAP_ENUM(hpoa_boolean,
@@ -462,10 +534,19 @@ OA_SOAP_ENUM(interconnectTrayType,
         INTERCONNECT_TRAY_TYPE_SAS,
         INTERCONNECT_TRAY_TYPE_MAX)
 
-OA_SOAP_ENUM(interconnectTraySizeType,
+enum interconnectTraySizeType {
         INTERCONNECT_TRAY_SIZE_TYPE_MT,
+        INTERCONNECT_TRAY_SIZE_TYPE_0X1,
         INTERCONNECT_TRAY_SIZE_TYPE_1X1,
-        INTERCONNECT_TRAY_SIZE_TYPE_2x1)
+        INTERCONNECT_TRAY_SIZE_TYPE_2x1,
+        INTERCONNECT_TRAY_SIZE_TYPE_3x1};
+
+OA_SOAP_ENUM_STRING(interconnectTraySizeType,
+        INTERCONNECT_TRAY_SIZE_TYPE_MT,
+	INTERCONNECT_TRAY_SIZE_TYPE-1X1,
+        INTERCONNECT_TRAY_SIZE_TYPE_1X1,
+        INTERCONNECT_TRAY_SIZE_TYPE_2X1,
+        INTERCONNECT_TRAY_SIZE_TYPE-2x1)
 
 OA_SOAP_ENUM(interconnectTrayPassThroughEnabled,
         INTERCONNECT_TRAY_PASSTHROUGH_UNKNOWN,
@@ -801,7 +882,40 @@ OA_SOAP_ENUM(eventType,
         EVENT_MEDIA_DRIVE_REMOVED2,
         EVENT_MEDIA_INSERTED2,
         EVENT_MEDIA_REMOVED2,
-        EVENT_ENC_GRP_CAP)
+        EVENT_ENC_GRP_CAP,
+        EVENT_BLADE_GRP_CAP_TIMEOUT,
+        EVENT_VLAN_INFO_CHANGED,
+        EVENT_SESSION_TIMEOUT_CHANGED,
+        EVENT_SBL_DOMAIN_INFO_CHANGED,
+        EVENT_FW_MGMT_SETTINGS_CHANGED,
+        EVENT_FW_DISCOVERY,
+        EVENT_FW_UPDATE,
+        EVENT_NET_SERVICE_RESTART,
+        EVENT_PS_OVERLOAD_REPAIRED,
+        EVENT_VCM_IPV6_URL_CHANGED,
+        EVENT_VCM_MIN_OA_FW_VER_CHANGED,
+        EVENT_SNMPV3_INFO_CHANGED,
+        EVENT_OA_ERS_CONFIG_CHANGED,
+        EVENT_OA_ERS_TEST_MANUAL,
+        EVENT_OA_ERS_DATACOLLECTION_MANUAL,
+        EVENT_OA_ERS_DATACOLLECTION_SUCCESS,
+        EVENT_OA_ERS_DATACOLLECTION_FAILURE,
+        EVENT_OA_ERS_MAINTENANCE_SET,
+        EVENT_OA_ERS_MAINTENANCE_CLEARED,
+        EVENT_OA_ERS_STATUS,
+        EVENT_OA_LOGIN_BANNER_SETTINGS_CHANGED,
+        EVENT_OA_ERS_EVENTS_CLEARED,
+        EVENT_ERS_CA_CERT_ADDED,
+        EVENT_ERS_CA_CERT_REMOVED,
+        EVENT_FW_MGMT_LOG_CLEARED,
+        EVENT_FW_MGMT_ISO_STATUS,
+        EVENT_LANG_PACK_ADDED,
+        EVENT_LANG_PACK_REMOVED,
+        EVENT_TCP_TIMEOUT_CHANGED,
+        EVENT_BLADE_FQDN_INFO_REFRESH,
+        EVENT_TRAY_FQDN_INFO_REFRESH,
+        EVENT_VCM_FQDN_INFO_REFRESH,
+        EVENT_EBIPAV6_INFO_CHANGED_EX)
 
 /* This is not part of the SOAP response data from the OA, but is useful
  * for identifying the type of data that comes back from getAllEvents().
@@ -958,6 +1072,11 @@ struct getBladeInfo
 {
         int bayNumber;
 };
+/* Structures that supply information to OA SOAP calls */
+struct getBladePortMap
+{
+        int bayNumber;
+};
 
 struct diagnosticChecks
 {
@@ -1070,6 +1189,7 @@ struct enclosureInfo
         char *pduSparePartNumber;
         xmlNode *extraData;             /* Items are struct extraDataInfo */
         int powerType; 
+        int enclosureStatus; 
 };
 
 struct oaStatus
@@ -1110,6 +1230,11 @@ struct oaInfo
         xmlNode *extraData;             /* Items are struct extraDataInfo */
 };
 
+struct OaId
+{
+        byte bayNumber;	
+};
+
 struct getOaInfo
 {
         int bayNumber;
@@ -1139,9 +1264,9 @@ struct bladeMpInfo
 
 struct bladeMezzSlotPort
 {
-        byte slotNumber;
-        byte interconnectTrayBayNumber;
-        byte interconnectTrayPortNumber;
+        char *slotNumber;
+        char *interconnectTrayBayNumber;
+        char *interconnectTrayPortNumber;
         xmlNode *extraData;             /* Items are struct extraDataInfo */
 };
 
@@ -1149,13 +1274,13 @@ struct bladeMezzSlotInfo
 {
         enum bladeMezzSlotType type;
         int sizeslot;
-        struct bladeMezzSlotPort slot;
+        xmlNode *slot;
         xmlNode *extraData;             /* Items are struct extraDataInfo */
 };
 
 struct bladeMezzDevPort
 {
-        byte portNumber;
+        char *portNumber;
         char *wwpn;
         enum fabricType fabric;
         enum fabricStatus status;
@@ -1168,26 +1293,25 @@ struct bladeMezzDevInfo
         enum bladeMezzDevType type;
         enum bladeMezzDevStatus status;
         int sizeport;
-        struct bladeMezzDevPort port;
+        xmlNode *port;
         xmlNode *extraData;             /* Items are struct extraDataInfo */
 };
 
 struct bladeMezzInfo
 {
-        byte mezzNumber;
-        struct bladeMezzSlotInfo mezzSlots;
-        struct bladeMezzDevInfo mezzDevices;
+        char *mezzNumber;
+        xmlNode *mezzSlots;
+        xmlNode *mezzDevices;
         xmlNode *extraData;             /* Items are struct extraDataInfo */
 };
 
 struct bladePortMap
 {
-        byte bladeBayNumber;
+        char *bladeBayNumber;
         enum portMapStatus status;
         enum bladeSizeType bladeSizeType;
-        byte numberOfMezzes;
-        int sizemezz;
-        struct bladeMezzInfo mezz;
+        char *numberOfMezzes;
+        xmlNode *mezz;
         xmlNode *extraData;             /* Items are struct extraDataInfo */
 };
 
@@ -1287,7 +1411,7 @@ struct interconnectTraySlotInfo
         byte interconnectTraySlotNumber;
         enum interconnectTrayType type;
         int sizeport;
-        struct interconnectTrayPortInfo port;
+        xmlNode *port;
         xmlNode *extraData;             /* Items are struct extraDataInfo */
 };
 
@@ -1299,7 +1423,7 @@ struct interconnectTrayPortMap
         enum interconnectTrayPassThroughEnabled passThroughModeEnabled;
         byte numberOfSlots;
         int sizeslot;
-        struct interconnectTraySlotInfo slot;
+        xmlNode *slot;
         xmlNode *extraData;             /* Items are struct extraDataInfo */
 };
 
@@ -1309,6 +1433,7 @@ struct powerSupplyInfo
         enum presence presence;
         char modelNumber[MAX_MODEL_NUM_LENGTH];
         char sparePartNumber[MAX_PART_NUM_LENGTH];
+        char productName[MAX_PRODUCT_NAME_LENGTH];
         char serialNumber[MAX_SERIAL_NUM_LENGTH];
         int capacity;
         int actualOutput;
@@ -1533,6 +1658,14 @@ struct oaNetworkInfo
         enum hpoa_boolean dynDnsEnabled;
         char *macAddress;
         char *ipAddress;
+        char ipv6Address0[255];
+        char ipv6Address1[255];
+        char ipv6Address2[255];
+        char ipv6Address3[255];
+        char *ipv6AddressType0;
+        char *ipv6AddressType1;
+        char *ipv6AddressType2;
+        char *ipv6AddressType3;
         char *netmask;
         char *gateway;
         xmlNode *dns;                   /* Items are char *ipAddress */
@@ -1861,6 +1994,58 @@ struct getFanZoneArrayResponse
         xmlNode *fanZoneArray;          /* Items are struct fanZone */
 };
 
+struct getBladeInfoArrayResponse
+{
+        xmlNode *bladeInfoArray;       /* Items are struct bladeInfo */
+};
+struct interconnectTrayInfoArrayResponse
+{
+        xmlNode *interconnectTrayInfoArray;
+};
+struct interconnectTrayStsArrayResponse
+{
+        xmlNode *interconnectTrayStsArray;
+};
+struct interconnectTrayPmArrayResponse
+{
+        xmlNode *interconnectTrayPmArray;
+};
+
+struct getPowerSupplyInfoArrayResponse
+{
+        xmlNode *powerSupplyInfoArray;
+};
+
+struct getPowerSupplyStsArrayResponse
+{
+        xmlNode *powerSupplyStsArray;
+};
+
+struct getFanInfoArrayResponse
+{
+        xmlNode *fanInfoArray;
+};
+
+struct getOaInfoArrayResponse
+{
+        xmlNode *oaInfoArray;
+};
+
+struct getOaStatusArrayResponse
+{
+        xmlNode *oaStatusArray;
+};
+
+struct getBladePortMapArrayResponse
+{
+        xmlNode *portMapArray;
+};
+
+struct getBladeStsArrayResponse
+{
+        xmlNode *bladeStsArray;
+};
+
 struct bayArray
 {
         int     size;                   /* Size of array */
@@ -1872,6 +2057,55 @@ struct getFanZoneArray
         struct bayArray bayArray;
 };
 
+struct getBladeInfoArray
+{
+        struct bayArray bayArray;
+};
+
+struct getInterconnectTrayInfoArray
+{
+        struct bayArray bayArray;
+};
+struct interconnectTrayStsArray
+{
+        struct bayArray bayArray;
+};
+struct interconnectTrayPmArray
+{
+        struct bayArray bayArray;
+};
+
+struct getPowerSupplyInfoArray
+{
+        struct bayArray bayArray;
+};
+
+struct getPowerSupplyStsArray
+{
+        struct bayArray bayArray;
+};
+
+struct getFanInfoArray
+{
+        struct bayArray bayArray;
+};
+struct getOaInfoArray
+{
+        struct bayArray bayArray;
+};
+struct getOaStatusArray
+{
+        struct bayArray bayArray;
+};
+struct getBladePortMapArray
+{
+        struct bayArray bayArray;
+};
+struct getBladeStsArray
+{
+        struct bayArray bayArray;
+};
+
 struct getEvent
 {
         int pid;
@@ -1879,11 +2113,12 @@ struct getEvent
         enum hpoa_boolean lcdEvents;
 };
 
-struct getAllEvents
+struct getAllEventsEx
 {
         int pid;
         enum hpoa_boolean waitTilEventHappens;
         enum hpoa_boolean lcdEvents;
+        char *oaFwVersion;
 };
 
 union _hpoa__data
@@ -2037,14 +2272,18 @@ int soap_getEvent(SOAP_CON *connection,
                   const struct getEvent *request,
                   struct eventInfo *response);
 
-int soap_getAllEvents(SOAP_CON *connection,
-                      const struct getAllEvents *request,
+int soap_getAllEventsEx(SOAP_CON *connection,
+                      const struct getAllEventsEx *request,
                       struct getAllEventsResponse *response);
 
 int soap_getBladeInfo(SOAP_CON *connection,
                       const struct getBladeInfo *request,
                       struct bladeInfo *response);
 
+int soap_getBladePortMap(SOAP_CON *connection,
+                      const struct getBladeInfo *request,
+                      struct bladePortMap *response);
+
 int soap_getBladeMpInfo(SOAP_CON *connection,
                         const struct getBladeMpInfo *request,
                         struct bladeMpInfo *response);
@@ -2076,6 +2315,9 @@ int soap_getOaInfo(SOAP_CON *connection,
                    const struct getOaInfo *request,
                    struct oaInfo *response);
 
+int soap_getOaId(SOAP_CON *connection,
+                 struct OaId *oaId);
+
 int soap_getInterconnectTrayStatus(SOAP_CON *connection,
                 const struct getInterconnectTrayStatus *request,
                 struct interconnectTrayStatus *response);
@@ -2165,12 +2407,84 @@ int soap_getBladeThermalInfoArray(SOAP_CON *con,
 				  struct bladeThermalInfoArrayResponse 
 								*response);
 
+int soap_getInterconnectTrayPortMap(SOAP_CON *con,
+                                 const struct getInterconnectTrayInfo *request,
+                                 struct interconnectTrayPortMap *response);
+
+int soap_getPowerSupplyInfoArray(SOAP_CON *con,
+                         const struct getPowerSupplyInfoArray *request,
+                         struct getPowerSupplyInfoArrayResponse *response,
+                         xmlDocPtr ps_info_doc);
+
+int soap_getPowerSupplyStatusArray(SOAP_CON *con,
+                         const struct getPowerSupplyStsArray *request,
+                         struct getPowerSupplyStsArrayResponse *response,
+                         xmlDocPtr ps_sts_doc);
+
+int soap_getFanInfoArray(SOAP_CON *con,
+                         const struct getFanInfoArray *request,
+                         struct getFanInfoArrayResponse *response,
+                         xmlDocPtr fan_info_doc);
+
+int soap_getBladeInfoArray(SOAP_CON *con,
+                         const struct getBladeInfoArray *request,
+                         struct getBladeInfoArrayResponse *response,
+                         xmlDocPtr bl_info_doc);
+
+int soap_getBladeStatusArray(SOAP_CON *con,
+                         const struct getBladeStsArray *request,
+                         struct getBladeStsArrayResponse *response,
+                         xmlDocPtr bl_sts_doc);
+
+int soap_getBladePortMapArray(SOAP_CON *con,
+                         const struct getBladePortMapArray *request,
+                         struct getBladePortMapArrayResponse *response,
+                         xmlDocPtr bl_pm_doc);
+
+int soap_getInterconnectTrayInfoArray(SOAP_CON *con,
+                            const struct getInterconnectTrayInfoArray *request,
+                            struct interconnectTrayInfoArrayResponse *response,
+                            xmlDocPtr intr_info_doc);
+
+int soap_getInterconnectTrayStatusArray(SOAP_CON *con,
+                            const struct interconnectTrayStsArray *request,
+                            struct interconnectTrayStsArrayResponse *response,
+                            xmlDocPtr intr_sts_doc);
+
+int soap_getInterconnectTrayPortMapArray(SOAP_CON *con,
+                            const struct interconnectTrayPmArray *request,
+                            struct interconnectTrayPmArrayResponse *response,
+                            xmlDocPtr intr_pm_doc);
+
+int soap_getOaInfoArray(SOAP_CON *con,
+                         const struct getOaInfoArray *request,
+                         struct getOaInfoArrayResponse *response,
+                         xmlDocPtr oa_info_doc);
+
+int soap_getOaStatusArray(SOAP_CON *con,
+                         const struct getOaStatusArray *request,
+                         struct getOaStatusArrayResponse *response,
+                         xmlDocPtr oa_sts_doc);
+
 /* Function prototypes for OA SOAP helper functions */
 void    soap_getExtraData(xmlNode *extraData, struct extraDataInfo *result);
 void    soap_getDiagnosticChecksEx(xmlNode *diag,
                                    struct diagnosticData *result);
 void    soap_getBladeCpuInfo(xmlNode *cpus, struct bladeCpuInfo *result);
 void    soap_getBladeNicInfo(xmlNode *nics, struct bladeNicInfo *result);
+void    soap_getBladeMezzInfo(xmlNode *mezz, struct bladeMezzInfo *result);
+void    soap_getBladeMezzDevInfo(xmlNode *mezzDevices,
+				 struct bladeMezzDevInfo *result);
+void    soap_getBladeMezzSlotInfo(xmlNode *mezzSlots,
+				  struct bladeMezzSlotInfo *result);
+void    soap_getBladeMezzSlotPort(xmlNode *slot,
+				  struct bladeMezzSlotPort *result);
+void    soap_getBladeMezzDevPort(xmlNode *port,
+				 struct bladeMezzDevPort *result);
+void    soap_getInterconnectTraySlotInfo(xmlNode *slot,
+				struct interconnectTraySlotInfo *result);
+void    soap_getInterconnectTrayPortInfo(xmlNode *port,
+				 struct interconnectTrayPortInfo *result);
 void    soap_getDiagnosticData(xmlNode *data, struct diagnosticData *result);
 void    soap_getBayAccess(xmlNode *bay, struct bayAccess *result);
 void    soap_getEncLink(xmlNode *data, struct encLink *result);
@@ -2184,4 +2498,17 @@ void    soap_fanInfo(xmlNode *fanZone, struct fanInfo *result);
 void    soap_deviceBayArray(xmlNode *node, byte *bay);
 void	soap_bladeThermalInfo(xmlNode *node, struct bladeThermalInfo *result);
 
+void    parse_powerSupplyInfo(xmlNode *node,struct powerSupplyInfo *response);
+void    parse_powerSupplyStatus(xmlNode *node,
+                                struct powerSupplyStatus *response);
+void    parse_bladeInfo(xmlNode *node, struct bladeInfo *response);
+void    parse_interconnectTrayInfo(xmlNode *node,struct interconnectTrayInfo *);
+void    parse_interconnectTrayStatus(xmlNode *node,
+                                     struct interconnectTrayStatus *);
+void    parse_oaInfo(xmlNode *node, struct oaInfo *response);
+void    parse_oaStatus(xmlNode *node, struct oaStatus *response);
+void    parse_bladeStatus(xmlNode *node, struct bladeStatus *response);
+void    parse_bladePortMap(xmlNode *node, struct bladePortMap *response);
+void    parse_interconnectTrayPortMap(xmlNode *portmap,
+					struct interconnectTrayPortMap *result);
 #endif  /* _INC_OASOAP_CALLS_H_ */
