@@ -91,6 +91,7 @@
 #include <glib.h>
 #include <oh_error.h>
 #include "oa_soap_callsupport.h"
+#include "sahpi_wrappers.h"
 
 /* Include file check */
 #ifndef LIBXML_PUSH_ENABLED
@@ -471,7 +472,7 @@ SOAP_CON        *soap_open(char *server,
                            char *username, char *password,
                            long timeout)
 {
-        SOAP_CON        *connection;
+        SOAP_CON        *connection = NULL;
 
         /* Basic pass parameter checking */
         if ((server == NULL) || (*server == '\0')) {
@@ -479,6 +480,11 @@ SOAP_CON        *soap_open(char *server,
                 return(NULL);
         }
 
+	if (!strcmp(server,"0.0.0.0:443")) {
+                err("Invalid OA IP 0.0.0.0.");
+                return(NULL);
+        }
+
         if ((username == NULL) || (*username == '\0')) {
                 err("missing OA username");
                 return(NULL);
@@ -503,7 +509,7 @@ SOAP_CON        *soap_open(char *server,
 
 
         /* Get a new OA SOAP connection structure and initialize it */
-        connection = g_malloc(sizeof(SOAP_CON)); /* New connection */
+        connection = g_malloc0(sizeof(SOAP_CON)); /* New connection */
         if (! connection) {
                 err("out of memory");
                 return(NULL);
@@ -511,6 +517,9 @@ SOAP_CON        *soap_open(char *server,
         strncpy(connection->server, server, OA_SOAP_SERVER_SIZE);
         strncpy(connection->username, username, OA_SOAP_USER_SIZE);
         strncpy(connection->password, password, OA_SOAP_USER_SIZE);
+        connection->server[OA_SOAP_SERVER_SIZE] = '\0';
+        connection->username[OA_SOAP_USER_SIZE] = '\0';
+        connection->password[OA_SOAP_USER_SIZE] = '\0';
         connection->timeout = timeout;
         connection->session_id[0] = '\0';
         connection->doc = NULL;
@@ -588,7 +597,7 @@ void            soap_close(SOAP_CON *connection)
             connection->req_high_water,
             OA_SOAP_REQ_BUFFER_SIZE);
 
-        g_free(connection);
+        wrap_g_free(connection);
 
         /* Note that soap_open() initialized the SSL library, but
          * soap_close() does nothing to clean up and free SSL objects.
@@ -620,11 +629,11 @@ void            soap_close(SOAP_CON *connection)
 static int      soap_message(SOAP_CON *connection, char *request,
                              xmlDocPtr *doc)
 {
-        int             nbytes;
-        int             ret;
+        int             nbytes = 0;
+        int             ret = 0;
         char *          header=NULL;                              
         char            response[OA_SOAP_RESP_BUFFER_SIZE];
-        xmlParserCtxtPtr parse;
+        xmlParserCtxtPtr parse = NULL;
 
         /* Error checking */
         if (! connection) {
@@ -668,9 +677,11 @@ static int      soap_message(SOAP_CON *connection, char *request,
                          connection->timeout)) {
                 (void) oh_ssl_disconnect(connection->bio, OH_SSL_BI);
                 err("oh_ssl_write() failed");
+                free(header);
                 return(-1);
         }
 
+        free(header);
         /* Write request to server */
         dbg("OA request(2):\n%s\n", request);
         if (oh_ssl_write(connection->bio, request, nbytes,
@@ -772,7 +783,6 @@ static int      soap_message(SOAP_CON *connection, char *request,
                 xmlFreeParserCtxt(parse);
                 return(-1);
         }
-	free(header);
         xmlFreeParserCtxt(parse);
         return(0);
 }
@@ -830,8 +840,10 @@ static int      soap_login(SOAP_CON *connection)
         /* Perform login request */
         if (soap_message(connection, buf, &doc)) {
                 err("failed to communicate with OA during login");
+                free(buf);
                 return(-1);
         }
+        free(buf);
 
 
         /* Parse looking for session ID.
@@ -844,9 +856,9 @@ static int      soap_login(SOAP_CON *connection)
         if ((sess_id = soap_value(login_element))) {
                 strncpy(connection->session_id, sess_id,
                         OA_SOAP_SESSIONKEY_SIZE);
+                connection->session_id[OA_SOAP_SESSIONKEY_SIZE] = '\0';
                 dbg("Opened session ID %s", connection->session_id);
                 /* Free the XML document */
-                free(buf);
                 xmlFreeDoc(doc);
                 return(0);              /* Normal, successful return */
         }
@@ -867,7 +879,6 @@ static int      soap_login(SOAP_CON *connection)
         else {
                 err("failed to find session ID during OA login");
         }
-        free(buf);
         xmlFreeDoc(doc);
         return(-1);
 }
@@ -1074,6 +1085,7 @@ int             soap_call(SOAP_CON *connection)
                         /* Substitute the current session key */
                         strncpy(session_pos, connection->session_id,
                                 OA_SOAP_SESSIONKEY_SIZE);
+                        connection->session_id[OA_SOAP_SESSIONKEY_SIZE] = '\0';
 
                         /* Perform SOAP call */
                         err = soap_message(connection,
