@@ -54,6 +54,8 @@
  **/
 
 #include "oa_soap_event.h"
+#include "sahpi_wrappers.h"
+#include <sys/time.h>
 
 /**
  * oa_soap_get_event
@@ -100,7 +102,7 @@ int oa_soap_get_event(void *oh_handler)
 gpointer oa_soap_event_thread(gpointer oa_pointer)
 {
         SaErrorT rv = SA_OK;
-        struct getAllEvents request;
+	 struct getAllEventsEx request;
         struct getAllEventsResponse response;
         struct oh_handler_state *handler = NULL;
         struct oa_info *oa = NULL;
@@ -111,6 +113,8 @@ gpointer oa_soap_event_thread(gpointer oa_pointer)
         SaHpiBoolT is_discovery_completed = SAHPI_FALSE;
         SaHpiBoolT listen_for_events = SAHPI_TRUE;
         char *user_name, *password, *url = NULL;  
+        struct timeval time1 = {0}, time2 = {0};
+	char oa_fw_buf[SAHPI_MAX_TEXT_BUFFER_LENGTH];
 
         if (oa_pointer == NULL) {
                 err("Invalid parameter");
@@ -122,20 +126,30 @@ gpointer oa_soap_event_thread(gpointer oa_pointer)
 	handler = oa->oh_handler;
 	oa_handler = handler->data;
 
-        dbg("OA SOAP event thread started for OA %s", oa->server);
+        dbg("Threadid= %p OA SOAP event thread started for OA %s", 
+                     g_thread_self(), oa->server);
+
+        /* subscribe to the events here. Use the same session if the
+           session is not expired.
+         */
+        rv = create_event_session(oa);
+        if (rv != SOAP_OK) {
+                err("Subscribe for events failed OA %s", oa->server);
+        }
+                gettimeofday(&time1, NULL);
 
         /* Check whether the plugin is initialized.
          * If not, wait till plugin gets initialized
          */
         while (is_plugin_initialized == SAHPI_FALSE) {
         	OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, NULL, NULL, NULL);
-                g_mutex_lock(oa_handler->mutex);
+                wrap_g_mutex_lock(oa_handler->mutex);
                 if (oa_handler->status == PRE_DISCOVERY ||
                     oa_handler->status == DISCOVERY_COMPLETED) {
-                        g_mutex_unlock(oa_handler->mutex);
+                        wrap_g_mutex_unlock(oa_handler->mutex);
                         is_plugin_initialized = SAHPI_TRUE;
                 } else {
-                        g_mutex_unlock(oa_handler->mutex);
+                        wrap_g_mutex_unlock(oa_handler->mutex);
                         dbg("Waiting for the plugin initialization "
                             "to complete.");
                         sleep(2);
@@ -147,12 +161,12 @@ gpointer oa_soap_event_thread(gpointer oa_pointer)
          */
         while (is_discovery_completed == SAHPI_FALSE) {
         	OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, NULL, NULL, NULL);
-                g_mutex_lock(oa_handler->mutex);
+                wrap_g_mutex_lock(oa_handler->mutex);
                 if (oa_handler->status == DISCOVERY_COMPLETED) {
-                        g_mutex_unlock(oa_handler->mutex);
+                        wrap_g_mutex_unlock(oa_handler->mutex);
                         is_discovery_completed = SAHPI_TRUE;
                 } else {
-                        g_mutex_unlock(oa_handler->mutex);
+                        wrap_g_mutex_unlock(oa_handler->mutex);
                         dbg("Waiting for the discovery to complete.");
                         sleep(2);
                 }
@@ -175,11 +189,11 @@ gpointer oa_soap_event_thread(gpointer oa_pointer)
         /* Check whether OA Status is ABSENT
          * If yes, wait till the OA status becomes ACTIVE or STANDBY
          */
-        g_mutex_lock(oa->mutex);
+        wrap_g_mutex_lock(oa->mutex);
         if (oa->oa_status != OA_ABSENT) {
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
         } else {
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
                 process_oa_out_of_access(handler, oa);
         }
 
@@ -222,16 +236,35 @@ gpointer oa_soap_event_thread(gpointer oa_pointer)
                 if (oa->event_con2 == NULL) 
                         sleep(2);
         }
+        free(url);
+        url = NULL;
+
+        gettimeofday(&time2, NULL);
+        if (time2.tv_sec-time1.tv_sec > SUBSCRIBE_TIMEOUT) {
+              rv = create_event_session(oa);
+              if (rv != SOAP_OK) {
+                       err("Subscribe for events failed OA %s",
+                              oa->server);
+              } else {
+                        warn("Re-discovery took %ld secs.",
+                                      (time2.tv_sec-time1.tv_sec));
+                        warn("Events might have been lost");
+              } 
+        }
 
         /* Intialize the event request structure */
         request.pid = oa->event_pid;
         request.waitTilEventHappens = HPOA_TRUE;
         request.lcdEvents = HPOA_FALSE;
+        memset(oa_fw_buf,0,SAHPI_MAX_TEXT_BUFFER_LENGTH);
+        snprintf(oa_fw_buf,SAHPI_MAX_TEXT_BUFFER_LENGTH,"%.2f",oa->fm_version);
+        request.oaFwVersion = oa_fw_buf;
 
         /* Listen for the events from OA */
         while (listen_for_events == SAHPI_TRUE) {
+                request.pid = oa->event_pid;
         	OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, NULL, NULL, NULL);
-                rv = soap_getAllEvents(oa->event_con, &request, &response);
+                rv = soap_getAllEventsEx(oa->event_con, &request, &response);
                 if (rv == SOAP_OK) {
                         retry_on_switchover = 0;
                         /* OA returns empty event response payload for LCD
@@ -248,17 +281,26 @@ gpointer oa_soap_event_thread(gpointer oa_pointer)
                          * 2.21. Re-try the getAllEvents SOAP XML call skipping
                          * the error handling.
                          */
+                         
+                         /* If Enclosure IP Mode is enabled, 
+                          * then make the Standby thread to sleep */ 
+                         while (oa_handler->ipswap && (oa->oa_status == STANDBY)) {
+        			OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, NULL, NULL, NULL);
+                                 dbg("Stand By Thread is going to Sleep for"
+                                     "20 secs as Enclosure IP Mode Is enabled");
+                                 oa_soap_sleep_in_loop(oa_handler, 20);
+                         }
                         if (oa->oa_status == STANDBY &&
                             get_oa_fw_version(handler) >= OA_2_21 &&
                             retry_on_switchover < MAX_RETRY_ON_SWITCHOVER) {
-                                sleep(WAIT_ON_SWITCHOVER);
-                                dbg("getAllEvents call failed, may be due to "
+                                oa_soap_sleep_in_loop(oa_handler, WAIT_ON_SWITCHOVER);
+                                dbg("getAllEventsEx call failed, may be due to "
                                     "OA switchover");
-                                dbg("Re-try the getAllEvents SOAP call");
+                                dbg("Re-try the getAllEventsEx SOAP call");
                                 retry_on_switchover++;
                         } else {
                                 /* Try to recover from the error */
-                                err("OA %s may not be accessible", oa->server);
+                                dbg("OA %s may not be accessible", oa->server);
                                 oa_soap_error_handling(handler, oa);
                                 request.pid = oa->event_pid;
 
@@ -291,18 +333,22 @@ gpointer oa_soap_event_thread(gpointer oa_pointer)
 							  HPI_CALL_TIMEOUT);
                                         if (oa->event_con2 == NULL) {
                                                 if (oa->oa_status == OA_ABSENT)
-                                                          sleep(60);
+                                                          oa_soap_sleep_in_loop(
+                                                                 oa_handler,60);
                                                 else
-                                                          sleep(5);
+                                                          oa_soap_sleep_in_loop(
+                                                                 oa_handler,5);
                                                 err("soap_open for \
-oa->event_con2 failed\n");
+                                                        oa->event_con2 failed");
                                         }
                                 }
+                                free(url);
+                                url = NULL;
                         } /* end of else (non-switchover error handling) */
                 } /* end of else (SOAP call failure handling) */
 
         } /* end of 'while(listen_for_events == SAHPI_TRUE)' loop */
-	free(url);
+        free(url);
         return (gpointer *) SA_OK;
 }
 
@@ -326,11 +372,13 @@ void oa_soap_error_handling(struct oh_handler_state *oh_handler,
                             struct oa_info *oa)
 {
         SaErrorT rv = SA_OK;
-        int is_switchover = SAHPI_FALSE;
         SaHpiBoolT is_oa_accessible = SAHPI_FALSE;
         struct oa_soap_handler *oa_handler = NULL;
         SaHpiInt32T error_code;
         char *user_name = NULL, *password = NULL;
+        struct OaId oaId;
+        SaHpiResourceIdT resource_id;
+        struct timeval time1 = {0}, time2 = {0};
 
         if (oh_handler == NULL || oa == NULL) {
                 err("Invalid parameters");
@@ -338,15 +386,16 @@ void oa_soap_error_handling(struct oh_handler_state *oh_handler,
         }
 
         oa_handler = (struct oa_soap_handler *) oh_handler->data;
+        rv = check_oa_status(oa_handler, oa, oa->event_con);
 
         /* If the OA is not PRESENT, then do not even try. Just get out */
         if ( oa->oa_status == OA_ABSENT ) 
                 return;
 
         /* Check whether OA was present. If not, event_con will be NULL */
-        g_mutex_lock(oa->mutex);
+        wrap_g_mutex_lock(oa->mutex);
         if (oa->event_con == NULL) {
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
                 /* Get the user_name and password from config file */
                 user_name = (char *) g_hash_table_lookup(oh_handler->config,
                                                          "OA_User_Name");
@@ -360,7 +409,7 @@ void oa_soap_error_handling(struct oh_handler_state *oh_handler,
                 error_code = SOAP_OK;
         } else {
                 error_code = soap_error_number(oa->event_con);
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
         }
 
         /* This loop ends when the OA is accessible */
@@ -378,7 +427,6 @@ void oa_soap_error_handling(struct oh_handler_state *oh_handler,
                         process_oa_out_of_access(oh_handler, oa);
                 }
 
-                /* Create a fresh event session */
                 rv = create_event_session(oa);
                 if (rv != SA_OK) {
                         /* Set the error code to  -1 to make sure
@@ -386,7 +434,8 @@ void oa_soap_error_handling(struct oh_handler_state *oh_handler,
                          */
                         error_code = -1;
                         continue;
-                }
+                } else 
+                       gettimeofday(&time1, NULL);
 
                 /* Sleep for a second, let OA stabilize
                  * TODO: Remove this workaround, when OA has the fix
@@ -398,17 +447,16 @@ void oa_soap_error_handling(struct oh_handler_state *oh_handler,
                        /* Always lock the oa_handler mutex and then oa_info
                         * mutex.  This is to avoid the deadlock.
                         */
-                        g_mutex_lock(oa_handler->mutex);
-                        g_mutex_lock(oa->mutex);
+                        wrap_g_mutex_lock(oa_handler->mutex);
+                        wrap_g_mutex_lock(oa->mutex);
                         /* Re-discover the resources as there is a high chances
                          * that we might have missed some events
                          */
                 	OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, oa_handler->mutex,
 						  oa->mutex, NULL);
-                        rv = oa_soap_re_discover_resources(oh_handler, oa,
-                                                           is_switchover);
-                        g_mutex_unlock(oa->mutex);
-                        g_mutex_unlock(oa_handler->mutex);
+                        rv = oa_soap_re_discover_resources(oh_handler, oa);
+                        wrap_g_mutex_unlock(oa->mutex);
+                        wrap_g_mutex_unlock(oa_handler->mutex);
                         if (rv != SA_OK) {
                                 is_oa_accessible = SAHPI_FALSE;
                                 err("Re-discovery failed for OA %s",
@@ -420,9 +468,37 @@ void oa_soap_error_handling(struct oh_handler_state *oh_handler,
                                 error_code = -1;
                         }
                 }
+                /* Create a fresh event session, PID  expires in 5 mins.  */
+                gettimeofday(&time2, NULL);
+                
+                if (time2.tv_sec-time1.tv_sec > SUBSCRIBE_TIMEOUT) {
+                        rv = create_event_session(oa);
+                        if (rv != SA_OK) {
+                                 /* Set the error code to  -1 to make sure
+                                    * recovery for OA out of access is done
+                                    */
+                                 err("create_event_session failed");
+                                 error_code = -1;
+                                 continue;
+                        } else {
+                                 err("Re-discovery took %ld secs.",
+                                      (time2.tv_sec-time1.tv_sec));
+                                 err("Events might have been lost");
+                        }
+                }
         }
 
         err("OA %s is accessible", oa->server);
+        /* Push the OA Link Status event*/
+        rv = soap_getOaId(oa->event_con, &oaId);
+        if(rv == SA_OK) {
+             resource_id = oa_handler->oa_soap_resources.oa.
+                                       resource_id[oaId.bayNumber - 1];
+             /* Process the OA link status sensor */
+             OA_SOAP_PROCESS_SENSOR_EVENT(
+                   OA_SOAP_SEN_OA_LINK_STATUS, 1,
+                            0, 0)
+        }
         return;
 }
 
@@ -479,13 +555,13 @@ void process_oa_out_of_access(struct oh_handler_state *oh_handler,
                 while (is_oa_present == SAHPI_FALSE) {
                 	OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, NULL, NULL,
 						  timer);
-                        g_mutex_lock(oa->mutex);
+                        wrap_g_mutex_lock(oa->mutex);
                         if (oa->oa_status != OA_ABSENT) {
-                                g_mutex_unlock(oa->mutex);
+                                wrap_g_mutex_unlock(oa->mutex);
                                 is_oa_present = SAHPI_TRUE;
                                 time_elapsed = 0.0;
                         } else {
-                                g_mutex_unlock(oa->mutex);
+                                wrap_g_mutex_unlock(oa->mutex);
                                 time_elapsed = g_timer_elapsed(timer,
                                                                &micro_seconds);
                                 /* Break the loop on reaching timeout value */
@@ -496,7 +572,7 @@ void process_oa_out_of_access(struct oh_handler_state *oh_handler,
                                 /* OA is not present,
                                  * wait for 30 seconds and check again
                                  */
-                                sleep(30);
+                                oa_soap_sleep_in_loop(oa_handler, 30);
                         }
                 }
 
@@ -566,7 +642,7 @@ void process_oa_out_of_access(struct oh_handler_state *oh_handler,
                                 /* longer                                    */
                                 if ((oa_handler->oa_switching == SAHPI_TRUE) ||
                                     (oa->oa_status == OA_ABSENT)) 
-                                        sleep(30);
+                                        oa_soap_sleep_in_loop(oa_handler, 30);
                                 else
                                         sleep(2);
                                 dbg("check_oa_status failed, oa_status is %d\n",
@@ -611,7 +687,6 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                        struct oa_info *oa,
                        struct getAllEventsResponse *response)
 {
-        SaErrorT rv;
         SaHpiInt32T loc=0;
         struct eventInfo event;
         struct oa_soap_handler *oa_handler = NULL;
@@ -628,6 +703,8 @@ void process_oa_events(struct oh_handler_state *oh_handler,
         	OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, NULL, NULL, NULL);
                 /* Get the event from eventInfoArray */
                 soap_getEventInfo(response->eventInfoArray, &event);
+		dbg("\nThread id=%p event %d received\n",
+				g_thread_self(), event.event);
                 switch (event.event) {
                         case EVENT_HEARTBEAT:
                                 dbg("HEART BEAT EVENT");
@@ -674,14 +751,14 @@ void process_oa_events(struct oh_handler_state *oh_handler,
 
                         case EVENT_FAN_INSERTED:
                                 dbg("EVENT_FAN_INSERTED");
-                                rv = process_fan_insertion_event(oh_handler,
+                                process_fan_insertion_event(oh_handler,
                                                                  oa->event_con2,
                                                                  &event);
                                 break;
 
                         case EVENT_FAN_REMOVED:
                                 dbg("EVENT_FAN_REMOVED");
-                                rv = process_fan_extraction_event(oh_handler,
+                                process_fan_extraction_event(oh_handler,
                                                                   &event);
                                 break;
 
@@ -689,7 +766,10 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 dbg("EVENT_FAN_GROUP_STATUS -- Not processed");
                                 break;
                         case EVENT_THERMAL_STATUS:
-                                dbg("EVENT_THERMAL_STATUS -- Not processed");
+                                dbg("EVENT_THERMAL_STATUS");
+				oa_soap_proc_enc_thermal(oh_handler,
+                                       oa->event_con2,
+                                       &(event.eventData.thermalInfo));
                                 break;
                         case EVENT_COOLING_STATUS:
 				dbg("EVENT_COOLING_STATUS");
@@ -708,14 +788,14 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 break;
                         case EVENT_PS_INSERTED:
                                 dbg("EVENT_PS_INSERTED");
-                                rv = process_ps_insertion_event(oh_handler,
+                                process_ps_insertion_event(oh_handler,
                                                                 oa->event_con2,
                                                                 &event);
                                 break;
 
                         case EVENT_PS_REMOVED:
                                 dbg("EVENT_PS_REMOVED");
-                                rv = process_ps_extraction_event(oh_handler,
+                                process_ps_extraction_event(oh_handler,
                                                                  &event);
                                 break;
 
@@ -733,7 +813,9 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 dbg("EVENT_AC_FAILURE -- Not processed");
                                 break;
                         case EVENT_PS_INFO:
-                                dbg("EVENT_PS_INFO -- Not processed");
+                                dbg("EVENT_PS_INFO");
+                                oa_soap_proc_ps_info(oh_handler, 
+                                        oa->event_con2, &event);
                                 break;
                         case EVENT_PS_SUBSYSTEM_STATUS:
 				dbg("EVENT_PS_SUBSYSTEM_STATUS");
@@ -752,7 +834,7 @@ void process_oa_events(struct oh_handler_state *oh_handler,
 
                         case EVENT_INTERCONNECT_RESET:
                                 dbg("EVENT_INTERCONNECT_RESET");
-                                rv = process_interconnect_reset_event(
+                                process_interconnect_reset_event(
                                         oh_handler, &event);
                                 break;
                         case EVENT_INTERCONNECT_UID:
@@ -760,19 +842,19 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 break;
                         case EVENT_INTERCONNECT_INSERTED:
                                 dbg("EVENT_INTERCONNECT_INSERTED");
-                                rv = process_interconnect_insertion_event(
+                                process_interconnect_insertion_event(
                                         oh_handler, oa->event_con2, &event);
                                 break;
 
                         case EVENT_INTERCONNECT_REMOVED:
                                 dbg("EVENT_INTERCONNECT_REMOVED");
-                                rv = process_interconnect_extraction_event(
+                                process_interconnect_extraction_event(
                                         oh_handler, &event);
                                 break;
 
                         case EVENT_INTERCONNECT_INFO:
                                 dbg("EVENT_INTERCONNECT_INFO");
-                                rv = process_interconnect_info_event(
+                                process_interconnect_info_event(
                                         oh_handler, oa->event_con2, &event);
                                 break;
                         case EVENT_INTERCONNECT_HEALTH_LED:
@@ -791,7 +873,7 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 break;
                         case EVENT_INTERCONNECT_POWER:
                                 dbg("EVENT_INTERCONNECT_POWER");
-                                rv = process_interconnect_power_event(
+                                process_interconnect_power_event(
                                         oh_handler, &event);
                                 break;
                         case EVENT_INTERCONNECT_PORTMAP:
@@ -821,12 +903,15 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 break;
 
                         case EVENT_BLADE_INSERTED:
-                                dbg("EVENT_BLADE_INSERTED -- Not processed");
+                                dbg("EVENT_BLADE_INSERTED");
+                                oa_soap_proc_server_inserted_event(oh_handler,
+                                                             oa->event_con2,
+                                                             &event);
                                 break;
 
                         case EVENT_BLADE_REMOVED:
                                 dbg("EVENT_BLADE_REMOVED");
-                                rv = process_server_extraction_event(oh_handler,
+                                process_server_extraction_event(oh_handler,
                                                                      &event);
                                 break;
 
@@ -855,11 +940,21 @@ void process_oa_events(struct oh_handler_state *oh_handler,
 						oa->event_con2,
 						&(event.eventData.bladeStatus));
                                 break;
+                        case EVENT_BLADE_THERMAL:
+                                dbg("EVENT_BLADE_THERMAL");
+                                oa_soap_proc_server_thermal(oh_handler,
+                                                oa->event_con2,
+                                                &(event.eventData.bladeStatus));
+                                break;
                         case EVENT_BLADE_INFO:
-                                dbg("EVENT_BLADE_INFO -- Not processed");
+                                dbg("EVENT_BLADE_INFO");
+				process_server_info_event(oh_handler, 
+						oa->event_con2, &event);
                                 break;
                         case EVENT_BLADE_MP_INFO:
-                                dbg("EVENT_BLADE_MP_INFO -- Not processed");
+                                dbg("EVENT_BLADE_MP_INFO");
+                                process_server_mp_info_event(oh_handler,
+                                                    oa->event_con2, &event);
                                 break;
                         case EVENT_ILO_READY:
                                 dbg("EVENT_ILO_READY -- Not processed");
@@ -963,24 +1058,26 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 break;
 
                         case EVENT_OA_INSERTED:
-                                dbg("EVENT_OA_INSERTED -- Not processed");
+				dbg("EVENT_OA_INSERTED");
+				oa_soap_proc_oa_inserted(oh_handler,
+					&event);
                                 break;
 
                         case EVENT_OA_REMOVED:
                                 dbg("EVENT_OA_REMOVED");
-                                rv = process_oa_extraction_event(oh_handler,
+                                process_oa_extraction_event(oh_handler,
                                                                  &event);
                                 break;
 
                         case EVENT_OA_INFO:
                                 dbg("EVENT_OA_INFO");
-                                rv = process_oa_info_event(oh_handler,
+                                process_oa_info_event(oh_handler,
                                                            oa->event_con2,
 							   &event);
                                 break;
                         case EVENT_OA_FAILOVER:
                                 dbg("EVENT_OA_FAILOVER");
-                                rv = process_oa_failover_event(oh_handler, oa);
+                                process_oa_failover_event(oh_handler, oa);
                                 /* We have done the re-discovery as part of
                                  * FAILOVER event processing.  Ignore the
                                  * events that are recived along with FAILOVER.
@@ -996,8 +1093,9 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 dbg("EVENT_OA_VCM -- Not processed");
                                 break;
                         case EVENT_NETWORK_INFO_CHANGED:
-                                dbg("EVENT_NETWORK_INFO_CHANGED "
-                                    "-- Not processed");
+                                dbg("EVENT_NETWORK_INFO_CHANGED");
+                                oa_soap_proc_enc_network_info_changed(oh_handler,
+ 				      &(event.eventData.enclosureNetworkInfo));
                                 break;
                         case EVENT_SNMP_INFO_CHANGED:
                                 dbg("EVENT_SNMP_INFO_CHANGED -- Not processed");
@@ -1065,7 +1163,9 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 dbg("EVENT_GROUP_CHANGED -- Not processed");
                                 break;
                         case EVENT_OA_REBOOT:
-                                dbg("EVENT_OA_REBOOT -- Not processed");
+                                dbg("EVENT_OA_REBOOT");
+                                process_oa_reboot_event(oh_handler, oa);
+                                response->eventInfoArray = NULL; 
                                 break;
                         case EVENT_OA_LOGOFF_REQUEST:
                                 dbg("EVENT_OA_LOGOFF_REQUEST -- Not processed");
@@ -1235,7 +1335,7 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 break;
                         case EVENT_BLADE_INSERT_COMPLETED:
                                 dbg("EVENT_BLADE_INSERT_COMPLETED");
-                                rv = process_server_insertion_event(oh_handler,
+                                process_server_insert_completed(oh_handler,
                                                                  oa->event_con2,
                                                                  &event, loc);
                                 break;
@@ -1243,6 +1343,23 @@ void process_oa_events(struct oh_handler_state *oh_handler,
                                 dbg("EVENT_EBIPA_INFO_CHANGED_EX "
                                     "-- Not processed");
                                 break;
+                        case EVENT_BLADE_FQDN_INFO_REFRESH:
+                                dbg("EVENT_BLADE_FQDN_INFO_REFRESH"
+                                    " -- Not processed");
+                                break;
+                        case EVENT_TRAY_FQDN_INFO_REFRESH:
+                                dbg("EVENT_TRAY_FQDN_INFO_REFRESH"
+                                    " -- Not processed");
+                                break;
+                        case EVENT_VCM_FQDN_INFO_REFRESH:
+                                dbg("EVENT_VCM_FQDN_INFO_REFRESH"
+                                    " -- Not processed");
+                                break;
+                        case EVENT_EBIPAV6_INFO_CHANGED_EX:
+                                dbg("EVENT_EBIPAV6_INFO_CHANGED_EX"
+                                    " -- Not processed");
+                                break;
+
                         default:
                                 dbg("EVENT NOT REGISTERED, Event id %d",
                                     event.event);
