@@ -60,6 +60,7 @@
  */
 
 #include "oa_soap_interconnect_event.h"
+#include "sahpi_wrappers.h"
 
 /**
  * process_interconnect_reset_event
@@ -224,7 +225,7 @@ SaErrorT process_interconnect_power_event(struct oh_handler_state *oh_handler,
                 oh_get_resource_data(oh_handler->rptcache,
                                      event.resource.ResourceId);
         if (hotswap_state == NULL) {
-                err("blade private info is NULL");
+                err("interconnect private info is NULL");
                 return SA_ERR_HPI_INTERNAL_ERROR;
         }
 
@@ -377,7 +378,8 @@ SaErrorT process_interconnect_insertion_event(struct oh_handler_state
         SaHpiInt32T bay_number;
         SaHpiResourceIdT resource_id;
 	GSList *asserted_sensors = NULL;
-	SaHpiRptEntryT *rpt;
+	SaHpiRptEntryT *rpt = NULL;
+	struct oa_soap_hotswap_state *hotswap_state = NULL;
 
         if (oh_handler == NULL || oa_event == NULL || con == NULL) {
                 err("Invalid parameters");
@@ -396,7 +398,7 @@ SaErrorT process_interconnect_insertion_event(struct oh_handler_state
         }
 
         /* Build the inserted interconnect RPT entry */
-        rv = build_interconnect_rpt(oh_handler, con, response.name,
+        rv = build_inserted_intr_rpt(oh_handler, con, response.name,
                                     bay_number, &resource_id, TRUE);
         if (rv != SA_OK) {
                 err("Failed to build the interconnect RPT");
@@ -411,8 +413,8 @@ SaErrorT process_interconnect_insertion_event(struct oh_handler_state
                  response.serialNumber, resource_id, RES_PRESENT);
 
         /* Build the inserted interconnect RDRs */
-        rv = build_interconnect_rdr(oh_handler, con,
-                                    bay_number, resource_id);
+        rv = build_inserted_interconnect_rdr(oh_handler, con,
+                                    bay_number, resource_id, TRUE);
         if (rv != SA_OK) {
                 err("Failed to build the interconnect RDR");
                 rv = oh_remove_resource(oh_handler->rptcache,
@@ -443,6 +445,36 @@ SaErrorT process_interconnect_insertion_event(struct oh_handler_state
                 SAHPI_HS_CAUSE_OPERATOR_INIT;
         oh_evt_queue_push(oh_handler->eventq, copy_oa_soap_event(&event));
 
+        /* Get the rpt entry of the server */
+        rpt = oh_get_resource_by_id(oh_handler->rptcache, resource_id);
+        if (rpt == NULL) {
+                err("resource RPT is NULL");
+                return 0;
+        }
+
+        hotswap_state = (struct oa_soap_hotswap_state *)
+                oh_get_resource_data(oh_handler->rptcache, resource_id);
+        if (hotswap_state == NULL) {
+                err("Failed to get hotswap state of server blade");
+                return 0;
+        }
+
+        hotswap_state->currentHsState = SAHPI_HS_STATE_ACTIVE;
+ 
+        update_hotswap_event(oh_handler, &event);
+        memcpy(&(event.resource), rpt, sizeof(SaHpiRptEntryT));
+        event.event.Source = event.resource.ResourceId;
+        event.event.EventDataUnion.HotSwapEvent.PreviousHotSwapState =
+                SAHPI_HS_STATE_INSERTION_PENDING;
+        event.event.EventDataUnion.HotSwapEvent.HotSwapState =
+                SAHPI_HS_STATE_ACTIVE;
+        /* INSERTION_PENDING to ACTIVE state change happened de
+         * to Auto policy of server blade
+         */
+        event.event.EventDataUnion.HotSwapEvent.CauseOfStateChange =
+                SAHPI_HS_CAUSE_AUTO_POLICY;
+        oh_evt_queue_push(oh_handler->eventq, copy_oa_soap_event(&event));
+
 	/* Raise the assert sensor events */
 	if (asserted_sensors) {
 	        rpt = oh_get_resource_by_id(oh_handler->rptcache, resource_id);
@@ -493,25 +525,30 @@ SaErrorT process_interconnect_info_event(struct oh_handler_state
 
         oa_handler = (struct oa_soap_handler *) oh_handler->data;
         bay_number = oa_event->eventData.interconnectTrayInfo.bayNumber;
+
+        if(!oa_event->eventData.interconnectTrayInfo.serialNumber){
+                err("Serial Number is NULL");
+                return SA_ERR_HPI_INVALID_PARAMS;
+        }
+
         len = strlen(oa_event->eventData.interconnectTrayInfo.serialNumber);
         serial_number = (char *)g_malloc0(sizeof(char) * len + 1);
         strcpy(serial_number, oa_event->eventData.interconnectTrayInfo.serialNumber);
         serial_number[len]='\0';
         if (strcmp(serial_number,"[Unknown]") == 0 )  {
-                g_free(serial_number);
+                wrap_g_free(serial_number);
                 return SA_ERR_HPI_OUT_OF_MEMORY;
         }
         name = oa_event->eventData.interconnectTrayInfo.name;
  
          resource_id = oa_handler->
                 oa_soap_resources.interconnect.resource_id[bay_number - 1];
-	
         /* Build the inserted interconnect RPT entry */
-        rv = build_interconnect_rpt(oh_handler, con, name,
+        rv = build_inserted_intr_rpt(oh_handler, con, name,
                                     bay_number, &resource_id, TRUE);
         if (rv != SA_OK) {
                 err("Failed to build the interconnect RPT");
-                g_free(serial_number);
+                wrap_g_free(serial_number);
                 return rv;
         }
 
@@ -526,10 +563,10 @@ SaErrorT process_interconnect_info_event(struct oh_handler_state
          * So just go ahead and correct it. When building the RDR the code does
          * take care of already existing RDR. 
          */
-        rv = build_interconnect_rdr(oh_handler, con,
-                                    bay_number, resource_id);
+        rv = build_inserted_interconnect_rdr(oh_handler, con,
+                                    bay_number, resource_id, FALSE);
 	
-        g_free(serial_number);
+        wrap_g_free(serial_number);
         return SA_OK;
 
 
@@ -597,7 +634,6 @@ void oa_soap_proc_interconnect_status(struct oh_handler_state *oh_handler,
 {
         struct oa_soap_hotswap_state *hotswap_state;
         SaHpiRptEntryT *rpt = NULL;
-        struct oh_event event;
         SaHpiResourceIdT resource_id;
         struct oa_soap_handler *oa_handler = NULL;
 	enum oa_soap_extra_data_health health_status;
@@ -613,7 +649,7 @@ void oa_soap_proc_interconnect_status(struct oh_handler_state *oh_handler,
 
         resource_id = oa_handler->oa_soap_resources.interconnect.
 			resource_id[status->bayNumber - 1];
-        /* Get the rpt entry of the server */
+        /* Get the rpt entry of the interconnect */
         rpt = oh_get_resource_by_id(oh_handler->rptcache, resource_id);
         if (rpt == NULL) {
                 err("resource RPT is NULL");
@@ -623,33 +659,10 @@ void oa_soap_proc_interconnect_status(struct oh_handler_state *oh_handler,
         hotswap_state = (struct oa_soap_hotswap_state *)
                 oh_get_resource_data(oh_handler->rptcache, resource_id);
         if (hotswap_state == NULL) {
-                err("Failed to get hotswap state of server blade");
+                err("Failed to get hotswap state of interconnect");
                 return;
         }
 
-        /* Check whether blade is in the insertion pending state and it is
-         * powered on
-         */
-        if (hotswap_state->currentHsState == SAHPI_HS_STATE_INSERTION_PENDING &&
-	    status->powered == POWER_ON) {
-                hotswap_state->currentHsState = SAHPI_HS_STATE_ACTIVE;
-
-		update_hotswap_event(oh_handler, &event);
-		memcpy(&(event.resource), rpt, sizeof(SaHpiRptEntryT));
-		event.event.Source = event.resource.ResourceId;
-                event.event.EventDataUnion.HotSwapEvent.PreviousHotSwapState =
-                        SAHPI_HS_STATE_INSERTION_PENDING;
-                event.event.EventDataUnion.HotSwapEvent.HotSwapState =
-                        SAHPI_HS_STATE_ACTIVE;
-                /* INSERTION_PENDING to ACTIVE state change happened de
-                 * to Auto policy of server blade
-                 */
-                event.event.EventDataUnion.HotSwapEvent.CauseOfStateChange =
-                        SAHPI_HS_CAUSE_AUTO_POLICY;
-                oh_evt_queue_push(oh_handler->eventq,
-                                  copy_oa_soap_event(&event));
-        }
-
 	/* Build operational status sensor rdr */
 	OA_SOAP_PROCESS_SENSOR_EVENT(OA_SOAP_SEN_OPER_STATUS,
 				     status->operationalStatus, 0, 0)
@@ -767,7 +780,7 @@ void oa_soap_proc_interconnect_thermal(struct oh_handler_state *oh_handler,
 	SaHpiFloat64T trigger_threshold;
 	struct getThermalInfo thermal_request;
 	struct thermalInfo thermal_response;
-	struct oa_soap_sensor_info *sensor_info;
+	struct oa_soap_sensor_info *sensor_info = NULL;
 	SaHpiRdrT *rdr;
 
         if (oh_handler == NULL || con== NULL || response == NULL) {
@@ -782,7 +795,8 @@ void oa_soap_proc_interconnect_thermal(struct oh_handler_state *oh_handler,
 
 	rdr = oh_get_rdr_by_type(oh_handler->rptcache, resource_id,
 				 SAHPI_SENSOR_RDR, OA_SOAP_SEN_TEMP_STATUS);
-	sensor_info = (struct oa_soap_sensor_info *)
+        if (rdr)
+	        sensor_info = (struct oa_soap_sensor_info *)
 			oh_get_rdr_data(oh_handler->rptcache, resource_id,
 					rdr->RecordId);
 
