@@ -43,6 +43,9 @@
  *                                        with current status of the system
  *
  *      process_oa_info_event()         - Processes the OA info event
+ *                                        If OA info event has the updated 
+ *                                        OA firmware version, same will be
+ *                                        updated to RPT and IDR entry.
  *                                        If OA info event is just after OA
  *                                        insertion event, then it processed.
  *                                        Else, it is ignored
@@ -54,6 +57,7 @@
  */
 
 #include "oa_soap_oa_event.h"
+#include "sahpi_wrappers.h"
 
 /**
  * process_oa_extraction_event
@@ -77,12 +81,23 @@ SaErrorT process_oa_extraction_event(struct oh_handler_state *oh_handler,
 {
         SaErrorT rv = SA_OK;
         SaHpiInt32T bay_number;
+	struct oa_soap_handler *oa_handler = NULL;
+	SaHpiResourceIdT resource_id;
 
         if (oh_handler == NULL || oa_event == NULL) {
                 err("Invalid parameters");
                 return SA_ERR_HPI_INVALID_PARAMS;
         }
-
+	oa_handler = (struct oa_soap_handler *) oh_handler->data;
+	bay_number = oa_event->eventData.oaStatus.bayNumber;
+	resource_id = oa_handler->oa_soap_resources.oa.resource_id[bay_number - 1];
+	rv = oa_soap_proc_sen_evt(oh_handler, resource_id, OA_SOAP_SEN_OA_REDUND,
+		oa_event->eventData.oaStatus.oaRedundancy, 0, 0);
+	if (rv != SA_OK) {
+		err("processing the sensor event for sensor %x has failed",
+			OA_SOAP_SEN_OA_REDUND);
+		return rv;
+	}
         /* The OA is sending the wrong bay_number for the removed OA
          * Hence if the bay number in the oa_event is 1,
          * process the event for bay number 2, or vice versa
@@ -145,13 +160,13 @@ SaErrorT process_oa_failover_event(struct oh_handler_state *oh_handler,
         SaHpiBoolT is_transition_complete = SAHPI_FALSE;
         SaHpiInt32T sleep_time = 0;
         struct oa_soap_handler *oa_handler = NULL;
-        struct getAllEvents request;
+        struct getAllEventsEx request;
         struct getAllEventsResponse response;
         struct eventInfo event;
         GTimer *timer = NULL;
         gulong micro_seconds;
         gdouble time_elapsed = 0;
-        int is_switchover = SAHPI_TRUE;
+	 char oa_fw_buf[SAHPI_MAX_TEXT_BUFFER_LENGTH];
 
         if (oh_handler == NULL || oa == NULL) {
                 err("Invalid parameters");
@@ -176,31 +191,34 @@ SaErrorT process_oa_failover_event(struct oh_handler_state *oh_handler,
         /* Always lock the oa_handler mutex and then oa_info mutex
          * This is to avoid the deadlock
          */
-        g_mutex_lock(oa_handler->mutex);
-        g_mutex_lock(oa->mutex);
+        wrap_g_mutex_lock(oa_handler->mutex);
+        wrap_g_mutex_lock(oa->mutex);
 
         /* Point the active_con to the current active OA's hpi_con */
         oa_handler->active_con = oa->hpi_con;
         /* This OA has become ACTIVE from STANDBY */
         oa->oa_status = ACTIVE;
-        g_mutex_unlock(oa->mutex);
+        wrap_g_mutex_unlock(oa->mutex);
 
         /* Set the other OA status as STANDBY. If the other OA is extracted,
          * then the other OA status will be set to ABSENT during re-discovery.
          */
         if (oa_handler->oa_1 == oa) {
-                g_mutex_lock(oa_handler->oa_2->mutex);
+                wrap_g_mutex_lock(oa_handler->oa_2->mutex);
                 oa_handler->oa_2->oa_status = STANDBY;
-                g_mutex_unlock(oa_handler->oa_2->mutex);
+                wrap_g_mutex_unlock(oa_handler->oa_2->mutex);
         } else {
-                g_mutex_lock(oa_handler->oa_1->mutex);
+                wrap_g_mutex_lock(oa_handler->oa_1->mutex);
                 oa_handler->oa_1->oa_status = STANDBY;
-                g_mutex_unlock(oa_handler->oa_1->mutex);
+                wrap_g_mutex_unlock(oa_handler->oa_1->mutex);
         }
 
         request.pid = oa->event_pid;
         request.waitTilEventHappens = HPOA_TRUE;
         request.lcdEvents = HPOA_FALSE;
+	 memset(oa_fw_buf,0,SAHPI_MAX_TEXT_BUFFER_LENGTH);
+	 snprintf(oa_fw_buf,SAHPI_MAX_TEXT_BUFFER_LENGTH,"%.2f",oa->fm_version);
+        request.oaFwVersion = oa_fw_buf;
 
         /* Start the timer */
         timer = g_timer_new();
@@ -212,14 +230,14 @@ SaErrorT process_oa_failover_event(struct oh_handler_state *oh_handler,
 
 		OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, oa_handler->mutex, NULL,
 					  timer);
-                g_mutex_lock(oa->mutex);
-                rv = soap_getAllEvents(oa->event_con, &request, &response);
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_lock(oa->mutex);
+                rv = soap_getAllEventsEx(oa->event_con, &request, &response);
+                wrap_g_mutex_unlock(oa->mutex);
                 if (rv != SOAP_OK) {
                         err("Get all events failed during OA switchover"
                              "processing for OA %s", oa->server);
                         /* Unlock the oa_handler mutex*/
-                        g_mutex_unlock(oa_handler->mutex);
+                        wrap_g_mutex_unlock(oa_handler->mutex);
                         /* Cleanup the timer */
                         g_timer_destroy(timer);
 
@@ -261,7 +279,7 @@ SaErrorT process_oa_failover_event(struct oh_handler_state *oh_handler,
         }
 
         /* Unlock the oa_handler mutex */
-        g_mutex_unlock(oa_handler->mutex);
+        wrap_g_mutex_unlock(oa_handler->mutex);
 
         /* Get the time (in seconds) since the timer has been started */
         time_elapsed = g_timer_elapsed(timer, &micro_seconds);
@@ -271,7 +289,7 @@ SaErrorT process_oa_failover_event(struct oh_handler_state *oh_handler,
         sleep_time = OA_STABILIZE_MAX_TIME - time_elapsed;
         dbg("Sleeping for %d seconds", sleep_time);
         if (sleep_time > 0) {
-               sleep(sleep_time);
+               oa_soap_sleep_in_loop(oa_handler, sleep_time);
         }
 	OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, NULL, NULL, NULL);
 
@@ -291,30 +309,30 @@ SaErrorT process_oa_failover_event(struct oh_handler_state *oh_handler,
          * happened while waiting for OA stabilization)
          * Return without doing re-discovery
          */
-        g_mutex_lock(oa->mutex);
+        wrap_g_mutex_lock(oa->mutex);
         if (oa->oa_status != ACTIVE) {
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
                 oa_handler->oa_switching=SAHPI_FALSE;
                 err("OA status already changed. OA switching completed");
                 return SA_OK;
         }
-        g_mutex_unlock(oa->mutex);
+        wrap_g_mutex_unlock(oa->mutex);
 
-        g_mutex_lock(oa_handler->mutex);
-        g_mutex_lock(oa->mutex);
+        wrap_g_mutex_lock(oa_handler->mutex);
+        wrap_g_mutex_lock(oa->mutex);
         /* Call getAllEvents to flush the OA event queue
          * Any resource state change will be handled as part of the re-discovery
          */
-        rv = soap_getAllEvents(oa->event_con, &request, &response);
+        rv = soap_getAllEventsEx(oa->event_con, &request, &response);
 
         /* Re-discover the resources as there is a high chances
          * that we might have missed some events
          */
 	OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, oa_handler->mutex, oa->mutex,
 				  NULL);
-        rv = oa_soap_re_discover_resources(oh_handler, oa, is_switchover);
-        g_mutex_unlock(oa->mutex);
-        g_mutex_unlock(oa_handler->mutex);
+        rv = oa_soap_re_discover_resources(oh_handler, oa);
+        wrap_g_mutex_unlock(oa->mutex);
+        wrap_g_mutex_unlock(oa_handler->mutex);
 
         /* At this point assume that switchover is complete */
         oa_handler->oa_switching=SAHPI_FALSE;
@@ -328,6 +346,50 @@ SaErrorT process_oa_failover_event(struct oh_handler_state *oh_handler,
         return SA_OK;
 }
 
+/**
+ * process_oa_reboot_event
+ *      @oh_handler: Pointer to openhpi handler structure
+ *      @oa:         Pointer to the OA structure
+ *
+ * Purpose:
+ *      Gets the OA REBOOT event.
+ *      perform the rediscovery for all the resources.
+ *
+ * Detailed Description:
+ *      - OA needs some time to stabilize, after getting the OA REBOOT event,
+ *        plug-in starts checking for events after 90 seconds
+	- Establish the new connection with OA.
+ *      - Since there are high chances for missing the information of changes
+ *        in the resources, re-discovery will be done before start listening
+ *        for events.
+ * Return values:
+ *      SA_OK                     - on success.
+ *      SA_ERR_HPI_INVALID_PARAMS - on wrong parameters.
+ **/
+SaErrorT process_oa_reboot_event(struct oh_handler_state *oh_handler,
+                                   struct oa_info *oa)
+{
+	SaErrorT rv = SA_OK;
+	SaHpiInt32T sleep_time = 0;
+	dbg("\nThread id = %p \n",g_thread_self());
+
+	if (oh_handler == NULL || oa == NULL) {
+		err("Invalid parameters");
+		return SA_ERR_HPI_INVALID_PARAMS;
+	}
+	sleep_time = OA_STABILIZE_MAX_TIME;
+	dbg("Sleeping for %d seconds", sleep_time);
+	if (sleep_time > 0) {
+               oa_soap_sleep_in_loop(oh_handler->data, sleep_time);
+	}
+    /* Call the oa_soap error handling function to re establish the connection
+     * with OA and rediscover all the resources
+     */
+	oa_soap_error_handling(oh_handler, oa);
+
+	return rv;
+}
+
 /**
  * process_oa_info_event
  *      @oh_handler: Pointer to openhpi handler structure
@@ -346,6 +408,9 @@ SaErrorT process_oa_failover_event(struct oh_handler_state *oh_handler,
  *        the stabilization of OA.
  *      - When OA insertion event received, OA will not be stabilized
  *        On recieving this event, ACTIVE hot swap event will be generated
+ *      - If the OA_INFO event has the updated OA firmware
+ *        version information then the new firmware version
+ *        will be updated to RPT and IDR entry.
  *
  * Return values:
  *      SA_OK                     - on success.
@@ -359,6 +424,9 @@ SaErrorT process_oa_info_event(struct oh_handler_state *oh_handler,
         SaErrorT rv = SA_OK;
         SaHpiInt32T bay_number;
         struct oa_soap_handler *oa_handler = NULL;
+        int i = 0;
+        struct oaInfo response;
+        SaHpiResourceIdT resource_id;
 
         if (oh_handler == NULL || con == NULL || oa_event == NULL) {
                 err("Invalid parameters");
@@ -367,10 +435,22 @@ SaErrorT process_oa_info_event(struct oh_handler_state *oh_handler,
 
         oa_handler = (struct oa_soap_handler *) oh_handler->data;
         bay_number = oa_event->eventData.oaInfo.bayNumber;
+        response = oa_event->eventData.oaInfo;
 
         if (oa_handler->oa_soap_resources.oa.presence[bay_number - 1] ==
                             RES_PRESENT) {
-                dbg("OA is present. Ignore event");
+                if(oa_event->eventData.oaInfo.fwVersion != NULL) {
+                        for (i = 0;
+                        i < oa_handler->oa_soap_resources.oa.max_bays; i++) {
+                                resource_id = oa_handler->oa_soap_resources.oa.
+                                resource_id[i];
+                                rv = update_oa_fw_version(oh_handler,
+                                                &response, resource_id);
+                                if (rv != SA_OK) {
+                                        err("OA Firmware Version not updated");
+                                }
+                        }
+                }
                 return SA_OK;
         }
 
@@ -486,6 +566,9 @@ void oa_soap_proc_oa_network_info(struct oh_handler_state *oh_handler,
         SaHpiInt32T bay_number;
         struct oa_soap_handler *oa_handler = NULL;
 	SaHpiResourceIdT resource_id;
+        struct extraDataInfo extra_data_info;
+        struct oa_info *temp = NULL;
+        xmlNode *extra_data = NULL;
 
         if (oh_handler == NULL || nw_info == NULL) {
                 err("Invalid parameters");
@@ -494,12 +577,87 @@ void oa_soap_proc_oa_network_info(struct oh_handler_state *oh_handler,
 
         oa_handler = (struct oa_soap_handler *) oh_handler->data;
         bay_number = nw_info->bayNumber;
+
+        switch (bay_number) {
+                case 1:
+                        temp = oa_handler->oa_1;
+                        break;
+                case 2:
+                        temp = oa_handler->oa_2;
+                        break;
+        }
 	resource_id =
 		oa_handler->oa_soap_resources.oa.resource_id[bay_number - 1];
+        extra_data = nw_info->extraData;
+        while (extra_data) {
+                soap_getExtraData(extra_data, &extra_data_info);
+                if ((!(strcmp(extra_data_info.name, "IpSwap"))) &&
+                         (extra_data_info.value != NULL)) {
+                          if(!(strcasecmp(extra_data_info.value,
+                                                  "true"))){
+                                    oa_handler->ipswap = HPOA_TRUE;
+                                    dbg("Enclosure IP Mode is Enabled");
+                          } else {
+                                    oa_handler->ipswap = HPOA_FALSE;
+                                    dbg("Enclosure IP Mode is Disabled");
+                          }
+                          break;
+                }
+                extra_data = soap_next_node(extra_data);
+        }
+        /* Copy the server IP address to oa_info structure */
+	wrap_g_mutex_lock(temp->mutex);
+        memset(temp->server, 0, MAX_URL_LEN);
+        strncpy(temp->server, nw_info->ipAddress,
+                        strlen(nw_info->ipAddress));
+	wrap_g_mutex_unlock(temp->mutex);
 
 	/* Process the OA link status sensor */
 	OA_SOAP_PROCESS_SENSOR_EVENT(OA_SOAP_SEN_OA_LINK_STATUS,
 				     nw_info->linkActive, 0, 0)
 
         return;
-}
+} /*  oa_soap_proc_oa_network_info */
+
+/**
+ * oa_soap_proc_oa_inserted
+ *      @oh_handler : Pointer to openhpi handler structure
+ *      @oa_event : Pointer to the OA inserted event 
+ *
+ * Purpose:
+ *      Processes the OA inserted event and generates the HPI sensor event.
+ *
+ * Detailed Description:
+ *  NA
+ *
+ * Return values:
+ *  NONE
+ **/
+void oa_soap_proc_oa_inserted(struct oh_handler_state *oh_handler,
+                            struct eventInfo *oa_event)
+{
+	SaErrorT rv = SA_OK;
+	SaHpiInt32T bay_number;
+	struct oa_soap_handler *oa_handler = NULL;
+	SaHpiResourceIdT resource_id;
+
+	if (oh_handler == NULL || oa_event == NULL) {
+		err("Invalid parameters");
+		return;
+	}
+
+	oa_handler = (struct oa_soap_handler *) oh_handler->data;
+	bay_number = oa_event->eventData.oaStatus.bayNumber;
+	resource_id = oa_handler->oa_soap_resources.oa.resource_id[bay_number - 1];
+
+	/* Process the operational status sensor */
+	rv = oa_soap_proc_sen_evt(oh_handler, resource_id, OA_SOAP_SEN_OA_REDUND, 
+		oa_event->eventData.oaStatus.oaRedundancy, 0, 0);
+	if (rv != SA_OK) {
+		err("processing the sensor event for sensor %x has failed", 
+			OA_SOAP_SEN_OA_REDUND);
+		return;
+	}
+	return;
+} /* oa_soap_proc_oa_inserted */
+
