@@ -42,7 +42,8 @@
  *
  *      process_server_power_event()      - Processes the server power event
  *
- *      process_server_insertion_event()  - Processes the server insertion event
+ *      process_server_insert_completed()  - Processes the server 
+ *                                           insert completed event
  *
  *      process_server_extraction_event() - Processes the server extraction
  *                                          event
@@ -50,6 +51,9 @@
  *      build_inserted_server_rpt()       - Builds the rpt entry for inserted
  *                                          server
  *
+ *      oa_soap_parse_memory_sensor_reading() - Get the memory error from 
+ *                                          extra_data field
+ *
  *	oa_soap_proc_server_status() 	  - Processes the server status event
  *
  *	oa_soap_serv_post_comp ()	  - Processes the blade post complete 
@@ -61,6 +65,7 @@
 
 #include "oa_soap_server_event.h"
 #include "oa_soap_discover.h"           /* for build_server_rpt() prototype */
+#include "sahpi_wrappers.h"
 
 /**
  * process_server_power_off_event
@@ -161,7 +166,6 @@ SaErrorT process_server_power_on_event(struct oh_handler_state *oh_handler,
                                        SaHpiInt32T bay_number)
 {
         SaErrorT rv = SA_OK;
-        struct oa_soap_handler *oa_handler;
         struct oa_soap_hotswap_state *hotswap_state = NULL;
         struct oa_soap_sensor_info *sensor_info=NULL;
         SaHpiRdrT *rdr = NULL;
@@ -172,7 +176,6 @@ SaErrorT process_server_power_on_event(struct oh_handler_state *oh_handler,
                 return SA_ERR_HPI_INVALID_PARAMS;
         }
 
-        oa_handler = (struct oa_soap_handler *) oh_handler->data;
         hotswap_state = (struct oa_soap_hotswap_state *)
                 oh_get_resource_data(oh_handler->rptcache,
                                      event->resource.ResourceId);
@@ -291,7 +294,6 @@ SaErrorT process_server_power_event(struct oh_handler_state *oh_handler,
         SaErrorT rv = SA_OK;
         SaHpiRptEntryT *rpt = NULL;
         struct oa_soap_handler *oa_handler = NULL;
-        struct oa_soap_hotswap_state hotswap_state;
         SaHpiInt32T bay_number, loc=1;
         struct oh_event event;
         SaHpiResourceIdT resource_id;
@@ -339,7 +341,8 @@ SaErrorT process_server_power_event(struct oh_handler_state *oh_handler,
                    get executed at all. */
 
                 dbg("resource RPT is NULL, starting Workaround");
-                rv = process_server_insertion_event(oh_handler, con, oa_event, loc); 
+                rv = process_server_insert_completed(oh_handler, con, oa_event,
+                                                     loc);
                 return rv;
         }
 
@@ -381,7 +384,6 @@ SaErrorT process_server_power_event(struct oh_handler_state *oh_handler,
 
                 /* Currently, OA is not sending the REBOOT event*/
                 case (POWER_REBOOT):
-                        hotswap_state.currentHsState = SAHPI_HS_STATE_ACTIVE;
                         event.event.EventDataUnion.HotSwapEvent.
                                 PreviousHotSwapState =
                                 SAHPI_HS_STATE_INSERTION_PENDING;
@@ -405,7 +407,39 @@ SaErrorT process_server_power_event(struct oh_handler_state *oh_handler,
 }
 
 /**
- * process_server_insertion_event
+ *  oa_soap_proc_server_inserted_event
+ *       @oh_handler: Pointer to openhpi handler structure
+ *       @con:        Pointer to SOAP_CON structure
+ *       @oa_event:   Pointer to the OA event structure
+ *
+ * Initializes the timer when EVENT_BLADE_INSERTED is received
+ * This event doesnot contain valid information
+ * Blade is added when  EVENT_BLADE_INSERT_COMPLETED event arrives.
+ *
+ **/
+
+SaErrorT oa_soap_proc_server_inserted_event(struct oh_handler_state *oh_handler,
+                                        SOAP_CON *con,
+                                        struct eventInfo *oa_event)
+{
+        SaHpiInt32T bay_number = 0;
+        struct oa_soap_handler *oa_handler = NULL;
+        time_t now = 0;
+
+        if (oh_handler == NULL || con == NULL || oa_event == NULL) {
+                err("Invalid parameters");
+                return SA_ERR_HPI_INVALID_PARAMS;
+        }
+        oa_handler = (struct oa_soap_handler *) oh_handler->data;
+        time(&now);
+        bay_number =
+                oa_event->eventData.bladeStatus.bayNumber;
+        oa_handler->server_insert_timer[bay_number - 1] = now;
+        return SA_OK;
+}
+
+/**
+ * process_server_insert_completed
  *      @oh_handler: Pointer to openhpi handler structure
  *      @con:        Pointer to SOAP_CON structure
  *      @oa_event:   Pointer to the OA event structure
@@ -425,10 +459,10 @@ SaErrorT process_server_power_event(struct oh_handler_state *oh_handler,
  *      SA_ERR_HPI_INVALID_PARAMS - on wrong parameters.
  *      SA_ERR_HPI_INTERNAL_ERROR - on failure
  **/
-SaErrorT process_server_insertion_event(struct oh_handler_state *oh_handler,
-                                        SOAP_CON *con,
-                                        struct eventInfo *oa_event,
-                                        SaHpiInt32T loc)
+SaErrorT process_server_insert_completed(struct oh_handler_state 
+                           *oh_handler, SOAP_CON *con,
+                           struct eventInfo *oa_event,
+                           SaHpiInt32T loc)
 {
         SaErrorT rv = SA_OK;
         struct getBladeInfo info;
@@ -451,14 +485,13 @@ SaErrorT process_server_insertion_event(struct oh_handler_state *oh_handler,
 
         if ((oa_event->eventData.bladeStatus.powered == POWER_ON) &&
             (loc == 0)) {
-                /* Usually power ON event comes after insertion complete, but 5% of the
-                   time it comes first. So out of order events are processed out of order.
-                   The power_on event code calls this function with loc=1 to avoid 
-                   recursion */
+                /* Usually power ON event comes after insertion complete, 
+                   but 5% of the time it comes first. So out of order events 
+                   are processed out of order.  The power_on event code calls 
+                   this function with loc=1 to avoid recursion */
                 rv = process_server_power_event(oh_handler, con, oa_event);
                 return rv;
         }
-
         info.bayNumber = bay_number;
         rv = soap_getBladeInfo(con, &info, &response);
         if (rv != SOAP_OK) {
@@ -466,6 +499,11 @@ SaErrorT process_server_insertion_event(struct oh_handler_state *oh_handler,
                 return SA_ERR_HPI_INTERNAL_ERROR;
         }
 
+        if (strcmp(response.name,"[Unknown]") == 0 ) {
+              err("Server type at bay %d is unknown. Please check",bay_number);
+              return rv;
+        }
+
 	/* Copy the blade name from response for future processing */ 
 	convert_lower_to_upper(response.name, strlen(response.name), 
 			       blade_name, MAX_NAME_LEN);
@@ -477,6 +515,13 @@ SaErrorT process_server_insertion_event(struct oh_handler_state *oh_handler,
                 return rv;
         }
 
+        time_t now = 0;
+        time(&now);
+        int delay = now - oa_handler->server_insert_timer[bay_number - 1];
+        if (delay)
+               dbg("Took %d secs to add blade at bay %d\n",delay, bay_number);
+        oa_handler->server_insert_timer[bay_number - 1] = 0;
+
         /* Update resource_status structure with resource_id, serial_number,
          * and presence status
          */
@@ -485,8 +530,8 @@ SaErrorT process_server_insertion_event(struct oh_handler_state *oh_handler,
                       response.serialNumber, rpt.ResourceId, RES_PRESENT);
 
         /* Build the server RDR */
-        rv = build_server_rdr(oh_handler, con, bay_number, rpt.ResourceId,
-			      blade_name);
+        rv = build_inserted_server_rdr(oh_handler, con, bay_number, rpt.ResourceId,
+			      blade_name, TRUE);
         if (rv != SA_OK) {
                 err("build inserted server RDR failed");
                 /* Free the inventory info from inventory RDR */
@@ -538,6 +583,167 @@ SaErrorT process_server_insertion_event(struct oh_handler_state *oh_handler,
         return SA_OK;
 }
 
+/**
+ * process_server_info_event
+ *      @oh_handler: Pointer to openhpi handler structure
+ *      @con:        Pointer to the SOAP_CON structure
+ *      @oa_event:   Pointer to OA event structure
+ *
+ * Purpose:
+ *      Check if the serial number is there, if so insert.
+ *      This is bad as the query does not get the complete details
+ *      We need to make OA do the right stuff.
+ *      This is created only as an additional mechanism to get the
+ *      part/serial number of the already inserted blade server.
+ *      So not much error handling
+ *
+ * Detailed Description: NA
+ *
+ * Return values:
+ *      SA_OK                     - success.
+ *      SA_ERR_HPI_INVALID_PARAMS - Invalid parameters
+ *      SA_ERR_HPI_OUT_OF_MEMORY  - Out of memory
+ **/
+SaErrorT process_server_info_event(struct oh_handler_state
+                                               *oh_handler,
+                                               SOAP_CON *con,
+                                               struct eventInfo *oa_event)
+{
+        SaErrorT rv = SA_OK;
+        struct oa_soap_handler *oa_handler = NULL;
+        SaHpiInt32T bay_number, len;
+        char *serial_number = NULL;
+        char *name = NULL;
+        char *Name = NULL;
+        SaHpiResourceIdT resource_id;
+       	char blade_name[MAX_NAME_LEN];
+        SaHpiRptEntryT *rpt = NULL;
+        struct oh_event event;
+        SaHpiRdrT *rdr = NULL;
+
+        if (oh_handler == NULL || oa_event == NULL) {
+                err("Invalid oh_handler and/or oa_event parameters");
+                return SA_ERR_HPI_INVALID_PARAMS;
+        }
+
+        oa_handler = (struct oa_soap_handler *) oh_handler->data;
+        bay_number = oa_event->eventData.bladeInfo.bayNumber;
+
+        /* Null ponter check for serialNumber and partNumber
+        *  if any one of them is Null, return immediately.
+        */
+        if(oa_event->eventData.bladeInfo.serialNumber == 0 ||
+                      oa_event->eventData.bladeInfo.partNumber == 0) {
+                return rv;
+        }
+      
+        if (strcmp(oa_event->eventData.bladeInfo.serialNumber,"[Unknown]") == 0 )  {
+                return rv;
+        }
+       
+        if(strcmp(oa_event->eventData.bladeInfo.partNumber,"[Unknown]") == 0) {
+                return rv;
+        }
+
+        name = oa_event->eventData.bladeInfo.name;
+        resource_id = oa_handler->
+                oa_soap_resources.server.resource_id[bay_number - 1];
+
+        if (strcmp(name,"[Unknown]") == 0 ) {
+                err("Server Blade name is Unknown...bay_number = %d\n", bay_number);
+		return rv;
+        }
+
+        len = strlen(oa_event->eventData.bladeInfo.serialNumber);
+        serial_number = (char *)g_malloc0(sizeof(char) * len + 1);
+        if(serial_number == NULL){
+                return SA_ERR_HPI_OUT_OF_MEMORY;
+        }
+        strcpy(serial_number, oa_event->eventData.bladeInfo.serialNumber);
+        serial_number[len]='\0';
+
+        /* Update resource_status structure with resource_id,
+         * serial_number, and presence status
+         */
+        oa_soap_update_resource_status(
+                 &oa_handler->oa_soap_resources.server, bay_number,
+                 serial_number, resource_id, RES_PRESENT);
+
+        /* Get the rpt entry of the resource */
+        rpt = oh_get_resource_by_id(oh_handler->rptcache, resource_id);
+        if (rpt == NULL) {
+		if (oa_handler->server_insert_timer[bay_number-1]){
+                        wrap_g_free(serial_number);
+                        return SA_OK;
+		}
+		err("server RPT NULL at bay %d",bay_number);
+                wrap_g_free(serial_number);
+                return SA_ERR_HPI_INTERNAL_ERROR;
+        }
+
+        /* Convert Blade name lower to upper */
+        convert_lower_to_upper(name, strlen(name),
+                 blade_name, MAX_NAME_LEN);
+        /* The RDR already exist, but the relevant data is available only now
+         * So just go ahead and correct it. When building the RDR the code does
+         * take care of already existing RDR.
+         */
+        rv = build_inserted_server_rdr(oh_handler, con,
+                               bay_number, resource_id, blade_name, FALSE);
+        if (rv != SA_OK) {
+        	err("Failed to add Server rdr");
+                wrap_g_free(serial_number);
+        	return rv;
+        }	
+
+        Name = (char *)rpt->ResourceTag.Data;
+        if (strcmp(Name, "[Unknown]") == 0) {
+        	oa_soap_trim_whitespace(name);
+        	rpt->ResourceTag.DataLength = strlen(name);
+        	memset(rpt->ResourceTag.Data, 0, SAHPI_MAX_TEXT_BUFFER_LENGTH);
+        	snprintf((char *) (rpt->ResourceTag.Data),
+                	 rpt->ResourceTag.DataLength + 1, "%s", name);
+        	/* Add the server rpt to the plugin RPTable */
+        	rv = oh_add_resource(oh_handler->rptcache, rpt, NULL, 0);
+        	if (rv != SA_OK) {
+                	err("Failed to add Server rpt");
+                        wrap_g_free(serial_number);
+                	return rv;
+        	}
+
+		/* Get the inventory RDR */
+		rdr = oh_get_rdr_by_type(oh_handler->rptcache, resource_id,
+					SAHPI_INVENTORY_RDR,
+					SAHPI_DEFAULT_INVENTORY_ID);
+		if (rdr == NULL) {
+			err("Inventory RDR is not found");
+                        wrap_g_free(serial_number);
+			return SA_ERR_HPI_NOT_PRESENT;
+		}
+		
+                memset(&event, 0, sizeof(struct oh_event));
+                event.event.EventType = SAHPI_ET_RESOURCE;
+                memcpy(&event.resource,
+                        rpt,
+                        sizeof(SaHpiRptEntryT));
+                event.event.Severity = SAHPI_INFORMATIONAL;
+                event.event.Source = event.resource.ResourceId;
+                if (oh_gettimeofday(&(event.event.Timestamp)) != SA_OK) {
+                	event.event.Timestamp = SAHPI_TIME_UNSPECIFIED;
+                }
+                event.event.EventDataUnion.ResourceEvent.
+                        ResourceEventType = SAHPI_RESE_RESOURCE_UPDATED;
+                event.rdrs = g_slist_append(event.rdrs, g_memdup(rdr,
+                        sizeof(SaHpiRdrT)));
+                event.hid = oh_handler->hid;
+                oh_evt_queue_push(oh_handler->eventq,
+                        copy_oa_soap_event(&event));
+        }
+
+        wrap_g_free(serial_number);
+        return SA_OK;
+}
+
 /**
  * process_server_extraction_event
  *      @oh_handler: Pointer to openhpi handler structure
@@ -629,14 +835,52 @@ SaErrorT build_inserted_server_rpt(struct oh_handler_state *oh_handler,
         rv = oh_add_resource(oh_handler->rptcache, rpt, hotswap_state, 0);
         if (rv != SA_OK) {
                 err("Failed to add Server rpt");
-                if (hotswap_state != NULL)
-                        g_free(hotswap_state);
+                wrap_g_free(hotswap_state);
                 return rv;
         }
 
         return SA_OK;
 }
 
+/**
+ * oa_soap_parse_memory_sensor_reading
+ *      @memoryErrors   : Pointer to mainMemoryErros sensor reading
+ *
+ * Purpose:
+ *      Parses main memory sensor reading
+ *
+ * Detailed Description: NA
+ *
+ * Return values:
+ *      Pointer       - on success.
+ *      NULL          - on error.
+ **/
+SaHpiUint8T *oa_soap_parse_memory_sensor_reading(char *memoryErrors)
+{
+        char *subStr = NULL, *sensor_reading = NULL;
+        int len = 0; 
+        if (memoryErrors == NULL) {
+                err("Invalid parameters");
+                return NULL;
+        }
+
+        sensor_reading = (char  *)g_malloc0
+                            (sizeof(char) * SAHPI_SENSOR_BUFFER_LENGTH);
+        memset(sensor_reading, 0, SAHPI_SENSOR_BUFFER_LENGTH);
+        subStr = strstr(memoryErrors, ";");
+        if (subStr) 
+                len = strlen(memoryErrors) - strlen(subStr);
+        else
+                len = strlen(memoryErrors);
+
+        if (len >= SAHPI_SENSOR_BUFFER_LENGTH) 
+                len = SAHPI_SENSOR_BUFFER_LENGTH - 1;
+        strncpy(sensor_reading, memoryErrors, len);
+        sensor_reading[len] = '\0';
+        
+        return (SaHpiUint8T *)sensor_reading;
+}
+
 /**
  * oa_soap_proc_server_status
  *      @oh_handler	: Pointer to openhpi handler structure
@@ -659,25 +903,107 @@ void oa_soap_proc_server_status(struct oh_handler_state *oh_handler,
 	SaHpiRptEntryT *rpt = NULL;
 	struct oa_soap_handler *oa_handler = NULL;
 	SaHpiResourceIdT resource_id;
+        SaHpiInt32T bay = 0;
 	enum diagnosticStatus diag_ex_status[OA_SOAP_MAX_DIAG_EX];
 	struct getBladeThermalInfoArray thermal_request;
 	struct bladeThermalInfoArrayResponse thermal_response;
+	xmlNode *extra_data = NULL;
+	struct extraDataInfo extra_data_info;
+        char *mainMemoryError = NULL;
+        SaHpiInt32T sensor_status, memErrFlag[16] = {0};
+        SaHpiInt32T sensor_class, sensor_value, sensor_num;
 
-	if (oh_handler == NULL || status == NULL) {
+	if (oh_handler == NULL || con == NULL || status == NULL) {
 		err("Invalid parameters");
 		return;
 	}
 
 	oa_handler = (struct oa_soap_handler *) oh_handler->data;
+        bay = status->bayNumber;
 	resource_id = oa_handler->oa_soap_resources.server.
-			resource_id[status->bayNumber - 1];
+			resource_id[bay - 1];
 	/* Get the rpt entry of the resource */
 	rpt = oh_get_resource_by_id(oh_handler->rptcache, resource_id);
 	if (rpt == NULL) {
-		err("resource RPT is NULL");
+                /* RPT is null. It may yet to be added. If the timer is
+                   on event comes early.  Just return */
+                if ((oa_handler->server_insert_timer[bay - 1]) || 
+                    (status->powered == POWER_UNKNOWN)) {
+                        return;
+                } 
+                err("RPT of Server bay at %d is NULL",bay);
 		return;
 	}
 
+	extra_data = status->extraData;
+	while (extra_data) {
+                soap_getExtraData(extra_data, &extra_data_info);
+                if (!(strcmp(extra_data_info.name, "mainMemoryErrors"))) {
+                   err("openhpid[%d]: Blade (id=%d) at %d has Memory Error: %s",
+                    getpid(), resource_id, bay, extra_data_info.value);
+                   memErrFlag[bay] = 1;
+                   oa_handler->memErrRecFlag[bay] = 1;
+                   break;
+                }
+                extra_data = soap_next_node(extra_data);
+        }
+
+        if(oa_handler->memErrRecFlag[bay]) {
+                /* This MEMORY event is created just to let the user
+                   know whether all memory modules are fine, if not,
+                   which memory module is generating an error */
+                if (memErrFlag[bay]) {
+                    mainMemoryError = (char *)
+                    oa_soap_parse_memory_sensor_reading(extra_data_info.value);
+                    rv = oa_soap_proc_mem_evt(oh_handler, resource_id,
+                                              OA_SOAP_SEN_MAIN_MEMORY_ERRORS,
+                                              mainMemoryError, SAHPI_CRITICAL);
+                    if (rv != SA_OK) {
+                            err("processing the memory event for sensor %x has"
+                                " failed", OA_SOAP_SEN_MAIN_MEMORY_ERRORS);
+                            wrap_g_free(mainMemoryError);
+                            return;
+                    }
+                    wrap_g_free(mainMemoryError);
+                    memErrFlag[bay] = 0;
+                } else {
+                    /* Get the sensor value */
+                    sensor_num = OA_SOAP_SEN_PRED_FAIL;
+                    sensor_class = oa_soap_sen_arr[sensor_num].sensor_class;
+                    sensor_value = status->operationalStatus;
+
+                    /* Check whether the sensor value is supported or not */
+                    if (oa_soap_sen_val_map_arr[sensor_class][sensor_value]
+                                                                       == -1) {
+                            err("Not supported sensor value %d detected.",
+                                                                sensor_value);
+                            return;
+                    }
+
+                    /* Get the assert state of the predictive failure sensor */
+                    sensor_status =
+                        oa_soap_sen_assert_map_arr[sensor_class][sensor_value];
+
+                    /* Check whether predictive failure gets de-asserted */
+                    if (sensor_status == OA_SOAP_SEN_ASSERT_FALSE) {
+                            /* Now predictive failure is de-asserted and
+                               there are no memory module errors, so send
+                               an event with "All Memory Modules are Ok" */
+                            mainMemoryError = "All Memory Modules are Ok";
+                            rv = oa_soap_proc_mem_evt(oh_handler, resource_id,
+                                              OA_SOAP_SEN_MAIN_MEMORY_ERRORS,
+                                              mainMemoryError, SAHPI_OK);
+                            if (rv != SA_OK) {
+                                    err("processing the memory event for "
+                                        "sensor %x has failed",
+                                         OA_SOAP_SEN_MAIN_MEMORY_ERRORS);
+                                    return;
+                            }
+                            oa_handler->memErrRecFlag[bay] = 0;
+                    }
+                }
+        }
+
 	/* Process operational status sensor */
 	OA_SOAP_PROCESS_SENSOR_EVENT(OA_SOAP_SEN_OPER_STATUS,
 				     status->operationalStatus, 0, 0)
@@ -812,8 +1138,7 @@ void oa_soap_proc_server_status(struct oh_handler_state *oh_handler,
 							EntityLocation -1] == 
 							SAHPI_POWER_ON) {
 			dbg("Ignore the blade status event from the partner"
-			    " blade %d which is in POWER ON state",
-			    status->bayNumber);
+			    " blade %d which is in POWER ON state", bay);
 			return;
 		}
 				
@@ -822,7 +1147,7 @@ void oa_soap_proc_server_status(struct oh_handler_state *oh_handler,
 			    " enable thermal sensors");
 
 			/* Make getBladeThermalInfoArray soap call */ 
-			thermal_request.bayNumber = status->bayNumber;
+			thermal_request.bayNumber = bay;
 			rv = soap_getBladeThermalInfoArray(con, 
 						&thermal_request, 
 						&thermal_response);
@@ -835,8 +1160,7 @@ void oa_soap_proc_server_status(struct oh_handler_state *oh_handler,
 			if ((rv != SA_OK) ||
 			    (thermal_response.bladeThermalInfoArray == NULL)) {
 				err("getBladeThermalInfo failed for blade or"
-				    "the blade %d is not in stable state",
-				    status->bayNumber);
+				    "the blade %d is not in stable state",bay);
 				return;
 			}
 
@@ -861,7 +1185,7 @@ void oa_soap_proc_server_status(struct oh_handler_state *oh_handler,
 			 * oa_soap_bay_pwr_status array
 			 */
 			oa_soap_bay_pwr_status[rpt->ResourceEntity.Entry[0].
-							EntityLocation -1] == 
+							EntityLocation -1] = 
 							SAHPI_POWER_ON;
 		} else if (status->powered == POWER_OFF) {
 			dbg("thermal sensors of blade already in disable state,"
@@ -912,9 +1236,16 @@ void oa_soap_serv_post_comp(struct oh_handler_state
 		err("resource RPT is NULL");
 		return;
 	}
-
 	/* Make getBladeThermalInfoArray soap call */ 
 	thermal_request.bayNumber = bay_number;
+
+	/* The sleep below is introduced to make sure
+	 * we have sufficient time for the sensor to
+	 * be available so that we do not miss any
+	 * temperatuer sensor.
+	 */
+        oa_soap_sleep_in_loop(oa_handler, 20);
+
 	rv = soap_getBladeThermalInfoArray(con, &thermal_request,
 					   &thermal_response);
 
@@ -925,11 +1256,23 @@ void oa_soap_serv_post_comp(struct oh_handler_state
 	 * a NULL response
 	 */
 	if ((rv != SA_OK) || (thermal_response.bladeThermalInfoArray == NULL)) {
-		err("getBladeThermalInfo failed for blade or"
+		err("getBladeThermalInfo array failed for blade or"
 		    "the blade is not in stable state");
 		return;
 	}
 
+	 
+        /* Modify the thermal sensors IdString and send
+	 * event to the infrastructure
+	 */
+        rv = oa_soap_modify_blade_thermal_rdr(oh_handler, thermal_response, rpt);
+        
+	if (rv != SA_OK) {
+                err("oa_soap_modify_blade_thermal_rdr for rpt %d failed %d",
+			resource_id,rv);
+                return;
+	}
+
 	/* Walk through the rdr list of the resource and enable only those 
 	 * sensor which have the "SensorPresent" value as "true" in 
 	 * getBladeThermalInfoArray response. Rest of the statically modeled
@@ -941,7 +1284,6 @@ void oa_soap_serv_post_comp(struct oh_handler_state
 		err("Failed to enable the thermal sensor");
 		return;
 	}
-	
 	return;
 }
 
@@ -1055,3 +1397,233 @@ SaErrorT oa_soap_set_thermal_sensor(struct oh_handler_state *oh_handler,
 	}
 	return SA_OK;
 }
+
+
+/**
+ * process_server_thermal_event
+ *      @oh_handler	: Pointer to openhpi handler structure
+ *      @con            : Pointer to the SOAP_CON structure
+ *      @status         : Pointer to blade status structure
+ *
+ * Purpose:
+ *      Processes and creates server / blade sensor thermal events
+ *
+ * Detailed Description: NA
+ *
+ * Return values:
+ *      NONE
+ **/
+void oa_soap_proc_server_thermal(struct oh_handler_state *oh_handler,
+                                       SOAP_CON *con,
+                                        struct bladeStatus *status)
+{
+        SaErrorT rv = SA_OK;
+        SaHpiResourceIdT resource_id;
+        struct oa_soap_handler *oa_handler = NULL;
+        SaHpiFloat64T trigger_reading;
+        SaHpiFloat64T trigger_threshold;
+        struct getBladeThermalInfoArray thermal_request;
+        struct bladeThermalInfoArrayResponse thermal_response;
+        struct bladeThermalInfo *blade_thermal_response = NULL;
+        struct oa_soap_sensor_info *sensor_info = NULL;
+        SaHpiRdrT *rdr = NULL;
+ 
+        if (oh_handler == NULL || con== NULL || status == NULL) {
+                err("Invalid parameters");
+                return;
+        }
+ 
+        oa_handler = (struct oa_soap_handler *) oh_handler->data;
+	resource_id = oa_handler->oa_soap_resources.server.
+                        resource_id[status->bayNumber - 1];
+
+ 
+        rdr = oh_get_rdr_by_type(oh_handler->rptcache, resource_id,
+                                 SAHPI_SENSOR_RDR, OA_SOAP_SEN_TEMP_STATUS);
+        if (rdr)
+                sensor_info = (struct oa_soap_sensor_info *)
+                        oh_get_rdr_data(oh_handler->rptcache, resource_id,
+                                        rdr->RecordId);
+ 
+ 
+        /* Based on the sensor status,
+         * determine the threshold which triggered the thermal event from 
+	 * Enclosure.
+         * Event with SENSOR_STATUS_CAUTION or SENSOR_STATUS_OK is
+         * generated only if CAUTION threshold is crossed.
+         * Event with SENSOR_STATUS_CRITICAL is generated only when CRITICAL
+         * threshold is crossed.
+         * Sensor current reading and trigger threshold are required for event
+         * generation. Sensor current reading is not provided by the event,
+         * hence make soap call to get the reading
+         */
+        thermal_request.bayNumber = status->bayNumber;
+ 
+        rv = soap_getBladeThermalInfoArray(con, &thermal_request, &thermal_response);
+
+        /* In addition to verifying return value from the soap call,
+         * Check whether the thermal response is NULL,
+         * blade resource might have transitioned to POWER-OFF state
+         * during the processing of this event hence resulting in
+         * a NULL response
+         */
+        if ((rv != SA_OK) || (thermal_response.bladeThermalInfoArray == NULL)) {
+                err("getBladeThermalInfo array failed for blade or"
+                    "the blade is not in stable state");
+                return;
+        }
+ 
+	blade_thermal_response = (struct bladeThermalInfo *)&thermal_response;
+
+        trigger_reading = (SaHpiInt32T)blade_thermal_response->temperatureC;
+ 
+        if ((status->thermal == SENSOR_STATUS_CAUTION &&
+             sensor_info->current_state != SAHPI_ES_UPPER_MAJOR) ||
+            (status->thermal == SENSOR_STATUS_OK &&
+              sensor_info->current_state !=  SAHPI_ES_UNSPECIFIED)) {
+                /* Trigger for this event is caution threshold */
+                trigger_threshold = blade_thermal_response->cautionThreshold;
+        } else if (status->thermal == SENSOR_STATUS_CRITICAL  &&
+                   sensor_info->current_state != SAHPI_ES_UPPER_CRIT) {
+                /* Trigger for this event is critical threshold */
+                trigger_threshold = blade_thermal_response->criticalThreshold;
+        } else {
+                dbg("Ignore the event. There is no change in the sensor state");
+                return;
+        }
+ 
+        /* Process the thermal event from Server / Blade and generate 
+	 * appropriate HPI event
+         */
+        OA_SOAP_PROCESS_SENSOR_EVENT(OA_SOAP_SEN_TEMP_STATUS,
+                                status->thermal,
+                                trigger_reading,trigger_threshold)
+ 
+        return;
+}
+
+/**
+ * process_server_mp_info_event
+ *      @oh_handler: Pointer to openhpi handler structure
+ *      @con:        Pointer to the SOAP_CON structure
+ *      @oa_event:   Pointer to OA event structure
+ *
+ * Purpose:
+ *      Process the blade mp info event
+ *
+ * Detailed Description: NA
+ *
+ * Return values:
+ *      SA_OK                     - success.
+ *      SA_ERR_HPI_INVALID_PARAMS - Invalid parameters
+ *      SA_ERR_HPI_NOT_PRESENT - RDR is not present
+ **/
+SaErrorT process_server_mp_info_event(struct oh_handler_state
+                                               *oh_handler,
+                                               SOAP_CON *con,
+                                               struct eventInfo *oa_event)
+{
+        SaErrorT rv = SA_OK;
+        struct oa_soap_handler *oa_handler = NULL;
+        SaHpiInt32T bay_number;
+        SaHpiResourceIdT resource_id;
+        SaHpiRptEntryT *rpt = NULL;
+        struct oh_event event;
+        SaHpiRdrT *rdr = NULL;
+        SaHpiIdrIdT IdrId = 0;
+        SaHpiIdrFieldT field;
+        SaHpiFloat64T fm_version;
+        SaHpiInt32T major;
+        SaHpiInt32T minor;
+        char *fwVersion = NULL;
+
+        if (oh_handler == NULL || oa_event == NULL || con == NULL) {
+                err("Invalid parameters");
+                return SA_ERR_HPI_INVALID_PARAMS;
+        }
+
+        oa_handler = (struct oa_soap_handler *) oh_handler->data;
+        bay_number = oa_event->eventData.bladeMpInfo.bayNumber;
+        fwVersion = oa_event->eventData.bladeMpInfo.fwVersion;
+
+        resource_id = oa_handler->
+                oa_soap_resources.server.resource_id[bay_number - 1];
+
+        rpt = oh_get_resource_by_id(oh_handler->rptcache, resource_id);
+        if (rpt == NULL) {
+                if (oa_handler->server_insert_timer[bay_number - 1]) {
+                        return SA_OK;
+                }
+                err("Server RPT at bay %d is NULL",bay_number);
+                return SA_ERR_HPI_NOT_PRESENT;
+        }
+
+        /* Get the inventory RDR */
+        rdr = oh_get_rdr_by_type(oh_handler->rptcache, resource_id,
+                                SAHPI_INVENTORY_RDR,
+                                SAHPI_DEFAULT_INVENTORY_ID);
+        if (rdr == NULL) {
+                err("Inventory RDR is not found");
+                return SA_ERR_HPI_NOT_PRESENT;
+        }
+
+        IdrId = rdr->RdrTypeUnion.InventoryRec.IdrId;
+
+        if (strcmp(fwVersion,"[Unknown]") == 0 )  {
+                WARN("fwVersion is Unknown for server at bay %d",bay_number);
+                return rv;
+        }
+
+        fm_version = atof(fwVersion);
+        major = (SaHpiUint8T)floor(fm_version);
+        minor = rintf((fm_version - major) * 100);
+
+        if(rpt->ResourceInfo.FirmwareMajorRev == major &&
+                   rpt->ResourceInfo.FirmwareMinorRev == minor){
+                   return rv;
+        }
+
+        if(major < rpt->ResourceInfo.FirmwareMajorRev ||
+                   minor < rpt->ResourceInfo.FirmwareMinorRev){
+                err("Blade Firmware for Bay %d is going to be downgraded", bay_number);
+        }
+
+        if (fwVersion != NULL) {
+                field.Type = SAHPI_IDR_FIELDTYPE_PRODUCT_VERSION;
+                field.Field.DataType = SAHPI_TL_TYPE_TEXT;
+                field.Field.Language = SAHPI_LANG_ENGLISH;
+                oa_soap_trim_whitespace(fwVersion);
+                field.Field.DataLength = strlen (fwVersion) + 1;
+                field.AreaId = 1;
+                field.FieldId = 3;
+
+                snprintf((char *)field.Field.Data, field.Field.DataLength,
+                        "%s", fwVersion);
+
+                rv = oa_soap_set_idr_field(oh_handler, resource_id, IdrId, &field);
+                if (rv != SOAP_OK) {
+                        err("oa_soap_set_idr_field failed");
+                        return rv;
+                }
+
+                //Event Handling
+                memset(&event, 0, sizeof(struct oh_event));
+                event.event.EventType = SAHPI_ET_RESOURCE;
+                memcpy(&event.resource,
+                        rpt,
+                        sizeof(SaHpiRptEntryT));
+                event.event.Severity = SAHPI_INFORMATIONAL;
+                event.event.Source = event.resource.ResourceId;
+                if (oh_gettimeofday(&(event.event.Timestamp)) != SA_OK) {
+                        event.event.Timestamp = SAHPI_TIME_UNSPECIFIED;
+                }
+                event.event.EventDataUnion.ResourceEvent.
+                        ResourceEventType = SAHPI_RESE_RESOURCE_UPDATED;
+                event.rdrs = g_slist_append(event.rdrs, g_memdup(rdr,
+                        sizeof(SaHpiRdrT)));
+                event.hid = oh_handler->hid;
+                oh_evt_queue_push(oh_handler->eventq,
+                        copy_oa_soap_event(&event));
+        }
+        return SA_OK;
+}
