@@ -102,9 +102,19 @@
  *
  *      convert_lower_to_upper          - Converts the lower case to upper case
  *
+ *      update_oa_fw_version()          - Updates the RPT entry and IDR entry
+ *                                        with OA firmware version
+ *
+ *      oa_soap_check_serial_number()   - Check the serial_number and
+ *                                        give a proper message
+ *
+ *      oa_soap_sleep_in_loop()   	- Sleep in 3 second intervals so that
+ *                                        thread could catch the signal and exit
+ *
  **/
 
 #include "oa_soap_utils.h"
+#include "sahpi_wrappers.h"
 
 /**
  * get_oa_soap_info
@@ -219,13 +229,20 @@ SaErrorT get_oa_state(struct oh_handler_state *oh_handler,
         SaHpiInt32T i, bay = 0, active_bay = 0, standby_bay = 0;
         SOAP_CON *hpi_con = NULL, *event_con = NULL;
         struct oa_info *this_oa = NULL, *other_oa = NULL;
-        char *endptr;
+        char *endptr = NULL;
+        struct extraDataInfo extra_data_info;
+        xmlNode *extra_data = NULL;
 
         if (oh_handler == NULL|| server == NULL) {
                 err("Invalid parameters");
                 return SA_ERR_HPI_INVALID_PARAMS;
         }
 
+	if (!strcmp(server,"0.0.0.0")) {
+                err("Invalid OA IP 0.0.0.0");
+                return SA_ERR_HPI_INVALID_PARAMS;
+	}
+
         oa_handler = (struct oa_soap_handler *) oh_handler->data;
 
         /* Create the OA URL */
@@ -246,17 +263,21 @@ SaErrorT get_oa_state(struct oh_handler_state *oh_handler,
         /* Estabish the connection with OA */
         hpi_con = soap_open(url, user_name, password, HPI_CALL_TIMEOUT);
         if (hpi_con == NULL) {
+                free(url);
                 err("hpi_con intialization for OA - %s has failed", server);
                 return SA_ERR_HPI_INTERNAL_ERROR;
         }
 
         event_con = soap_open(url, user_name, password, EVENT_CALL_TIMEOUT);
         if (event_con == NULL) {
+                free(url);
                 err("event_con intialization for OA - %s has failed", server);
                 soap_close(hpi_con);
                 return SA_ERR_HPI_INTERNAL_ERROR;
         }
 
+        free(url);
+        url = NULL;
         /* Check whether user_name has admin rights */
         rv = check_oa_user_permissions(oa_handler, hpi_con, user_name);
         if (rv != SA_OK) {
@@ -341,16 +362,38 @@ SaErrorT get_oa_state(struct oh_handler_state *oh_handler,
                         return SA_ERR_HPI_INTERNAL_ERROR;
                 }
 
+                extra_data = network_info_response.extraData;
+                while (extra_data) {
+                        soap_getExtraData(extra_data, &extra_data_info);
+                        if ((!(strcmp(extra_data_info.name, "IpSwap"))) &&
+                                    (extra_data_info.value != NULL)) {
+                                    if(!(strcasecmp(extra_data_info.value,
+                                                  "true"))) {
+                                             oa_handler->ipswap = HPOA_TRUE;
+                                    } else {
+                                             oa_handler->ipswap = HPOA_FALSE;
+                                    }
+                                    break;
+                        }
+                        extra_data = soap_next_node(extra_data);
+                }
+
+
                 /* Find the active and standby bay number, IP address
                  * and firmware version */
                 switch (oa_role) {
                         case ACTIVE:
                                 active_bay = i;
                                 memset(active_ip, 0, MAX_URL_LEN);
-                                strncpy(active_ip,
-                                        network_info_response.ipAddress,
-                                        strlen(network_info_response.
-                                                       ipAddress));
+                                rv = oa_soap_get_oa_ip(server,
+                                                       network_info_response,
+                                                       active_ip);
+                                if (rv != SOAP_OK) {
+                                        err("Get Active OA IP failed");
+                                        soap_close(hpi_con);
+                                        soap_close(event_con);
+                                        return SA_ERR_HPI_INTERNAL_ERROR;
+                                }
 
                                 memset(active_fm, 0, MAX_BUF_SIZE);
                                 strncpy(active_fm, firmware, strlen(firmware));
@@ -368,10 +411,15 @@ SaErrorT get_oa_state(struct oh_handler_state *oh_handler,
                         case STANDBY:
                                 standby_bay = i;
                                 memset(standby_ip, 0, MAX_URL_LEN);
-                                strncpy(standby_ip,
-                                        network_info_response.ipAddress,
-                                        strlen(network_info_response.
-                                                       ipAddress));
+                                rv = oa_soap_get_oa_ip(server,
+                                                       network_info_response,
+                                                       standby_ip);
+                                if (rv != SOAP_OK) {
+                                        err("Get Standby OA IP failed");
+                                        soap_close(hpi_con);
+                                        soap_close(event_con);
+                                        return SA_ERR_HPI_INTERNAL_ERROR;
+                                }
 
                                 memset(standby_fm, 0, MAX_BUF_SIZE);
                                 strncpy(standby_fm, firmware, strlen(firmware));
@@ -416,7 +464,6 @@ SaErrorT get_oa_state(struct oh_handler_state *oh_handler,
         if (active_bay == 0 || standby_bay == 0)
                 return SA_OK;
 
-        memset(url, 0, MAX_URL_LEN);
         memset(other_oa->server, 0, MAX_URL_LEN);
         /* Construct the other OA url and copy the IP address to oa_info
          * structure
@@ -453,6 +500,7 @@ SaErrorT get_oa_state(struct oh_handler_state *oh_handler,
                                       password, HPI_CALL_TIMEOUT);
         if (other_oa->hpi_con == NULL) {
                 err("Initializing the hpi_con for OA %s failed", url);
+                free(url);
                 /* If this OA status is ACTIVE, then return error, else ignore
                  * If standby OA is not accessible, then the recovery from
                  * this problem will be done by the event thread.
@@ -475,6 +523,7 @@ SaErrorT get_oa_state(struct oh_handler_state *oh_handler,
                                         password, EVENT_CALL_TIMEOUT);
         if (other_oa->event_con == NULL) {
                 err("Initializing the event_con for OA %s failed", url);
+                free(url);
                 /* If this OA status is ACTIVE, then return error, else
                  * ignore
                  */
@@ -605,7 +654,7 @@ SaErrorT del_rdr_from_event(struct oh_event *event)
                 }
                 /* Pop out the RDR from the RDRs list  */
                 event->rdrs = g_slist_remove(event->rdrs, (gpointer)rdr);
-                g_free(rdr);
+                wrap_g_free(rdr);
                 /* Get the next RDR */
                 node = event->rdrs;
         } while (node != NULL);
@@ -651,11 +700,11 @@ SaErrorT check_oa_status(struct oa_soap_handler *oa_handler,
         else
                 status.bayNumber = 2;
 
-        g_mutex_lock(oa->mutex);
+        wrap_g_mutex_lock(oa->mutex);
         rv = soap_getOaStatus(con, &status, &status_response);
         if (rv != SOAP_OK) {
                 err("Get OA status call failed");
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
                 return SA_ERR_HPI_INTERNAL_ERROR;
         }
 
@@ -666,11 +715,11 @@ SaErrorT check_oa_status(struct oa_soap_handler *oa_handler,
                  * Wait till it it becomes Active
                  */
                 err("OA is in transition state");
-                sleep(OA_STABILIZE_MAX_TIME);
+                oa_soap_sleep_in_loop(oa_handler, OA_STABILIZE_MAX_TIME);
                 rv = soap_getOaStatus(con, &status, &status_response);
                 if (rv != SOAP_OK) {
                         err("Get OA status call failed");
-                        g_mutex_unlock(oa->mutex);
+                        wrap_g_mutex_unlock(oa->mutex);
                         return SA_ERR_HPI_INTERNAL_ERROR;
                 }
                 /* Check OA is still in TRANSITION state
@@ -679,28 +728,34 @@ SaErrorT check_oa_status(struct oa_soap_handler *oa_handler,
                 if (status_response.oaRole == TRANSITION) {
                         err("OA is in TRANSITION for a long time");
                         err("Please correct the OA");
-                        g_mutex_unlock(oa->mutex);
+                        wrap_g_mutex_unlock(oa->mutex);
                         return SA_ERR_HPI_INTERNAL_ERROR;
                  }
         }
 
-        oa->oa_status = status_response.oaRole;
+       /* If Enclosure IP mode is enabled then ipswap becomes true, 
+          then Active OA IP is always same. So do not change the Role */
+
+        if(!oa_handler->ipswap) {
+                oa->oa_status = status_response.oaRole;
+        }
+
         if (oa->oa_status == ACTIVE) {
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
                 /* Always lock the oa_handler mutex and then oa_info mutex
                  * This is to avoid the deadlock
                  */
-                g_mutex_lock(oa_handler->mutex);
-                g_mutex_lock(oa->mutex);
+                wrap_g_mutex_lock(oa_handler->mutex);
+                wrap_g_mutex_lock(oa->mutex);
                 /* Point the active_con to Active OA's hpi_con */
                 if (oa_handler->active_con != oa->hpi_con) {
                         oa_handler->active_con = oa->hpi_con;
                         err("OA %s has become Active", oa->server);
                 }
-                g_mutex_unlock(oa->mutex);
-                g_mutex_unlock(oa_handler->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa_handler->mutex);
         } else
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
 
         return SA_OK;
 }
@@ -715,8 +770,9 @@ SaErrorT check_oa_status(struct oa_soap_handler *oa_handler,
  *      check oa user permissions, even for the oa administrator.
  *
  * Detailed Description:
- *      - checks whether OA user has the ADMINISTRATOR permission
- *        and can access all the resources in HP BladeSystem c-Class
+ *      - checks whether OA user has ADMINISTRATOR or OPERATOR
+ *        permission and can access all the resources in HP 
+ *        BladeSystem c-Class
  *
  * Return values:
  *      SA_OK                     - on success.
@@ -759,9 +815,9 @@ SaErrorT check_oa_user_permissions(struct oa_soap_handler *oa_handler,
                 return SA_ERR_HPI_INTERNAL_ERROR;
         }
 
-        /* Check for the ADMINISTRATOR rights */
-        if (response.acl != ADMINISTRATOR) {
-                err("User - %s does not have Administrator rights for OA %s",
+        /* Check for the ADMINISTRATOR or OPERATOR rights */
+        if ((response.acl != ADMINISTRATOR) && (response.acl != OPERATOR))  {
+                err("User - %s is not Administrator or Operator on OA %s", 
                      user_name, con->server);
                 err("Please give full permissions to user - %s", user_name);
                 return SA_ERR_HPI_INTERNAL_ERROR;
@@ -832,7 +888,7 @@ SaErrorT check_oa_user_permissions(struct oa_soap_handler *oa_handler,
 
 SaErrorT check_discovery_failure(struct oh_handler_state *oh_handler)
 {
-        SaErrorT oa1_rv, oa2_rv;
+        SaErrorT oa1_rv, oa2_rv, rv;
         struct oa_soap_handler *oa_handler = NULL;
 
         if (oh_handler == NULL) {
@@ -846,6 +902,12 @@ SaErrorT check_discovery_failure(struct oh_handler_state *oh_handler)
         oa2_rv = SA_ERR_HPI_INTERNAL_ERROR;
 
         /* If the hpi_con is NULL, then OA is not reachable */
+        rv = get_oa_soap_info(oh_handler);
+       	if (rv != SA_OK) {
+                oa_handler->status = PLUGIN_NOT_INITIALIZED;
+                err("Get OA SOAP info failed");
+                return rv;
+        }
         if (oa_handler->oa_1->hpi_con != NULL) {
                 /* Get the status of the OA in slot 1 */
                 oa1_rv = check_oa_status(oa_handler, oa_handler->oa_1,
@@ -901,7 +963,7 @@ SaErrorT lock_oa_soap_handler(struct oa_soap_handler *oa_handler)
         }
 
         /* Try to lock the oa_handler mutex */
-        lock_state = g_mutex_trylock(oa_handler->mutex);
+        lock_state = wrap_g_mutex_trylock(oa_handler->mutex);
         if (lock_state == FALSE) {
                 err("OA SOAP Handler is locked.");
                 err("No operation is allowed in this state");
@@ -910,7 +972,7 @@ SaErrorT lock_oa_soap_handler(struct oa_soap_handler *oa_handler)
         }
 
         /* Unlock the oa_handler mutex */
-        g_mutex_unlock(oa_handler->mutex);
+        wrap_g_mutex_unlock(oa_handler->mutex);
         return SA_OK;
 }
 
@@ -995,15 +1057,15 @@ SaErrorT create_event_session(struct oa_info *oa)
                 return SA_ERR_HPI_INVALID_PARAMS;
         }
 
-        g_mutex_lock(oa->mutex);
+        wrap_g_mutex_lock(oa->mutex);
         if (oa->event_con == NULL) {
                 dbg("OA may not be accessible");
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
                 return SA_ERR_HPI_INTERNAL_ERROR;
         }
 
         rv = soap_subscribeForEvents(oa->event_con, &pid);
-        g_mutex_unlock(oa->mutex);
+        wrap_g_mutex_unlock(oa->mutex);
         if (rv != SOAP_OK) {
                 err("Subscribe for events failed");
                 return SA_ERR_HPI_INTERNAL_ERROR;
@@ -1052,20 +1114,20 @@ void create_oa_connection(struct oa_soap_handler *oa_handler,
                  is_oa_present = SAHPI_FALSE;
                  while (is_oa_present == SAHPI_FALSE) {
                 	OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, NULL, NULL, NULL);
-                        g_mutex_lock(oa->mutex);
+                        wrap_g_mutex_lock(oa->mutex);
                         if (oa->oa_status != OA_ABSENT) {
-                                g_mutex_unlock(oa->mutex);
+                                wrap_g_mutex_unlock(oa->mutex);
                                 is_oa_present = SAHPI_TRUE;
                         } else {
-                                g_mutex_unlock(oa->mutex);
+                                wrap_g_mutex_unlock(oa->mutex);
                                 /* OA is not present,
                                  * wait for 30 seconds and check again
                                  */
-                                sleep(30);
+                                oa_soap_sleep_in_loop(oa_handler, 30);
                         }
                 }
 
-                g_mutex_lock(oa->mutex);
+                wrap_g_mutex_lock(oa->mutex);
                 /* Close the soap_con strctures */
                 if (oa->hpi_con != NULL) {
                         soap_close(oa->hpi_con);
@@ -1075,7 +1137,7 @@ void create_oa_connection(struct oa_soap_handler *oa_handler,
                         soap_close(oa->event_con);
                         oa->event_con = NULL;
                 }
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
 
                 rv = initialize_oa_con(oa, user_name, password);
                 if ((rv != SA_OK) && (oa->oa_status != OA_ABSENT)) {
@@ -1122,12 +1184,18 @@ SaErrorT initialize_oa_con(struct oa_info *oa,
                 return SA_ERR_HPI_INVALID_PARAMS;
         }
 
-        g_mutex_lock(oa->mutex);
+	if (!strcmp(oa->server,"0.0.0.0")) {
+                err("Invalid OA IP  0.0.0.0");
+                return SA_ERR_HPI_INVALID_PARAMS;
+        }
+
+        wrap_g_mutex_lock(oa->mutex);
         rv = asprintf(&url, "%s" PORT, oa->server);			
         if(rv == -1){
                 free(url);
                 err("Failed to allocate memory for buffer to        \
                                              hold OA credentials");
+                wrap_g_mutex_unlock(oa->mutex);
                 return SA_ERR_HPI_OUT_OF_MEMORY;
         }
 
@@ -1135,8 +1203,9 @@ SaErrorT initialize_oa_con(struct oa_info *oa,
         oa->hpi_con = soap_open(url, user_name, password,
                                 HPI_CALL_TIMEOUT);
         if (oa->hpi_con == NULL) {
+                free(url);
                 /* OA may not be reachable */
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
                 return SA_ERR_HPI_INTERNAL_ERROR;
         }
 
@@ -1146,13 +1215,14 @@ SaErrorT initialize_oa_con(struct oa_info *oa,
         oa->event_con = soap_open(url, user_name, password,
                                   EVENT_CALL_TIMEOUT);
         if (oa->event_con == NULL) {
+                free(url);
                 /* OA may not be reachable */
-                g_mutex_unlock(oa->mutex);
+                wrap_g_mutex_unlock(oa->mutex);
                 soap_close(oa->hpi_con);
                 oa->hpi_con = NULL;
                 return SA_ERR_HPI_INTERNAL_ERROR;
         }
-        g_mutex_unlock(oa->mutex);
+        wrap_g_mutex_unlock(oa->mutex);
 	free(url);
         return SA_OK;
 
@@ -1263,92 +1333,62 @@ void release_oa_soap_resources(struct oa_soap_handler *oa_handler)
         /* Release memory of blade presence, resource id and blade
          * serial number arrays
          */
-        if (oa_handler->oa_soap_resources.server.presence != NULL) {
-                g_free(oa_handler->oa_soap_resources.server.presence);
-        }
-        if (oa_handler->oa_soap_resources.server.resource_id != NULL) {
-                g_free(oa_handler->oa_soap_resources.server.resource_id);
-        }
+        wrap_g_free(oa_handler->oa_soap_resources.server.presence);
+        wrap_g_free(oa_handler->oa_soap_resources.server.resource_id);
+
 	if(oa_handler->oa_soap_resources.server.serial_number != NULL) {
 	    for (i = 0; i < oa_handler->oa_soap_resources.server.max_bays; i++)
 	    {
-		if (oa_handler->oa_soap_resources.server.serial_number[i] !=
-                    NULL) {
-                        g_free(oa_handler->oa_soap_resources.server.
-                               serial_number[i]);
-		}
+                wrap_g_free(oa_handler->oa_soap_resources.server.serial_number[i]);
             }
-            g_free(oa_handler->oa_soap_resources.server.serial_number);
+            wrap_g_free(oa_handler->oa_soap_resources.server.serial_number);
 	}
 
         /* Release memory of interconnect presence and serial number array */
-        if (oa_handler->oa_soap_resources.interconnect.presence != NULL) {
-                g_free(oa_handler->oa_soap_resources.interconnect.presence);
-        }
-        if (oa_handler->oa_soap_resources.interconnect.resource_id != NULL) {
-                g_free(oa_handler->oa_soap_resources.interconnect.resource_id);
-        }
+        wrap_g_free(oa_handler->oa_soap_resources.interconnect.presence);
+        wrap_g_free(oa_handler->oa_soap_resources.interconnect.resource_id);
+
 	if(oa_handler->oa_soap_resources.interconnect.serial_number != NULL) {
             for (i = 0; i < oa_handler->oa_soap_resources.interconnect.max_bays;
              i++) {
-                if (oa_handler->oa_soap_resources.interconnect.
-                    serial_number[i] != NULL) {
-                        g_free(oa_handler->oa_soap_resources.interconnect.
-                               serial_number[i]);
-		}
+                      wrap_g_free(oa_handler->oa_soap_resources.interconnect.
+                          serial_number[i]);
             }
-            g_free(oa_handler->oa_soap_resources.interconnect.serial_number);
+            wrap_g_free(oa_handler->oa_soap_resources.interconnect.serial_number);
 	}
 
         /* Release memory of OA presence and serial number array */
-        if (oa_handler->oa_soap_resources.oa.presence != NULL) {
-                g_free(oa_handler->oa_soap_resources.oa.presence);
-        }
-        if (oa_handler->oa_soap_resources.oa.resource_id != NULL) {
-                g_free(oa_handler->oa_soap_resources.oa.resource_id);
-        }
+        wrap_g_free(oa_handler->oa_soap_resources.oa.presence);
+        wrap_g_free(oa_handler->oa_soap_resources.oa.resource_id);
+
 	if(oa_handler->oa_soap_resources.oa.serial_number != NULL) {
             for (i = 0; i < oa_handler->oa_soap_resources.oa.max_bays; i++) {
-                if (oa_handler->oa_soap_resources.oa.serial_number[i] != NULL) {
-                        g_free(oa_handler->oa_soap_resources.oa.
-                               serial_number[i]);
-		}
+                wrap_g_free(oa_handler->oa_soap_resources.oa.
+                    serial_number[i]);
             }
-            g_free(oa_handler->oa_soap_resources.oa.serial_number);
+            wrap_g_free(oa_handler->oa_soap_resources.oa.serial_number);
 	}
 
         /* Release memory of fan presence.  Since fans do not have serial
          * numbers, a serial numbers array does not need to be released.
          */
-        if (oa_handler->oa_soap_resources.fan.presence != NULL) {
-                g_free(oa_handler->oa_soap_resources.fan.presence);
-        }
-        if (oa_handler->oa_soap_resources.fan.resource_id != NULL) {
-                g_free(oa_handler->oa_soap_resources.fan.resource_id);
-        }
+        wrap_g_free(oa_handler->oa_soap_resources.fan.presence);
+        wrap_g_free(oa_handler->oa_soap_resources.fan.resource_id);
 
         /* Release memory of fan zone resource id */
-        if (oa_handler->oa_soap_resources.fan_zone.resource_id != NULL) {
-                g_free(oa_handler->oa_soap_resources.fan_zone.resource_id);
-        }
+        wrap_g_free(oa_handler->oa_soap_resources.fan_zone.resource_id);
 
         /* Release memory of power supply presence and serial number array */
-        if (oa_handler->oa_soap_resources.ps_unit.presence !=NULL) {
-                g_free(oa_handler->oa_soap_resources.ps_unit.presence);
-        }
-        if (oa_handler->oa_soap_resources.ps_unit.resource_id !=NULL) {
-                g_free(oa_handler->oa_soap_resources.ps_unit.resource_id);
-        }
+        wrap_g_free(oa_handler->oa_soap_resources.ps_unit.presence);
+        wrap_g_free(oa_handler->oa_soap_resources.ps_unit.resource_id);
+
 	if(oa_handler->oa_soap_resources.ps_unit.serial_number != NULL) {
             for (i = 0; i < oa_handler->oa_soap_resources.ps_unit.max_bays; i++)
 	    {
-                if (oa_handler->oa_soap_resources.ps_unit.serial_number[i]
-                    != NULL) {
-                        g_free(oa_handler->oa_soap_resources.
-                                ps_unit.serial_number[i]);
-		}
+                wrap_g_free(oa_handler->oa_soap_resources.
+                            ps_unit.serial_number[i]);
             }
-            g_free(oa_handler->oa_soap_resources.ps_unit.serial_number);
+            wrap_g_free(oa_handler->oa_soap_resources.ps_unit.serial_number);
 	}
 }
 
@@ -1428,7 +1468,8 @@ SaErrorT update_oa_info(struct oh_handler_state *oh_handler,
          * 'yy' is the minor version
          */
         fm_version = atof(response->fwVersion);
-        rpt->ResourceInfo.FirmwareMajorRev = major = rintf(fm_version);
+        rpt->ResourceInfo.FirmwareMajorRev = major = 
+				(SaHpiUint8T)floor(fm_version);
         rpt->ResourceInfo.FirmwareMinorRev = rintf((fm_version - major) * 100);
 
         return SA_OK;
@@ -1528,3 +1569,346 @@ char * oa_soap_trim_whitespace(char *s) {
   return(s);
 }
 
+/**
+ * update_oa_fw_version
+ *      @oh_handler:  Pointer to the plugin handler
+ *      @response:    Pointer to the oaInfo structure
+ *      @resource_id: Resource Id
+ *
+ * Purpose:
+ *      Updates the RPT entry and IDR entry with OA firmware version
+ *
+ * Detailed Descrption: NA
+ *
+ * Return values:
+ *      SA_HPI_ERR_INAVLID_PARAMS - on invalid parameters
+ *      SA_HPI_ERR_INTERNAL_ERROR - on failure
+ *      SA_OK                     - on success
+ **/
+
+SaErrorT update_oa_fw_version(struct oh_handler_state *oh_handler,
+                              struct oaInfo *response,
+                              SaHpiResourceIdT resource_id)
+{
+        SaErrorT rv = SA_OK;
+        SaHpiFloat64T fm_version;
+        SaHpiRptEntryT *rpt = NULL;
+        SaHpiRdrT *rdr = NULL;
+        SaHpiIdrIdT IdrId = 0;
+        SaHpiIdrFieldT field;
+        struct oh_event event;
+        SaHpiInt32T major;
+        SaHpiInt32T minor;
+
+        if (oh_handler == NULL || response == NULL) {
+                printf("Invalid parameter");
+                return SA_ERR_HPI_INVALID_PARAMS;
+        }
+
+        rpt = oh_get_resource_by_id(oh_handler->rptcache, resource_id);
+        if (rpt == NULL) {
+                err("OA rpt is not present");
+                return SA_ERR_HPI_INTERNAL_ERROR;
+        }
+
+        if (strlen(response->fwVersion) == 0) {
+                err("Firmware version is null string");
+                return SA_ERR_HPI_INTERNAL_ERROR;
+        }
+
+        fm_version = atof(response->fwVersion);
+        major = (SaHpiUint8T)floor(fm_version);
+        minor = rintf((fm_version - major) * 100);
+
+        if(rpt->ResourceInfo.FirmwareMajorRev == major &&
+                rpt->ResourceInfo.FirmwareMinorRev == minor){
+                return rv;
+        }
+
+        if(( major == rpt->ResourceInfo.FirmwareMajorRev &&
+        minor < rpt->ResourceInfo.FirmwareMinorRev ) ||
+        major < rpt->ResourceInfo.FirmwareMajorRev ) {
+                WARN("OA Firmware Version downgraded");
+        }
+	
+        rpt->ResourceInfo.FirmwareMajorRev = major;
+        rpt->ResourceInfo.FirmwareMinorRev = minor;
+
+        /* Get the inventory RDR */
+        rdr = oh_get_rdr_by_type(oh_handler->rptcache, resource_id,
+                                SAHPI_INVENTORY_RDR,
+                                SAHPI_DEFAULT_INVENTORY_ID);
+        if (rdr == NULL) {
+                err("Inventory RDR is not found");
+                return SA_ERR_HPI_INTERNAL_ERROR;
+        }
+
+        IdrId = rdr->RdrTypeUnion.InventoryRec.IdrId;
+
+        memset(&field, 0, sizeof(SaHpiIdrFieldT));
+        field.Type = SAHPI_IDR_FIELDTYPE_PRODUCT_VERSION;
+        field.Field.DataType = SAHPI_TL_TYPE_TEXT;
+        field.Field.Language = SAHPI_LANG_ENGLISH;
+        oa_soap_trim_whitespace(response->fwVersion);
+        field.Field.DataLength = strlen(response->fwVersion);
+        field.AreaId = 1;
+        field.FieldId = 3;
+        snprintf((char *)field.Field.Data,
+                        strlen(response->fwVersion)+ 1,
+                        "%s",response->fwVersion);
+
+        rv = oa_soap_set_idr_field(oh_handler, resource_id, IdrId,
+                                                &field);
+        if (rv != SOAP_OK) {
+                err("oa_soap_set_idr_field failed");
+                return rv;
+        }
+
+        /* Event Handling */
+        memset(&event, 0, sizeof(struct oh_event));
+        event.event.EventType = SAHPI_ET_RESOURCE;
+        memcpy(&event.resource,
+                    rpt,
+                    sizeof(SaHpiRptEntryT));
+        event.event.Severity = SAHPI_INFORMATIONAL;
+        event.event.Source = event.resource.ResourceId;
+        if (oh_gettimeofday(&(event.event.Timestamp)) != SA_OK) {
+                event.event.Timestamp = SAHPI_TIME_UNSPECIFIED;
+        }
+        event.event.EventDataUnion.ResourceEvent.
+        ResourceEventType = SAHPI_RESE_RESOURCE_UPDATED;
+        event.rdrs = g_slist_append(event.rdrs, g_memdup(rdr,
+                     sizeof(SaHpiRdrT)));
+        event.hid = oh_handler->hid;
+        oh_evt_queue_push(oh_handler->eventq,
+        copy_oa_soap_event(&event));
+
+        return SA_OK;
+}
+
+/**
+ * oa_soap_get_oa_ip
+ *      @server:     Pointer to hostname/IP address of the OA
+ *      @network_info_response: Pointer to oaNetworkInfo response structure
+ *      @oa_ip:     Pointer to Active/Standby OA IP address
+ *
+ * Purpose:
+ *      Gets the active/standby OA IP address
+ *
+ * Detailed Description: NA
+ *
+ * Return values:
+ *      SA_OK  - on success.
+ *      SA_ERR_HPI_INVALID_PARAMS - on wrong parameters
+ **/
+SaErrorT oa_soap_get_oa_ip(char *server, 
+                           struct oaNetworkInfo network_info_response,
+                           char *oa_ip)
+{
+        int ipv6, local_ipv6;
+        char *interface_name;
+        struct extraDataInfo extra_data_info;
+        xmlNode *extra_data = NULL;
+
+        if (server == NULL) {
+                err("Invalid parameters");
+                return SA_ERR_HPI_INVALID_PARAMS;
+        }
+
+        /* intialize the flags for ipv6  */
+        ipv6 = strstr(server, ":") == NULL ? 0 : 1;
+        local_ipv6 = (strstr(server, "fe80")- server) == 0 ? 1 : 0;
+
+        extra_data = network_info_response.extraData;
+        while (extra_data) {
+                soap_getExtraData(extra_data, &extra_data_info);
+                if ((!(strcmp(extra_data_info.name, "Ipv6Address0"))) 
+                       && (extra_data_info.value != NULL)) {
+                        memcpy(network_info_response.ipv6Address0,
+                               extra_data_info.value,
+                               strlen(extra_data_info.value) -
+                               strlen(strchr(extra_data_info.value, '/')));
+                }
+                if ((!(strcmp(extra_data_info.name, "Ipv6Address1")))
+                       && (extra_data_info.value != NULL)) {
+                        memcpy(network_info_response.ipv6Address1,
+                               extra_data_info.value,
+                               strlen(extra_data_info.value) -
+                               strlen(strchr(extra_data_info.value, '/')));
+                }
+                if ((!(strcmp(extra_data_info.name, "Ipv6Address2")))
+                       && (extra_data_info.value != NULL)) {
+                        memcpy(network_info_response.ipv6Address2,
+                               extra_data_info.value,
+                               strlen(extra_data_info.value) -
+                               strlen(strchr(extra_data_info.value, '/')));
+                }
+                if ((!(strcmp(extra_data_info.name, "Ipv6Address3")))
+                       && (extra_data_info.value != NULL)) {
+                        memcpy(network_info_response.ipv6Address3,
+                               extra_data_info.value,
+                               strlen(extra_data_info.value) -
+                               strlen(strchr(extra_data_info.value, '/')));
+                }
+                if ((!(strcmp(extra_data_info.name, "Ipv6AddressType0"))) &&
+                                           (extra_data_info.value != NULL)) {
+                        network_info_response.ipv6AddressType0 = 
+                                                 extra_data_info.value;
+                }
+                if ((!(strcmp(extra_data_info.name, "Ipv6AddressType1"))) &&
+                                           (extra_data_info.value != NULL)) {
+                        network_info_response.ipv6AddressType1 =
+                                                extra_data_info.value;
+                }
+                if ((!(strcmp(extra_data_info.name, "Ipv6AddressType2"))) &&
+                                           (extra_data_info.value != NULL)) {
+                        network_info_response.ipv6AddressType2 =
+                                                extra_data_info.value;
+                }
+                if ((!(strcmp(extra_data_info.name, "Ipv6AddressType3"))) &&
+                                           (extra_data_info.value != NULL)) {
+                        network_info_response.ipv6AddressType3 =
+                                                extra_data_info.value;
+                }
+                extra_data = soap_next_node(extra_data);
+        }
+
+        if(!ipv6) {
+                strncpy(oa_ip, network_info_response.ipAddress,
+                        strlen(network_info_response.ipAddress));
+        } else if(!local_ipv6) {
+                if ((!(strcmp(network_info_response.ipv6AddressType0,
+                                                              "STATIC")))) {
+                        strncpy(oa_ip, network_info_response.ipv6Address0,
+                                strlen(network_info_response.ipv6Address0));
+                } else if ((!(strcmp(network_info_response.ipv6AddressType1,
+                                                              "STATIC")))) {
+                        strncpy(oa_ip, network_info_response.ipv6Address1,
+                                strlen(network_info_response.ipv6Address1));
+                } else if ((!(strcmp(network_info_response.ipv6AddressType2,
+                                                              "STATIC")))) {
+                        strncpy(oa_ip, network_info_response.ipv6Address2,
+                                strlen(network_info_response.ipv6Address2));
+                } else if ((!(strcmp(network_info_response.ipv6AddressType3,
+                                                              "STATIC")))) {
+                        strncpy(oa_ip, network_info_response.ipv6Address3,
+                                strlen(network_info_response.ipv6Address3));
+                }
+        } else {
+                if ((!(strcmp(network_info_response.ipv6AddressType0,
+                                                           "LINKLOCAL")))) {
+                        strncpy(oa_ip, network_info_response.ipv6Address0,
+                                strlen(network_info_response.ipv6Address0));
+                } else if ((!(strcmp(network_info_response.ipv6AddressType1,
+                                                           "LINKLOCAL")))) {
+                        strncpy(oa_ip, network_info_response.ipv6Address1,
+                                strlen(network_info_response.ipv6Address1));
+                } else if ((!(strcmp(network_info_response.ipv6AddressType2,
+                                                           "LINKLOCAL")))) {
+                        strncpy(oa_ip, network_info_response.ipv6Address2,
+                                strlen(network_info_response.ipv6Address2));
+                } else if ((!(strcmp(network_info_response.ipv6AddressType3,
+                                                           "LINKLOCAL")))) {
+                        strncpy(oa_ip, network_info_response.ipv6Address3,
+                                strlen(network_info_response.ipv6Address3));
+                }
+                interface_name = strchr(server, '%');
+                strcat(oa_ip, interface_name);
+        }
+        return SA_OK;
+}
+
+/**
+ * oa_soap_check_serial_number()
+ *      @slot:              slot number of the blade
+ *      @serial_number:     Pointer to serial_number string
+ *
+ * Purpose:
+ *      Just prints out a message. OA sends out information from iLO as it
+ *      becomes available. So empty, "[Unknown]" serial numbers are common
+ *      During a re-discovery after a switchover a blade with a good serial
+ *      number could be replaced with a blade with a bad serial number. So 
+ *      only thing we could do is give a warning message. Nothing more. It 
+ *      is users responsility to correct it using RBSU/boot utility
+ *
+ * Detailed Description: NA
+ *
+ * Return values:
+ *                Void
+ **/
+void oa_soap_check_serial_number(int slot, char *serial_number)
+{
+        int j=0, len=0;
+
+        if (serial_number == 0 ) { 
+               WARN("Blade(%d) serialNumber is NULL",slot);
+        } else if ((len = strlen(serial_number)) == 0) { 
+               WARN("Blade(%d) serialNumber is empty",slot);
+        } else if (strcmp(serial_number,"[Unknown]")) {
+               if (len >= 9) 
+                    len = 9;
+               for (j =0; j < len; j++){
+                     if (isalnum(serial_number[j]))
+                         continue;
+                     else {
+                         CRIT("Blade(%d) serialNumber %s is "
+                              "invalid",slot,serial_number);
+                         break;
+                     }
+               }
+        } else {
+               dbg("Blade(%d) serialNumber is [Unknown]",slot);
+        }
+}
+
+/**
+ * oa_soap_sleep_in_loop()
+ *      @oa_handler: 	Pointer to OA SOAP handler structure
+ *      @secs:		Total time to sleep in seconds
+ * 
+ * Purpose:
+ *	OA takes a long time to switchover and there are many situations
+ *	where we need to wait for more than 3 seconds. In all thse situations
+ * 	if a signal arrives, it is better to get out asap. So we check for the
+ *	shutdown_event_thread after every 3 seconds and get out if it is true
+ *	This will enable full, proper shutdown of the plugin in case of kill
+ *
+ * Detailed Description: NA
+ *
+ * Return values:
+ *                Exit or OK
+ **/
+SaErrorT oa_soap_sleep_in_loop(struct oa_soap_handler *oa_handler, int secs)
+{
+        int i=3, j=0; 
+        GThread *my_id;
+        if (oa_handler == NULL || oa_handler->oa_1 == NULL || 
+                 oa_handler->oa_2 == NULL || secs <= 0 ) {
+                err("Wrong parameters\n");
+                return SA_ERR_HPI_INTERNAL_ERROR;
+        }
+        if (secs < 4) {
+                sleep(secs);
+                return SA_OK;
+        } 
+        my_id = g_thread_self();
+        while (j < secs) {
+
+               /* Exit only from event threads, owned by plugin */
+               if (my_id == oa_handler->oa_1->thread_handler ||
+                   my_id == oa_handler->oa_2->thread_handler ) {
+                        OA_SOAP_CHEK_SHUTDOWN_REQ(oa_handler, NULL, NULL, NULL);
+               } else {
+                        /* In case of infrastructure threads just return */
+                        if( oa_handler->shutdown_event_thread )
+                                return SA_OK;
+               } 
+               if (j+i > secs)
+                       i=secs-j;         
+               if ( i > 0 )
+                       sleep(i);
+               j = j+i;
+                
+        }
+        return SA_OK;
+}
