@@ -54,7 +54,9 @@ struct sim_rpt sim_rpt_array[] = {
                                                 SAHPI_CAPABILITY_POWER |
                                                 SAHPI_CAPABILITY_RESET |
                                                 SAHPI_CAPABILITY_WATCHDOG |
-                                                SAHPI_CAPABILITY_CONTROL,
+                                                SAHPI_CAPABILITY_CONTROL |
+                                                SAHPI_CAPABILITY_DIMI |
+                                                SAHPI_CAPABILITY_FUMI,
                         .ResourceSeverity = SAHPI_CRITICAL,
 			.ResourceFailed = SAHPI_FALSE,
                 },
@@ -169,7 +171,8 @@ struct sim_rpt sim_rpt_array[] = {
                         .ResourceCapabilities = SAHPI_CAPABILITY_EVT_DEASSERTS |
 			                        SAHPI_CAPABILITY_RESOURCE |
 			                        SAHPI_CAPABILITY_RDR |
-			                        SAHPI_CAPABILITY_SENSOR,
+			                        SAHPI_CAPABILITY_SENSOR |
+						SAHPI_CAPABILITY_CONTROL,
                         .ResourceSeverity = SAHPI_MAJOR,
  			.ResourceFailed = SAHPI_FALSE,
                 },
@@ -1142,7 +1145,7 @@ struct sim_control sim_chassis_controls[] = {
                         .Num = 1,
                         .OutputType = SAHPI_CTRL_LED,
                         .Type = SAHPI_CTRL_TYPE_DIGITAL,
-                        .TypeUnion.Digital.Default = SAHPI_CTRL_STATE_ON,
+                        .TypeUnion.Digital.Default = SAHPI_CTRL_STATE_ON,   
 			.DefaultMode = {
 				.Mode = SAHPI_CTRL_MODE_AUTO,
 				.ReadOnly = SAHPI_TRUE,
@@ -1449,6 +1452,29 @@ struct sim_control sim_hs_dasd_controls[] = {
 };
 
 struct sim_control sim_fan_controls[] = {
+	{
+		.index = 1,
+		.control = {
+			.Num = 1,
+			.OutputType = SAHPI_CTRL_FAN_SPEED,
+			.Type = SAHPI_CTRL_TYPE_ANALOG,
+			.TypeUnion = {
+				.Analog = {
+					.Min = 0,
+					.Max = 100,
+					.Default = 80
+				}
+			},
+			.DefaultMode = {
+				.Mode = SAHPI_CTRL_MODE_MANUAL,
+				.ReadOnly = SAHPI_FALSE
+			},
+			.WriteOnly = SAHPI_FALSE,
+			.Oem = 0
+		},
+		.mode = SAHPI_CTRL_MODE_MANUAL,
+		.comment = "Fan Analog Control"
+	},
 
         {} /* Terminate array with a null element */
 };
@@ -1844,3 +1870,145 @@ struct sim_inventory sim_fan_inventory[] = {
         {} /* Terminate array with a null element */
 };
 
+struct sim_dimi sim_chassis_dimis[] = {
+    {
+            .dimirec = {
+                    .DimiNum = 1,
+                    .Oem = 0,
+            },
+            
+            .info = {
+                .NumberOfTests =1,
+                .TestNumUpdateCounter=0,
+            },
+            .test = {
+                    .TestName={
+                        .DataType = SAHPI_TL_TYPE_TEXT,
+                        .Language = SAHPI_LANG_ENGLISH,
+                        .DataLength = 7,
+                        .Data = "unknown"
+                    },
+                    .ServiceImpact=SAHPI_DIMITEST_NONDEGRADING,
+                    .EntitiesImpacted= {
+                        {
+                            .EntityImpacted.Entry= {
+                                {
+                                        .EntityType = SAHPI_ENT_PROCESSOR,
+                                        .EntityLocation = 1
+                                },
+                                {
+                                        .EntityType = SAHPI_ENT_POWER_SUPPLY,
+                                        .EntityLocation = 1
+                                },
+                                {
+                                        .EntityType = SAHPI_ENT_ADD_IN_CARD,
+                                        .EntityLocation = 1
+                                },
+                                {
+                                        .EntityType = SAHPI_ENT_POWER_UNIT,
+                                        .EntityLocation = 1
+                                },
+                                {
+                                        .EntityType = SAHPI_ENT_ROOT,
+                                        .EntityLocation = 0
+                                },
+                            },    
+                            .ServiceImpact=SAHPI_DIMITEST_NONDEGRADING,
+                        }
+                    },
+                    .NeedServiceOS= SAHPI_FALSE,
+                    .ExpectedRunDuration=1000000000,
+                    .TestCapabilities=SAHPI_DIMITEST_CAPABILITY_TESTCANCEL,
+                    .TestParameters = {
+                                {
+                                        .ParamName = "Some Test Param Name",
+                                        .ParamInfo = {
+                                                .DataType = SAHPI_TL_TYPE_TEXT,
+                                                .Language = SAHPI_LANG_ENGLISH,
+                                                .DataLength = 7,
+                                                .Data = "Unknown"
+                                        },
+                                        .ParamType = SAHPI_DIMITEST_PARAM_TYPE_BOOLEAN,
+                                        .DefaultParam.parambool = SAHPI_TRUE
+                                }
+                    }
+            },
+            .comment = "Dimi 1 simulator",
+    },
+    
+    {} /* Terminate array with a null element */
+};
+
+struct sim_fumi sim_chassis_fumis[] = {
+    {
+            .fumirec = {
+                    .Num = 1,
+                    .AccessProt = SAHPI_FUMI_PROT_FTP,
+                    .Capability = SAHPI_FUMI_CAP_BANKCOPY,
+                    .NumBanks = 1,
+                    .Oem = 0,
+            },
+            
+            .srcinfo = {
+                    .SourceUri = {
+                        .DataType = SAHPI_TL_TYPE_TEXT,
+                        .Language = SAHPI_LANG_ENGLISH,
+                        .DataLength = 7,
+                        .Data = "unknown"
+                    },
+                    .SourceStatus = SAHPI_FUMI_SRC_VALID,
+                    .Identifier = {
+                        .DataType = SAHPI_TL_TYPE_TEXT,
+                        .Language = SAHPI_LANG_ENGLISH,
+                        .DataLength = 3,
+                        .Data = "abc"
+                    },
+                    .Description = {
+                        .DataType = SAHPI_TL_TYPE_TEXT,
+                        .Language = SAHPI_LANG_ENGLISH,
+                        .DataLength = 3,
+                        .Data = "xyz"
+                    },
+                    .DateTime = {
+                        .DataType = SAHPI_TL_TYPE_TEXT,
+                        .Language = SAHPI_LANG_ENGLISH,
+                        .DataLength = 3,
+                        .Data = "09/05/2008"
+                    },
+                    .MajorVersion = 0,
+                    .MinorVersion = 0,
+                    .AuxVersion = 0
+            },
+            
+            .info = {
+                    .BankId = 0,
+                    .BankSize = 250,
+                    .Position = 1,
+                    .BankState = SAHPI_FUMI_BANK_ACTIVE,
+                    .Identifier = {
+                        .DataType = SAHPI_TL_TYPE_TEXT,
+                        .Language = SAHPI_LANG_ENGLISH,
+                        .DataLength = 3,
+                        .Data = "abc"
+                    },
+                    .Description = {
+                        .DataType = SAHPI_TL_TYPE_TEXT,
+                        .Language = SAHPI_LANG_ENGLISH,
+                        .DataLength = 3,
+                        .Data = "xyz"
+                    },
+                    .DateTime = {
+                        .DataType = SAHPI_TL_TYPE_TEXT,
+                        .Language = SAHPI_LANG_ENGLISH,
+                        .DataLength = 3,
+                        .Data = "09/05/2008"
+                    },
+                    .MajorVersion = 0,
+                    .MinorVersion = 0,
+                    .AuxVersion = 0
+            },
+            .comment = "Fumi 1 simulator",
+    },
+    
+    {} /* Terminate array with a null element */
+};
