@@ -19,6 +19,7 @@
 
 #include "codec.h"
 #include "handler.h"
+#include "log.h"
 #include "resource.h"
 #include "structs.h"
 #include "timers.h"
@@ -36,12 +37,13 @@ namespace TA {
 static void MakeDefaultRptEntry( const SaHpiEntityPathT& ep,
                                  SaHpiRptEntryT& rpte )
 {
-    static SaHpiGuidT guid = { 0xFB, 0x2B, 0x5D, 0xD5, 0x4E, 0x7D, 0x49, 0xF5,
-                               0x93, 0x97, 0xC2, 0xFE, 0xC2, 0x1B, 0x40, 0x10 };
+    static const SaHpiGuidT guid = { 0xFB, 0x2B, 0x5D, 0xD5, 0x4E, 0x7D, 0x49, 0xF5,
+                                     0x93, 0x97, 0xC2, 0xFE, 0xC2, 0x1B, 0x40, 0x10 };
     SaHpiEntityPathT ep2 = ep;
     SaHpiResourceIdT rid = oh_uid_from_entity_path( &ep2 );
 
-    rpte.EntryId                = SAHPI_ENTRY_UNSPECIFIED;
+    //rpte.EntryId                = SAHPI_ENTRY_UNSPECIFIED;
+    rpte.EntryId                = rid; // NB! This is OpenHPI-specific
     rpte.ResourceId             = rid;
 
     SaHpiResourceInfoT& ri = rpte.ResourceInfo;
@@ -76,13 +78,12 @@ static void MakeDefaultRptEntry( const SaHpiEntityPathT& ep,
  * class cResource
  *************************************************************/
 cResource::cResource( cHandler& handler,
-                      cTimers& timers,
                       const SaHpiEntityPathT& ep )
 
     : cObject( AssembleResourceObjectName( ep ), SAHPI_FALSE ),
-      cInstruments( *this ),
+      cInstruments( handler, *this ),
       m_handler( handler ),
-      m_timers( timers )
+      m_log( 0 )
 {
     MakeDefaultRptEntry( ep, m_rpte );
     m_failed             = m_rpte.ResourceFailed;
@@ -101,8 +102,10 @@ cResource::cResource( cHandler& handler,
 
 cResource::~cResource()
 {
+    delete m_log;
+    m_log = 0;
+    m_handler.CancelTimer( this );
     SetVisible( false );
-    m_timers.CancelTimer( this );
 }
 
 const SaHpiRptEntryT& cResource::GetRptEntry() const
@@ -130,6 +133,10 @@ void cResource::UpdateCaps( SaHpiCapabilitiesT caps )
     m_rpte.ResourceCapabilities |= caps;
 }
 
+cLog * cResource::GetLog() const
+{
+    return m_log;
+}
 
 // HPI interface
 SaErrorT cResource::SetTag( const SaHpiTextBufferT& tag )
@@ -160,7 +167,7 @@ SaErrorT cResource::CancelHsPolicy( const SaHpiTimeoutT& timeout )
             return SA_ERR_HPI_INVALID_REQUEST;
     }
 
-    m_timers.CancelTimer( this );
+    m_handler.CancelTimer( this );
 
     return SA_OK;
 }
@@ -220,7 +227,7 @@ SaErrorT cResource::SetHsState( SaHpiHsStateT state )
             return SA_ERR_HPI_INVALID_REQUEST;
     }
 
-    m_timers.CancelTimer( this );
+    m_handler.CancelTimer( this );
     m_new_hs_state = state;
     CommitChanges();
 
@@ -255,7 +262,7 @@ SaErrorT cResource::RequestHsAction( SaHpiHsActionT action )
     }
 
     CommitChanges();
-    m_timers.SetTimer( this, t );
+    m_handler.SetTimer( this, t );
 
     return SA_OK;
 }
@@ -409,6 +416,20 @@ void cResource::PostEvent( SaHpiEventTypeT type,
                            const InstrumentList& updates,
                            const InstrumentList& removals ) const
 {
+    if ( m_log ) {
+        const cInstrument * instr = 0;
+        if ( !updates.empty() ) {
+            instr = updates.front();
+        } else if ( !removals.empty() ) {
+            instr = removals.front();
+        }
+        if ( instr ) {
+            const SaHpiRdrT& rdr = instr->GetRdr();
+            m_log->AddEntry( type, data, severity, &rdr, &m_rpte );
+        } else {
+            m_log->AddEntry( type, data, severity, 0, &m_rpte );
+        }
+    }
     if ( !IsVisible() ) {
         return;
     }
@@ -506,6 +527,10 @@ bool cResource::CreateChild( const std::string& name )
     if ( rc ) {
         return true;
     }
+    if ( name == cLog::classname ) {
+        CreateLog();
+        return true;
+    }
     rc = CreateInstrument( name );
     if ( rc ) {
         return true;
@@ -522,6 +547,10 @@ bool cResource::RemoveChild( const std::string& name )
     if ( rc ) {
         return true;
     }
+    if ( name == cLog::classname ) {
+        RemoveLog();
+        return true;
+    }
     rc = RemoveInstrument( name );
     if ( rc ) {
         return true;
@@ -533,6 +562,9 @@ bool cResource::RemoveChild( const std::string& name )
 void cResource::GetChildren( Children& children ) const
 {
     cObject::GetChildren( children );
+    if ( m_log ) {
+        children.push_back( m_log );
+    }
     cInstruments::GetChildren( children );
 }
 
@@ -613,6 +645,24 @@ void cResource::AfterVarSet( const std::string& var_name )
     CommitChanges();
 }
 
+void cResource::CreateLog()
+{
+    if ( m_log == 0 ) {
+        m_log = new cLog();
+        m_rpte.ResourceCapabilities |= cLog::RequiredResourceCap();
+        PostResourceEvent( SAHPI_RESE_RESOURCE_UPDATED );
+    }
+}
+
+void cResource::RemoveLog()
+{
+    if ( m_log ) {
+        delete m_log;
+        m_log = 0;
+        m_rpte.ResourceCapabilities &= ~cLog::RequiredResourceCap();
+        PostResourceEvent( SAHPI_RESE_RESOURCE_UPDATED );
+    }
+}
 
 void cResource::TimerEvent()
 {
@@ -659,10 +709,10 @@ void cResource::CommitChanges()
     SaHpiTimeoutT ait, aet, t;
     if ( m_hs_state == SAHPI_HS_STATE_INSERTION_PENDING ) {
         GetTimeouts( t, aet );
-        m_timers.SetTimer( this, t );
+        m_handler.SetTimer( this, t );
     } else if ( m_hs_state == SAHPI_HS_STATE_EXTRACTION_PENDING ) {
         GetTimeouts( ait, t );
-        m_timers.SetTimer( this, t );
+        m_handler.SetTimer( this, t );
     }
 }
 
