@@ -21,6 +21,7 @@
 #include <SaHpi.h>
 
 #include "instrument.h"
+#include "timers.h"
 
 
 namespace TA {
@@ -29,17 +30,23 @@ namespace TA {
 /**************************************************************
  * class cWatchdog
  *************************************************************/
-class cWatchdog : public cInstrument
+class cWatchdog : public cInstrument, private cTimerCallback
 {
 public:
 
     static const std::string classname;
 
-    explicit cWatchdog( cResource& resource, SaHpiWatchdogNumT num );
+    explicit cWatchdog( cHandler& handler,
+                        cResource& resource,
+                        SaHpiWatchdogNumT num );
     virtual ~cWatchdog();
 
 public:  // HPI interface
 
+    SaErrorT Get( SaHpiWatchdogT& wdt ) const;
+    SaErrorT Set( const SaHpiWatchdogT& wdt );
+    SaErrorT Reset();
+
 protected: // cObject virtual functions
 
     virtual void GetVars( cVars& vars );
@@ -56,9 +63,19 @@ private:
         return SAHPI_CAPABILITY_WATCHDOG;
     }
 
+private: // cTimerCallback virtual functions
+
+    virtual void TimerEvent();
+
+private:
+
+    void ProcessTick();
+    void PostEvent( SaHpiWatchdogActionEventT ae );
+
 private: // data
 
     const SaHpiWatchdogRecT& m_rec;
+    SaHpiWatchdogT m_wdt;
 };
 
 
