@@ -66,6 +66,7 @@
 #include <openssl/crypto.h>
 #include <openssl/bio.h>
 #include <openssl/rand.h>
+#include <openssl/engine.h>
 #include <unistd.h>
 #include <string.h>
 #include <sys/select.h>
@@ -73,6 +74,12 @@
 #include <oh_ssl.h>
 #include <oh_error.h>
 
+#include <sys/ioctl.h>
+#include <sys/socket.h>
+#include <sys/types.h>
+#include <arpa/inet.h>
+#include <netdb.h>
+#include <sahpi_wrappers.h>
 
 /* Data types used by this module */
 struct CRYPTO_dynlock_value {
@@ -82,9 +89,12 @@ struct CRYPTO_dynlock_value {
 
 /* Global (static) data for this module */
 static int	oh_ssl_init_done = 0;	/* Will be set true when done */
-static GMutex **mutexes = NULL;		/* Holds array of SSL mutexes */
+static GMutex **mutexes = NULL;         /* Holds array of SSL mutexes */
+#if GLIB_CHECK_VERSION (2, 32, 0)
+static GMutex ssl_mutexes; /* Lock for above */
+#else
 static GStaticMutex ssl_mutexes = G_STATIC_MUTEX_INIT; /* Lock for above */
-
+#endif
 
 /* Local (static) functions, used by this module.  Note that these aren't
  * necessary if we aren't compiling as a threaded implementation, so we
@@ -129,27 +139,27 @@ static void lock_function(int mode, int type, const char * file, int line)
 	/* Do we have an array of mutex pointers yet? */
 	if (! mutexes) {
 		/* Messing with this requires the static lock */
-		g_static_mutex_lock(&ssl_mutexes);
+		wrap_g_static_mutex_lock(&ssl_mutexes);
 		if (! mutexes) {	/* Need to check again */
 			mutexes = (GMutex **)g_malloc0(CRYPTO_num_locks() *
 						       sizeof(GMutex *));
 			if (! mutexes) {
-				err("out of memory");
-				g_static_mutex_unlock(&ssl_mutexes);
+				CRIT("out of memory");
+				wrap_g_static_mutex_unlock(&ssl_mutexes);
 				return;
 			}
 		}
-		g_static_mutex_unlock(&ssl_mutexes);
+		wrap_g_static_mutex_unlock(&ssl_mutexes);
 	}
 
 	/* Have we initialized this particular mutex? */
 	if (! mutexes[type]) {
 		/* Same firedrill as above */
-		g_static_mutex_lock(&ssl_mutexes);
+		wrap_g_static_mutex_lock(&ssl_mutexes);
 		if (! mutexes[type]) {
-			mutexes[type] = g_mutex_new();
+			mutexes[type] = wrap_g_mutex_new_init();
 		}
-		g_static_mutex_unlock(&ssl_mutexes);
+		wrap_g_static_mutex_unlock(&ssl_mutexes);
 	}
 
 	/* Finally, go ahead and lock or unlock it */
@@ -177,11 +187,11 @@ static struct CRYPTO_dynlock_value *dyn_create_function(const char *file,
 	struct CRYPTO_dynlock_value *value;
 
 	if ((value = (struct CRYPTO_dynlock_value *)
-			g_malloc(sizeof(struct CRYPTO_dynlock_value)))) {
-		value->mutex = g_mutex_new();
+			g_malloc0(sizeof(struct CRYPTO_dynlock_value)))) {
+		value->mutex = wrap_g_mutex_new_init();
 	}
 	else {
-		err("out of memory");
+		CRIT("out of memory");
 	}
 
 	return(value);
@@ -225,7 +235,7 @@ static void dyn_lock_function(int mode, struct CRYPTO_dynlock_value *l,
 static void	dyn_destroy_function(struct CRYPTO_dynlock_value *l,
 				     const char *file, int line)
 {
-	g_mutex_free(l->mutex);
+	wrap_g_mutex_free_clear(l->mutex);
 	g_free(l);
 }
 
@@ -250,7 +260,6 @@ static int	thread_setup(void)
 }
 
 
-#if 0					/* Not used right now */
 /**
  * thread_cleanup
  *
@@ -273,22 +282,21 @@ static int	thread_cleanup(void)
 	CRYPTO_set_dynlock_destroy_callback(NULL);
 
 	/* Clean up and destroy mutexes, if we have any */
-	g_static_mutex_lock(&ssl_mutexes);
+	wrap_g_static_mutex_lock(&ssl_mutexes);
 	if (mutexes) {
 		for (i = 0; i < CRYPTO_num_locks(); i++) {
 			if (mutexes[i]) {
-				g_mutex_free(mutexes[i]);
+				wrap_g_mutex_free_clear(mutexes[i]);
 			}
 		}
 		g_free(mutexes);
 		mutexes = NULL;
 	}
-	g_static_mutex_free(&ssl_mutexes);
+        wrap_g_static_mutex_unlock(&ssl_mutexes);
+	wrap_g_static_mutex_free_clear(&ssl_mutexes);
 
 	return(0);			/* No errors */
 }
-#endif /* Not used right now */
-
 
 /**
  * oh_ssl_init
@@ -318,7 +326,7 @@ int		oh_ssl_init(void)
 
 		/* Initialize the SSL library */
 		if (! SSL_library_init()) {
-			err("SSL_library_init() failed");
+			CRIT("SSL_library_init() failed");
 			return(-1);
 		}
 
@@ -329,7 +337,7 @@ int		oh_ssl_init(void)
 
 		/* Set up multi-thread protection functions */
 		if (thread_setup() ) {
-			err("SSL multi-thread protection setup call failed");
+			CRIT("SSL multi-thread protection setup call failed");
 			return(-1);
 		}
 
@@ -338,6 +346,37 @@ int		oh_ssl_init(void)
 	return(0);			/* Successful return */
 }
 
+/**
+ * oh_ssl_finit
+ *
+ * Finalizes the OpenSSL library. The calls used in this routine releases global
+ * data created/initialized by the OpenSSL library.
+ *
+ * Note that it is the responisbility of the caller of this function to make
+ * sure that no other threads are making the OpenSSL library calls. The openhpid
+ * should close all the threads and call this function from the main (single)
+ * thread.
+ *
+ * Return value: None
+ **/
+void oh_ssl_finit(void)
+{
+        /* TODO: Check whether any other SSL library cleanup should be called */
+        thread_cleanup();
+        ENGINE_cleanup();
+        CONF_modules_unload(0);
+        ERR_free_strings();
+        EVP_cleanup();
+        CRYPTO_cleanup_all_ex_data();
+        /* The valgrind is showing possible memory leak by
+         * SSL_COMP_get_compression_methods() call.
+         *
+         * Call to SSL_free_comp_methods() may resolve the memory leak.
+         * But not able to find this call in the openssl 0.9.8e
+         * TODO: Find whether its a real problem or not
+         */
+
+}
 
 /**
  * oh_ssl_ctx_init
@@ -360,14 +399,14 @@ SSL_CTX         *oh_ssl_ctx_init()
 
         ctx = SSL_CTX_new(SSLv23_client_method());
         if (ctx == NULL) {
-                err("SSL_CTX_new() failed");
+                CRIT("SSL_CTX_new() failed");
                 return(NULL);
         }
 
         SSL_CTX_set_options(ctx, SSL_OP_TLS_ROLLBACK_BUG | SSL_OP_ALL);
 
         if (! SSL_CTX_set_default_verify_paths(ctx)) {
-                err("SSL_CTX_set_default_verify_paths() failed");
+                CRIT("SSL_CTX_set_default_verify_paths() failed");
                 return(NULL);
         }
 
@@ -386,7 +425,7 @@ SSL_CTX         *oh_ssl_ctx_init()
 int             oh_ssl_ctx_free(SSL_CTX *ctx)
 {
         if (ctx == NULL) {
-                err("unexpected NULL ctx pointer");
+                CRIT("unexpected NULL ctx pointer");
                 return(-1);
         }
 
@@ -412,117 +451,119 @@ BIO             *oh_ssl_connect(char *hostname, SSL_CTX *ctx, long timeout)
 {
         BIO             *bio;
         SSL             *ssl;
-        fd_set          readfds;
-        fd_set          writefds;
-        struct timeval  tv;
-        int             fd;
         int             err;
+        int len, retval = 0;
+        int RetVal, socket_desc = 0;
+        char *Server = NULL;
+        char *Port = NULL;
+        struct addrinfo Hints, *AddrInfo = NULL, *ai = NULL;
+
+        memset(&Hints, 0, sizeof(Hints));
+        Hints.ai_family = AF_UNSPEC;
+        Hints.ai_socktype = SOCK_STREAM;
+        len = strlen(hostname);
 
         if (hostname == NULL) {
-                err("NULL hostname in oh_ssl_connect()");
+                CRIT("NULL hostname in oh_ssl_connect()");
                 return(NULL);
         }
         if (ctx == NULL) {
-                err("NULL ctx in oh_ssl_connect()");
+                CRIT("NULL ctx in oh_ssl_connect()");
                 return(NULL);
         }
         if (timeout < 0) {
-                err("inappropriate timeout in oh_ssl_connect()");
+                CRIT("inappropriate timeout in oh_ssl_connect()");
                 return(NULL);
         }
 
-        /* Start with a new SSL BIO */
-        bio = BIO_new_ssl_connect(ctx);
-        if (bio == NULL) {
-                err("BIO_new_ssl_connect() failed");
-                return(NULL);
+        /* Allocate memory to a char pointer "Server" */
+        Server = (char *) g_malloc0(sizeof(char) * len);
+        if (Server == NULL){
+                CRIT("out of memory");
+                return NULL;
         }
-
-        /* Set up connection parameters for this BIO */
-        BIO_set_conn_hostname(bio, hostname);
-        BIO_set_nbio(bio, 1);           /* Set underlying socket to
-                                         * non-blocking mode
-                                         */
-
-        /* Set up SSL session parameters */
-        BIO_get_ssl(bio, &ssl);
-        if (ssl == NULL) {
-                err("BIO_get_ssl() failed");
-                BIO_free_all(bio);
-                return(NULL);
-        }
-        SSL_set_mode(ssl, SSL_MODE_ENABLE_PARTIAL_WRITE);
-
-        /* Ready to open the connection.  Note that this call will probably
-         * take a while, so we need to retry it, watching for a timeout.
+        memset(Server, 0, len);
+        /* hostname contains "Port" along with "IP Address". As, only
+         * "IP Address" is needed for some of the below operations, so copy
+         * "IP Address" from hostname to "Server".
          */
-        while (1) {
-                if (BIO_do_connect(bio) == 1) {
-                        break;          /* Connection established */
-                }
-                if (! BIO_should_retry(bio)) { /* Hard error? */
-                        err("BIO_do_connect() failed");
-                        err("SSL error: %s",
-                            ERR_reason_error_string(ERR_get_error()));
-                        BIO_free_all(bio);
-                        return(NULL);
-                }
+        strncpy(Server, hostname, (len - 4));
 
-                /* Wait until there's a change in the socket's status or until
-                 * the timeout period.
-                 *
-                 * Get underlying file descriptor, needed for the select call.
-                 */
-                fd = BIO_get_fd(bio, NULL);
-                if (fd == -1) {
-                        err("BIO isn't initialized in oh_ssl_connect()");
-                        BIO_free_all(bio);
-                        return(NULL);
-                }
-
-                FD_ZERO(&readfds);
-                FD_ZERO(&writefds);
-                if (BIO_should_read(bio)) {
-                        FD_SET(fd, &readfds);
-                }
-                else if (BIO_should_write(bio)) {
-                        FD_SET(fd, &writefds);
-                }
-                else {                  /* This is BIO_should_io_special().
-                                         * Not sure what "special" needs to
-                                         * wait for, but both read and write
-                                         * seems to work without unnecessary
-                                         * retries.
-                                         */
-                        FD_SET(fd, &readfds);
-                        FD_SET(fd, &writefds);
-                }
-                if (timeout) {
-                        tv.tv_sec = timeout;
-                        tv.tv_usec = 0;
-                        err = select(fd + 1, &readfds, &writefds, NULL, &tv);
-                }
-                else {                  /* No timeout */
-                        err = select(fd + 1, &readfds, &writefds, NULL, NULL);
-                }
-
-                /* Evaluate select() return code */
-                if (err < 0) {
-                        err("error during select()");
-                        BIO_free_all(bio);
-                        return(NULL);
-                }
-                if (err == 0) {
-                        err("connection timeout to %s", hostname);
-                        BIO_free_all(bio);
-                        return(NULL);   /* Timeout */
-                }
+        /* Allocate memory to a char pointer "Port" */
+        Port = (char *) g_malloc0(sizeof(char) * 4);
+        if (Port == NULL){
+                CRIT("out of memory");
+                g_free(Server);
+                return NULL;
         }
+        /* As Port number is needed separately for some of the below
+         * operations, so copy port number from hostname to "Port".
+         */
+        strncpy(Port, hostname + (len - 3), 3);
+        
+        /* Create socket address structure to prepare client socket */
+        RetVal = getaddrinfo(Server, Port, &Hints, &AddrInfo);
+        if (RetVal != 0) {
+                CRIT("Cannot resolve address [%s] and port [%s],"
+                     " error %d: %s",
+                       Server, Port, RetVal, gai_strerror(RetVal));
+                g_free(Server);
+                g_free(Port);
+                return NULL;
+        }
+        
+        ai = AddrInfo;
+        /* Create a socket point */
+        socket_desc = socket(ai->ai_family, ai->ai_socktype,
+                                            ai->ai_protocol);
+        if (socket_desc == -1) {
+                CRIT("Socket failed with error: %s", 
+                      strerror(errno));
+                g_free(Server);
+                g_free(Port);
+                freeaddrinfo(AddrInfo);	
+                return NULL;
+        }
+
+        /* Now connect to target IP Address */
+        retval = connect(socket_desc, ai->ai_addr, ai->ai_addrlen);
+        if (retval != 0) {
+                CRIT("Socket connect failed with error: %s",
+                      strerror(errno));
+                g_free(Server);
+                g_free(Port);
+                freeaddrinfo(AddrInfo);	
+                close(socket_desc);
+                return NULL;
+        }
+
+        /* Create new SSL structure for connection */
+        ssl = SSL_new(ctx);
+
+        /* Connect ssl object with a socket descriptor */
+        SSL_set_fd(ssl, socket_desc);
+
+        /* Initiate SSL connection */
+        err = SSL_connect(ssl);
+        if (err != 1) {
+                CRIT("SSL connection failed");
+                g_free(Server);
+                g_free(Port);
+                freeaddrinfo(AddrInfo);	
+                close(socket_desc);
+                return (NULL);
+        }
+
+        bio = BIO_new(BIO_f_ssl());             /* create an ssl BIO */
+        BIO_set_ssl(bio, ssl, BIO_CLOSE);       /* assign the ssl BIO to SSL */
 
         /* TODO: Do I need to set the client or server mode here?  I don't
          * think so.
          */
 
+        g_free(Server);
+        g_free(Port);
+        freeaddrinfo(AddrInfo);	
         return(bio);
 }
 
@@ -540,10 +581,10 @@ BIO             *oh_ssl_connect(char *hostname, SSL_CTX *ctx, long timeout)
 int             oh_ssl_disconnect(BIO *bio, enum OH_SSL_SHUTDOWN_TYPE shutdown)
 {
         SSL             *ssl;
-        int             ret;
+        int             ret, fd;
 
         if (bio == NULL) {
-                err("NULL bio in oh_ssl_disconnect()");
+                CRIT("NULL bio in oh_ssl_disconnect()");
                 return(-1);
         }
 
@@ -552,26 +593,33 @@ int             oh_ssl_disconnect(BIO *bio, enum OH_SSL_SHUTDOWN_TYPE shutdown)
          */
         BIO_get_ssl(bio, &ssl);
         if (ssl == NULL) {
-                err("BIO_get_ssl() failed");
+                CRIT("BIO_get_ssl() failed");
                 return(-1);
         }
         ret = SSL_shutdown(ssl);
         if (ret == -1) {
-                err("SSL_shutdown() failed");
+                CRIT("SSL_shutdown() failed");
                 /* Continuing on to free BIO memory */
         }
         else if ((ret == 0) && (shutdown == OH_SSL_BI)) {
                 /* Still need stage 2 shutdown (see SSL_shutdown() man page) */
                 ret = SSL_shutdown(ssl);
                 if (ret == -1) {
-                        err("SSL_shutdown() failed");
+                        CRIT("SSL_shutdown() failed");
                         /* Continuing on to free BIO memory */
                 }
                 else if (ret == 0) {
-                        err("stage 2 of SSL_shutdown() failed");
+                        CRIT("stage 2 of SSL_shutdown() failed");
                         /* Continuing on to free BIO memory */
                 }
         }
+        /* Close the socket */
+        fd = SSL_get_fd(ssl);
+        if (fd == -1) {
+                CRIT("SSL_get_fd() failed");
+                return(-1);
+        }
+        close(fd);
 
         /* Free the connection */
         BIO_free_all(bio);
@@ -612,35 +660,36 @@ int             oh_ssl_read(BIO *bio, char *buf, int size, long timeout)
         int             done;
         int             err;
         int             fd;
+        int             e = 0;
 
         if (bio == NULL) {
-                err("NULL bio in oh_ssl_read()");
+                CRIT("NULL bio in oh_ssl_read()");
                 return(-1);
         }
         if (buf == NULL) {
-                err("NULL buf in oh_ssl_read()");
+                CRIT("NULL buf in oh_ssl_read()");
                 return(-1);
         }
         if (size <= 0) {
-                err("inappropriate size in oh_ssl_read()");
+                CRIT("inappropriate size in oh_ssl_read()");
                 return(-1);
         }
         if (timeout < 0) {
-                err("inappropriate timeout in oh_ssl_read()");
+                CRIT("inappropriate timeout in oh_ssl_read()");
                 return(-1);
         }
 
         /* Get underlying file descriptor, needed for select call */
         fd = BIO_get_fd(bio, NULL);
         if (fd == -1) {
-                err("BIO doesn't seem to be initialized in oh_ssl_read()");
+                CRIT("BIO doesn't seem to be initialized in oh_ssl_read()");
                 return(-1);
         }
 
         /* We also need the SSL connection pointer */
         BIO_get_ssl(bio, &ssl);
         if (ssl == NULL) {
-                err("BIO_get_ssl() failed");
+                CRIT("BIO_get_ssl() failed");
                 return(-1);
         }
 
@@ -679,7 +728,7 @@ int             oh_ssl_read(BIO *bio, char *buf, int size, long timeout)
 
                 /* Evaluate select() return code */
                 if (err < 0) {
-                        err("error during select()");
+                        CRIT("error during select()");
                         return(-1);
                 }
                 if (err == 0) {
@@ -689,6 +738,7 @@ int             oh_ssl_read(BIO *bio, char *buf, int size, long timeout)
                 /* The socket has something.  Ready to try (or re-try)
                  * the read call.
                  */
+                ERR_clear_error();
                 bytes = SSL_read(ssl, buf, size);
                 switch (SSL_get_error(ssl, bytes)) {
                         case SSL_ERROR_NONE:
@@ -710,10 +760,30 @@ int             oh_ssl_read(BIO *bio, char *buf, int size, long timeout)
                         case SSL_ERROR_WANT_WRITE:
                                 read_wait = 0;
                                 break;
+                        case SSL_ERROR_SSL:
+                                e = ERR_get_error();
+                                CRIT("SSL_read reported error %s",
+                                           ERR_error_string(e, NULL));
+                                return(-1);
+                        case SSL_ERROR_SYSCALL:
+                                e = ERR_get_error();
+                                if (bytes == 0 ) {
+                                      CRIT("No bytes read");
+                                } else if ( bytes == -1 ) {
+                                      CRIT("Reading data error %s",
+                                            strerror(errno));
+                                } else {
+                                      CRIT("SSL_read error %s",
+                                                ERR_error_string(e, NULL));
+                                } 
+                                return(-1);
                         default:
                                 /* Some other sort of error */
-                                err("error %d from SSL_read", bytes);
+                                e = ERR_get_error();
+                                CRIT("SSL_read reported error %s",
+                                           ERR_error_string(e, NULL));
                                 return(-1);
+
                 }
         }
 
@@ -753,35 +823,36 @@ int             oh_ssl_write(BIO *bio, char *buf, int size, long timeout)
         int             err;
         int             fd;
         int             sent;
+        int             e = 0;
 
         if (bio == NULL) {
-                err("NULL bio in oh_ssl_write()");
+                CRIT("NULL bio in oh_ssl_write()");
                 return(-1);
         }
         if (buf == NULL) {
-                err("NULL buf in oh_ssl_write()");
+                CRIT("NULL buf in oh_ssl_write()");
                 return(-1);
         }
         if (size <= 0) {
-                err("inappropriate size in oh_ssl_write()");
+                CRIT("inappropriate size in oh_ssl_write()");
                 return(-1);
         }
         if (timeout < 0) {
-                err("inappropriate timeout in oh_ssl_write()");
+                CRIT("inappropriate timeout in oh_ssl_write()");
                 return(-1);
         }
 
         /* Get underlying file descriptor, needed for select call */
         fd = BIO_get_fd(bio, NULL);
         if (fd == -1) {
-                err("BIO doesn't seem to be initialized in oh_ssl_write()");
+                CRIT("BIO doesn't seem to be initialized in oh_ssl_write()");
                 return(-1);
         }
 
         /* We also need the SSL connection pointer */
         BIO_get_ssl(bio, &ssl);
         if (ssl == NULL) {
-                err("BIO_get_ssl() failed");
+                CRIT("BIO_get_ssl() failed");
                 return(-1);
         }
 
@@ -819,7 +890,7 @@ int             oh_ssl_write(BIO *bio, char *buf, int size, long timeout)
 
                 /* Evaluate select() return code */
                 if (err < 0) {
-                        err("error during select()");
+                        CRIT("error during select()");
                         return(-1);
                 }
                 if (err == 0) {
@@ -829,6 +900,7 @@ int             oh_ssl_write(BIO *bio, char *buf, int size, long timeout)
                 /* The socket is ready.  Ready to try (or re-try) the write
                  * call.
                  */
+                ERR_clear_error();
                 bytes = SSL_write(ssl, buf + sent, size - sent);
                 switch (SSL_get_error(ssl, bytes)) {
                         case SSL_ERROR_NONE:
@@ -842,7 +914,7 @@ int             oh_ssl_write(BIO *bio, char *buf, int size, long timeout)
                                 /* Connection was closed.  Since we're trying
                                  * to write, this is an error condition.
                                  */
-                                err("remote host unexpectedly closed"
+                                CRIT("remote host unexpectedly closed"
                                     " the connection");
                                 return(-1);
                         case SSL_ERROR_WANT_READ:
@@ -851,9 +923,28 @@ int             oh_ssl_write(BIO *bio, char *buf, int size, long timeout)
                         case SSL_ERROR_WANT_WRITE:
                                 write_wait = 1;
                                 break;
+                        case SSL_ERROR_SSL:
+                                e = ERR_get_error();
+                                CRIT("SSL_write reported error %s",
+                                           ERR_error_string(e, NULL));
+                                return(-1);
+                        case SSL_ERROR_SYSCALL:
+                                e = ERR_get_error();
+                                if (bytes == 0 ) {
+                                      CRIT("No bytes written");
+                                } else if ( bytes == -1 ) {
+                                      CRIT("Writing data error %s",
+                                            strerror(errno));
+                                } else {
+                                      CRIT("SSL_write error %s",
+                                                ERR_error_string(e, NULL));
+                                } 
+                                return(-1);
                         default:
                                 /* Some other sort of error */
-                                err("error %d from SSL_write", bytes);
+                                e = ERR_get_error();
+                                CRIT("SSL_write reported error %s",
+                                              ERR_error_string(e, NULL));
                                 return(-1);
                 }
         }
