@@ -20,7 +20,6 @@
 #include <string.h>
 
 #include <SaHpi.h>
-#include <openhpi.h>
 #include <oh_utils.h>
 #include <el_utils.h>
 
@@ -50,14 +49,14 @@ int main(int argc, char **argv)
 
         retc = oh_el_info(el, NULL);
         if (retc == SA_OK) {
-                err("ERROR: oh_el_info failed.");
+                CRIT("oh_el_info failed.");
                 return 1;
         }
 
         /* close el without saving to file*/
         retc = oh_el_close(el);
         if (retc != SA_OK) {
-                err("ERROR: oh_el_close on el failed.");
+                CRIT("oh_el_close on el failed.");
                 return 1;
         }
 
